
-- CATALEG
CREATE TABLE GR_ESTATREVISIOEXT (
    ID                  NUMBER(9)       NOT NULL, 
    DESCRIPCIO          VARCHAR2(50)    NOT NULL, 
    CONSTRAINT GR_ESTATREVISIOEXT_PK PRIMARY KEY (ID)
);

-- CATALEG
CREATE TABLE GR_ESTAT (
    ID                  NUMBER(9)       NOT NULL, 
    DESCRIPCIO          VARCHAR2(50)    NOT NULL, 
    CONSTRAINT GR_ESTAT_PK PRIMARY KEY (ID)
);

-- CATALEG
CREATE TABLE GR_ESTATMITIGACIO (
    ID                  NUMBER(9)       NOT NULL, 
    DESCRIPCIO          VARCHAR2(50)    NOT NULL, 
    DBAIXA              DATE,
    ID_ESTATFINAL       NUMBER(9),
    CONSTRAINT GR_ESTATMITIGACIO_FINAL_FK FOREIGN KEY (ID_ESTATFINAL) REFERENCES GR_ESTAT(ID),
    CONSTRAINT GR_ESTATMITIGACIO_PK PRIMARY KEY (ID)
);

-- CATALEG
CREATE TABLE GR_TIPUSCONTROL (
    ID                  NUMBER(9,0) NOT NULL, 
    DESCRIPCIO          VARCHAR2(50) NOT NULL, 
    CONSTRAINT GR_TIPUSCONTROL_PK PRIMARY KEY (ID)
);

-- CATALEG
CREATE TABLE GR_ORIGEN (
    ID                  NUMBER(9,0) NOT NULL, 
    DESCRIPCIO          VARCHAR2(50) NOT NULL, 
    CONSTRAINT GR_ORIGEN_PK PRIMARY KEY (ID)
);

-- CATALEG
CREATE TABLE GR_CRITICITAT (
    ID                  NUMBER(9)       NOT NULL, 
    VALOR               NUMBER(5)       NOT NULL,
    DESCRIPCIO          VARCHAR2(50)    NOT NULL, 
    CONSTRAINT GR_CRITICITAT_PK PRIMARY KEY (ID)
);

-- CATALEG
CREATE TABLE GR_IMPORTANCIA (
    ID                  NUMBER(9)       NOT NULL, 
    VALOR               NUMBER(5)       NOT NULL,
    DESCRIPCIO          VARCHAR2(50)    NOT NULL, 
    CONSTRAINT GR_IMPORTANCIA_PK PRIMARY KEY (ID)
);

CREATE TABLE GR_DEPARTAMENT (
    ID                  NUMBER(9)       NOT NULL, 
    DESCRIPCIO          VARCHAR2(50)    NOT NULL UNIQUE, 
    DBAIXA              DATE,
    CONSTRAINT GR_DEPARTAMENT_PK PRIMARY KEY (ID)
);

CREATE TABLE GR_PERSONA (
    ID                  NUMBER(9)       NOT NULL, 
    NOM                 VARCHAR2(120)   NOT NULL, 
    COGNOM1             VARCHAR2(120)   NOT NULL, 
    COGNOM2             VARCHAR2(120), 
    DOCUMENT            VARCHAR2(50)    NOT NULL UNIQUE, 
    DBAIXA              DATE,
    ID_DEPARTAMENT      NUMBER(9),
    CONSTRAINT GR_PERSONA_PK PRIMARY KEY (ID),
    CONSTRAINT GR_PERSONA_DEPARTAMENT_FK FOREIGN KEY (ID_DEPARTAMENT) REFERENCES GR_DEPARTAMENT(ID) 
);

CREATE TABLE GR_CATEGORIA (
    ID                  NUMBER(9)       NOT NULL, 
    DESCRIPCIO          VARCHAR2(50)    NOT NULL, 
    DCREACIO            DATE            DEFAULT SYSDATE,
    DBAIXA              DATE,
    ID_RESPONSABLE      NUMBER(9),
    CONSTRAINT GR_CATEGORIA_PK PRIMARY KEY (ID),
    CONSTRAINT GR_CATEGORIA_RESPONSABLE_FK FOREIGN KEY (ID_RESPONSABLE) REFERENCES GR_PERSONA(ID) 
);

CREATE TABLE GR_CATEGORIA_IMPORTANCIA (
    ID                  NUMBER(9)       NOT NULL, 
    ID_CATEGORIA        NUMBER(9)       NOT NULL, 
    ID_IMPORTANCIA      NUMBER(9)       NOT NULL, 
    DACTIU              DATE            NOT NULL,
    DDESACTIVAT         DATE,
    CONSTRAINT GR_CATEGORIA_IMPORTANCIA_PK PRIMARY KEY (ID),
    CONSTRAINT GR_CATEGORIA_IMPORTANCIA_CAT_FK FOREIGN KEY (ID_CATEGORIA) REFERENCES GR_CATEGORIA(ID),
    CONSTRAINT GR_CATEGORIA_IMPORTANCIA_IMP_FK FOREIGN KEY (ID_IMPORTANCIA) REFERENCES GR_IMPORTANCIA(ID)
);

CREATE TABLE GR_DEPARTAMENT_CATEGORIA (
    ID                  NUMBER(9)       NOT NULL, 
    ID_DEPARTAMENT      NUMBER(9)       NOT NULL, 
    ID_CATEGORIA        NUMBER(9)       NOT NULL, 
    DCREACIO            DATE            DEFAULT SYSDATE,
    DBAIXA              DATE,
    CONSTRAINT GR_DEPARTAMENT_CATEGORIA_PK PRIMARY KEY (ID),
    CONSTRAINT GR_DEPARTAMENT_CATEGORIA_DEP_FK FOREIGN KEY (ID_DEPARTAMENT) REFERENCES GR_DEPARTAMENT(ID),
    CONSTRAINT GR_DEPARTAMENT_CATEGORIA_CAT_FK FOREIGN KEY (ID_CATEGORIA) REFERENCES GR_CATEGORIA(ID) 
);

CREATE TABLE GR_CONTROLINTERN (
    ID                  NUMBER(9)       NOT NULL, 
    DESCRIPCIO          VARCHAR2(120)   NOT NULL, 
    DCREACIO            DATE            DEFAULT SYSDATE,
    DBAIXA              DATE,
    -- CONTROLNUM          NUMBER(3)       NOT NULL,
    CONSTRAINT GR_CONTROLINTERN_PK PRIMARY KEY (ID)
);

CREATE TABLE GR_CONTROLINTERN_DPTOS_IMPLICATS (
    ID_CONTROLINTERN    NUMBER(9)       NOT NULL, 
    ID_DEPARTAMENT_CATEGORIA    NUMBER(9)       NOT NULL, 
    CONSTRAINT GR_CONTROLINTERN_DPTOS_IMPLICATS_PK PRIMARY KEY (ID_CONTROLINTERN, ID_DEPARTAMENT_CATEGORIA),
    CONSTRAINT GR_CONTROLINTERN_DPTOS_IMPLICATS_CI_FK FOREIGN KEY (ID_CONTROLINTERN) REFERENCES GR_CONTROLINTERN(ID),
    CONSTRAINT GR_CONTROLINTERN_DPTOS_IMPLICATS_DI_FK FOREIGN KEY (ID_DEPARTAMENT_CATEGORIA) REFERENCES GR_DEPARTAMENT_CATEGORIA(ID)
);

CREATE TABLE GR_CONTROLINTERN_EQUIP (
    ID_CONTROLINTERN    NUMBER(9)       NOT NULL, 
    ID_PERSONA          NUMBER(9)       NOT NULL, 
    DALTA               DATE            NOT NULL,
    DBAIXA              DATE,
    CONSTRAINT GR_CONTROLINTERN_EQUIP_PK PRIMARY KEY (ID_CONTROLINTERN, ID_PERSONA),
    CONSTRAINT GR_CONTROLINTERN_EQUIP_CI_FK  FOREIGN KEY (ID_CONTROLINTERN) REFERENCES GR_CONTROLINTERN(ID),
    CONSTRAINT GR_CONTROLINTERN_EQUIP_PERSONA_FK  FOREIGN KEY (ID_PERSONA) REFERENCES GR_PERSONA(ID)
);

CREATE TABLE GR_CONTROL (
    ID                  NUMBER(9)       NOT NULL, 
    DESCRIPCIO          VARCHAR2(50)    NOT NULL, 
    DCREACIO            DATE            DEFAULT SYSDATE,
    DBAIXA              DATE,
    ID_CONTROLINTERN    NUMBER(9)       NOT NULL,
    ID_DEPARTAMENTREVISIO  NUMBER(9),
    ID_DEPARTAMENTEXECUCIO NUMBER(9),
    ID_TIPUSCONTROL     NUMBER(9),
    CONSTRAINT GR_CONTROL_PK PRIMARY KEY (ID),
    CONSTRAINT GR_CONTROL_CONTROLINTERN_FK FOREIGN KEY (ID_CONTROLINTERN) REFERENCES GR_CONTROLINTERN(ID),
    CONSTRAINT GR_CONTROL_DPTOREVISIO_FK   FOREIGN KEY (ID_DEPARTAMENTREVISIO) REFERENCES GR_DEPARTAMENT(ID),
    CONSTRAINT GR_CONTROL_DPTOEXECUCIO_FK  FOREIGN KEY (ID_DEPARTAMENTEXECUCIO) REFERENCES GR_DEPARTAMENT(ID),
    CONSTRAINT GR_CONTROL_TIPUSCONTROL_FK  FOREIGN KEY (ID_TIPUSCONTROL) REFERENCES GR_TIPUSCONTROL(ID)
);

CREATE TABLE GR_CAMPANYA (
    ID                  NUMBER(9)       NOT NULL, 
    DINICI              DATE            NOT NULL,
    DFI                 DATE            NOT NULL,
    DESCRIPCIO          VARCHAR2(120)    NOT NULL, 
    PERSONA_LIDER_ID    NUMBER(9),
    CONSTRAINT GR_CAMPANYA_PK PRIMARY KEY (ID),
    CONSTRAINT GR_CAMPANYA_LIDER_FK  FOREIGN KEY (PERSONA_LIDER_ID) REFERENCES GR_PERSONA(ID)
);

CREATE TABLE GR_CAMPANYA_CONTROL (
    ID                  NUMBER(9)       NOT NULL, 
    ID_CAMPANYA         NUMBER(9)       NOT NULL, 
    ID_CONTROL          NUMBER(9)       NOT NULL, 
    CONSTRAINT GR_CAMPANYA_CONTROL_PK PRIMARY KEY (ID),
    CONSTRAINT GR_CAMPANYA_CONTROL_CAMPANYA_FK  FOREIGN KEY (ID_CAMPANYA) REFERENCES GR_CAMPANYA(ID),
    CONSTRAINT GR_CAMPANYA_CONTROL_CONTROL_FK  FOREIGN KEY (ID_CONTROL) REFERENCES GR_CONTROL(ID)
);

CREATE TABLE GR_AUDITINTERNA (
    ID                  NUMBER(9)       NOT NULL, 
    DESCRIPCIO          VARCHAR2(120)   NOT NULL, 
    DBAIXA              DATE,
    CONSTRAINT GR_AUDITINTERNA_PK PRIMARY KEY (ID)
);

CREATE TABLE GR_AUDITINTERNA_EQUIP (
    ID_AUDITINTERNA     NUMBER(9)       NOT NULL, 
    ID_PERSONA          NUMBER(9)       NOT NULL, 
    DALTA               DATE            NOT NULL,
    DBAIXA              DATE,
    CONSTRAINT GR_AUDITINTERNA_EQUIP_PK PRIMARY KEY (ID_AUDITINTERNA, ID_PERSONA),
    CONSTRAINT GR_AUDITINTERNA_EQUIP_AI_FK  FOREIGN KEY (ID_AUDITINTERNA) REFERENCES GR_AUDITINTERNA(ID),
    CONSTRAINT GR_AUDITINTERNA_EQUIP_PERSONA_FK  FOREIGN KEY (ID_PERSONA) REFERENCES GR_PERSONA(ID)
);

CREATE TABLE GR_REVISIO (
    ID                  NUMBER(9)       NOT NULL, 
    DINICI              DATE            NOT NULL,
    DFI                 DATE            NOT NULL,
    OBJECTIU            VARCHAR2(200)   NOT NULL,
    DAVIS               DATE,
    ID_DEPARTAMENTIMPLICAT NUMBER(9)    NOT NULL,
    ID_AUDITINTERNA     NUMBER(9)       NOT NULL,
    DBAIXA              DATE,
    CONSTRAINT GR_REVISIO_PK PRIMARY KEY (ID),
    CONSTRAINT GR_REVISIO_DPTOIMPLICAT_FK  FOREIGN KEY (ID_DEPARTAMENTIMPLICAT) REFERENCES GR_DEPARTAMENT(ID),
    CONSTRAINT GR_REVISIO_AUDITINTERNA_FK  FOREIGN KEY (ID_AUDITINTERNA) REFERENCES GR_AUDITINTERNA(ID)
);

CREATE TABLE GR_REVISIO_PERSONA (
    ID_REVISIO          NUMBER(9)       NOT NULL, 
    ID_PERSONA          NUMBER(9)       NOT NULL, 
    DALTA               DATE            NOT NULL,
    DBAIXA              DATE,
    CONSTRAINT GR_REVISIO_PERSONA_PK PRIMARY KEY (ID_REVISIO, ID_PERSONA),
    CONSTRAINT GR_REVISIO_PERSONA_REV_FK  FOREIGN KEY (ID_REVISIO) REFERENCES GR_REVISIO(ID),
    CONSTRAINT GR_REVISIO_PERSONA_PER_FK  FOREIGN KEY (ID_PERSONA) REFERENCES GR_PERSONA(ID)
);

CREATE TABLE GR_ACCIO (
    ID                  NUMBER(9)       NOT NULL, 
    DESCRIPCIO          VARCHAR2(400)   NOT NULL, 
    DCREACIO            DATE            NOT NULL,
    DTANCAMENT          DATE,
    DBAIXA              DATE,
    NOM                 VARCHAR2(120)   NOT NULL,
    ID_ESTATMITIGACIO   NUMBER(9)       NOT NULL,
    ID_PERSONARESPONSABLE NUMBER(9),
    CONSTRAINT GR_ACCIO_PK PRIMARY KEY (ID),
    CONSTRAINT GR_ACCIO_ESTATMITIGACIO_FK  FOREIGN KEY (ID_ESTATMITIGACIO) REFERENCES GR_ESTATMITIGACIO(ID),
    CONSTRAINT GR_ACCIO_RESPONSABLE_FK  FOREIGN KEY (ID_PERSONARESPONSABLE) REFERENCES GR_PERSONA(ID)
);

CREATE TABLE GR_AUDITEXTERNA (
    ID                  NUMBER(9)       NOT NULL, 
    NOMEMPRESA          VARCHAR2(200)   NOT NULL,
    DBAIXA              DATE            ,
    CONSTRAINT GR_AUDITEXTERNA_PK PRIMARY KEY (ID)
);

CREATE TABLE GR_PERSONAEXTERNA (
    ID                  NUMBER(9)       NOT NULL, 
    NOM                 VARCHAR2(120)   NOT NULL, 
    COGNOM1             VARCHAR2(120)   NOT NULL, 
    COGNOM2             VARCHAR2(120)   , 
    DOCUMENT            VARCHAR2(50)    NOT NULL,
    DBAIXA              DATE,
    ID_AUDITEXTERNA     NUMBER(9),
    CONSTRAINT GR_PERSONAEXTERNA_PK PRIMARY KEY (ID),
    CONSTRAINT GR_PERSONAEXTERNA_AE_FK FOREIGN KEY (ID_AUDITEXTERNA) REFERENCES GR_AUDITEXTERNA(ID) 
);

CREATE TABLE GR_REVISIOEXTERNA (
    ID                  NUMBER(9)       NOT NULL, 
    DINICI              DATE,
    DFI                 DATE,
    COST                NUMBER(12,2),
    ID_AUDITEXTERNA     NUMBER(9)       NOT NULL,
    CONSTRAINT GR_REVISIOEXTERNA_PK PRIMARY KEY (ID),
    CONSTRAINT GR_REVISIOEXTERNA_AE_FK FOREIGN KEY (ID_AUDITEXTERNA) REFERENCES GR_AUDITEXTERNA(ID) 
);

ALTER TABLE GR_REVISIO 
    ADD (ID_REVISIOEXTERNA NUMBER(9));
ALTER TABLE GR_REVISIO
    ADD CONSTRAINT GR_REVISIO_REXT_FK FOREIGN KEY (ID_REVISIOEXTERNA) REFERENCES GR_REVISIOEXTERNA(ID);

CREATE TABLE GR_REVISIOEXTERNA_PERSONAEXTERNA (
    ID_REVISIOEXTERNA   NUMBER(9)       NOT NULL, 
    ID_PERSONEXTERNA    NUMBER(9)       NOT NULL, 
    CONSTRAINT GR_REVISIOEXTERNA_PERSONAEXTERNA_PK PRIMARY KEY (ID_REVISIOEXTERNA, ID_PERSONEXTERNA),
    CONSTRAINT GR_REVISIOEXTERNA_PERSONAEXTERNA_REV_FK FOREIGN KEY (ID_REVISIOEXTERNA) REFERENCES GR_REVISIOEXTERNA(ID),
    CONSTRAINT GR_REVISIOEXTERNA_PERSONAEXTERNA_PER_FK FOREIGN KEY (ID_PERSONEXTERNA) REFERENCES GR_PERSONAEXTERNA(ID)
);

CREATE TABLE GR_RESULTAT (
    ID                  NUMBER(9)       NOT NULL, 
    DOCUMENT            BLOB,
    ID_ESTATREVISIOEXTERNA NUMBER(9),
    CONSTRAINT GR_RESULTAT_PK PRIMARY KEY (ID),
    CONSTRAINT GR_RESULTAT_ESTAT_FK FOREIGN KEY (ID_ESTATREVISIOEXTERNA) REFERENCES GR_ESTATREVISIOEXT(ID) 
);

CREATE TABLE GR_RISC (
    ID                  NUMBER(9)       NOT NULL, 
    DCREACIO            DATE            DEFAULT SYSDATE NOT NULL,
    DFI                 DATE,
    DESCRIPCIO          VARCHAR2(200)   NOT NULL,
    IMPACTE             NUMBER(5),
    ID_CATEGORIA        NUMBER(9),
    ID_ORIGEN           NUMBER(9),
    ID_CRITICITAT       NUMBER(9),
    ID_ESTAT            NUMBER(9),
    ID_REVISIOEXTERNA   NUMBER(9),
    ID_ACCIO            NUMBER(9),
    ID_REVISIO          NUMBER(9),
    ID_CONTROL          NUMBER(9),
    ID_PERSONARESPONSABLE NUMBER(9),
    CONSTRAINT GR_RISC_PK PRIMARY KEY (ID),
    CONSTRAINT GR_RISC_CATEGORIA_FK FOREIGN KEY (ID_CATEGORIA) REFERENCES GR_CATEGORIA(ID),
    CONSTRAINT GR_RISC_ORIGEN_FK FOREIGN KEY (ID_ORIGEN) REFERENCES GR_ORIGEN(ID),
    CONSTRAINT GR_RISC_CRITICITAT_FK FOREIGN KEY (ID_CRITICITAT) REFERENCES GR_CRITICITAT(ID),
    CONSTRAINT GR_RISC_ESTAT_FK FOREIGN KEY (ID_ESTAT) REFERENCES GR_ESTAT(ID),
    CONSTRAINT GR_RISC_REVISIOEXTERNA_FK FOREIGN KEY (ID_REVISIOEXTERNA) REFERENCES GR_REVISIOEXTERNA(ID),
    CONSTRAINT GR_RISC_ACCIO_FK FOREIGN KEY (ID_ACCIO) REFERENCES GR_ACCIO(ID),
    CONSTRAINT GR_RISC_REVISIO_FK FOREIGN KEY (ID_REVISIO) REFERENCES GR_REVISIO(ID),
    CONSTRAINT GR_RISC_CONTROL_FK FOREIGN KEY (ID_CONTROL) REFERENCES GR_CONTROL(ID),
    CONSTRAINT GR_RISC_RESPONSABLE_FK FOREIGN KEY (ID_PERSONARESPONSABLE) REFERENCES GR_PERSONA(ID)
);

ALTER TABLE GR_CAMPANYA_CONTROL 
    ADD (ID_RISC NUMBER(9));
ALTER TABLE GR_CAMPANYA_CONTROL
    ADD CONSTRAINT GR_CAMPANYA_CONTROL_RISC_FK FOREIGN KEY (ID_RISC) REFERENCES GR_RISC(ID);

ALTER TABLE GR_ACCIO 
    ADD (ID_RISC NUMBER(9) NOT NULL);
ALTER TABLE GR_ACCIO
    ADD CONSTRAINT GR_ACCIO_RISC_FK FOREIGN KEY (ID_RISC) REFERENCES GR_RISC(ID);


-- *********************
-- TAULES MOSTREIG INICI

CREATE TABLE GR_MOSTREIG (
    ID                  NUMBER(9)       NOT NULL, 
    DMOSTREIG           DATE            DEFAULT SYSDATE NOT NULL, 
    CONSTRAINT GR_MOSTREIG_PK PRIMARY KEY (ID)
);

CREATE TABLE GR_RISC_MOSTREIG AS (SELECT * FROM GR_RISC WHERE 1=2);
ALTER TABLE  GR_RISC_MOSTREIG RENAME COLUMN ID TO ID_RISC;
ALTER TABLE  GR_RISC_MOSTREIG ADD (ID_MOSTREIG NUMBER(9));
ALTER TABLE  GR_RISC_MOSTREIG ADD CONSTRAINT GR_RISC_MOSTREIG_PK PRIMARY KEY (ID_MOSTREIG, ID_RISC);
ALTER TABLE  GR_RISC_MOSTREIG ADD CONSTRAINT GR_RISC_MOSTREIG_FK FOREIGN KEY (ID_MOSTREIG) REFERENCES GR_MOSTREIG(ID);
ALTER TABLE  GR_RISC_MOSTREIG ADD CONSTRAINT GR_RISC_MOSTREIG_RISC_FK FOREIGN KEY (ID_RISC) REFERENCES GR_RISC(ID);

CREATE TABLE GR_CONTROL_MOSTREIG AS (SELECT * FROM GR_CONTROL WHERE 1=2);
ALTER TABLE  GR_CONTROL_MOSTREIG RENAME COLUMN ID TO ID_CONTROL;
ALTER TABLE  GR_CONTROL_MOSTREIG ADD (ID_MOSTREIG NUMBER(9));
ALTER TABLE  GR_CONTROL_MOSTREIG ADD CONSTRAINT GR_CONTROL_MOSTREIG_PK PRIMARY KEY (ID_MOSTREIG, ID_CONTROL);
ALTER TABLE  GR_CONTROL_MOSTREIG ADD CONSTRAINT GR_CONTROL_MOSTREIG_FK FOREIGN KEY (ID_MOSTREIG) REFERENCES GR_MOSTREIG(ID);
ALTER TABLE  GR_CONTROL_MOSTREIG ADD CONSTRAINT GR_CONTROL_MOSTREIG_CONTROL_FK FOREIGN KEY (ID_CONTROL) REFERENCES GR_CONTROL(ID);

CREATE TABLE GR_REVISIO_MOSTREIG AS (SELECT * FROM GR_REVISIO WHERE 1=2);
ALTER TABLE  GR_REVISIO_MOSTREIG RENAME COLUMN ID TO ID_REVISIO;
ALTER TABLE  GR_REVISIO_MOSTREIG ADD (ID_MOSTREIG NUMBER(9));
ALTER TABLE  GR_REVISIO_MOSTREIG ADD CONSTRAINT GR_REVISIO_MOSTREIG_PK PRIMARY KEY (ID_MOSTREIG, ID_REVISIO);
ALTER TABLE  GR_REVISIO_MOSTREIG ADD CONSTRAINT GR_REVISIO_MOSTREIG_FK FOREIGN KEY (ID_MOSTREIG) REFERENCES GR_MOSTREIG(ID);
ALTER TABLE  GR_REVISIO_MOSTREIG ADD CONSTRAINT GR_REVISIO_MOSTREIG_REV_FK FOREIGN KEY (ID_REVISIO) REFERENCES GR_REVISIO(ID);

CREATE TABLE GR_REVISIOEXTERNA_MOSTREIG AS (SELECT * FROM GR_REVISIOEXTERNA WHERE 1=2);
ALTER TABLE  GR_REVISIOEXTERNA_MOSTREIG RENAME COLUMN ID TO ID_REVISIOEXTERNA;
ALTER TABLE  GR_REVISIOEXTERNA_MOSTREIG ADD (ID_MOSTREIG NUMBER(9));
ALTER TABLE  GR_REVISIOEXTERNA_MOSTREIG ADD CONSTRAINT GR_REVISIOEXTERNA_MOSTREIG_PK PRIMARY KEY (ID_MOSTREIG, ID_REVISIOEXTERNA);
ALTER TABLE  GR_REVISIOEXTERNA_MOSTREIG ADD CONSTRAINT GR_REVISIOEXTERNA_MOSTREIG_FK FOREIGN KEY (ID_MOSTREIG) REFERENCES GR_MOSTREIG(ID);
ALTER TABLE  GR_REVISIOEXTERNA_MOSTREIG ADD CONSTRAINT GR_REVISIOEXTERNA_MOSTREIG_REVEXT_FK FOREIGN KEY (ID_REVISIOEXTERNA) REFERENCES GR_REVISIOEXTERNA(ID);

CREATE TABLE GR_ACCIO_MOSTREIG AS (SELECT * FROM GR_ACCIO WHERE 1=2);
ALTER TABLE  GR_ACCIO_MOSTREIG RENAME COLUMN ID TO ID_ACCIO;
ALTER TABLE  GR_ACCIO_MOSTREIG ADD (ID_MOSTREIG NUMBER(9));
ALTER TABLE  GR_ACCIO_MOSTREIG ADD CONSTRAINT GR_ACCIO_MOSTREIG_PK PRIMARY KEY (ID_MOSTREIG, ID_ACCIO);
ALTER TABLE  GR_ACCIO_MOSTREIG ADD CONSTRAINT GR_ACCIO_MOSTREIG_FK FOREIGN KEY (ID_MOSTREIG) REFERENCES GR_MOSTREIG(ID);
ALTER TABLE  GR_ACCIO_MOSTREIG ADD CONSTRAINT GR_ACCIO_MOSTREIG_ACCIO_FK FOREIGN KEY (ID_ACCIO) REFERENCES GR_ACCIO(ID);

-- TAULES MOSTREIG FINAL
-- *********************

