-- ******************
-- LOGS

-- taula per emmagatzemar els logs de l'aplicació
CREATE TABLE GR_LOG (
    ID                  NUMBER(12)      NOT NULL, 
    EXECUCIO            DATE            DEFAULT SYSDATE NOT NULL,
    PROCEDIMENT         VARCHAR2(120)   NOT NULL, 
    PAR_INPUT           VARCHAR2(4000), 
    PAR_OUTPUT          VARCHAR2(4000),
    RSP                 VARCHAR2(40)    ,
    RSPDESC             VARCHAR2(400)   ,
    CONSTRAINT GR_LOG_PK PRIMARY KEY (ID)
);
CREATE INDEX IDX_LOG ON GR_LOG (EXECUCIO ASC);

-- sequencia per la taula de logs
CREATE SEQUENCE SQ_LOG
  INCREMENT BY 1 
  START WITH 1 
  MAXVALUE 999999999999 
  MINVALUE 1 
  CACHE 20 
  ORDER;


-- cataleg dels errors controlats del sistema
CREATE TABLE GR_ERRORS_CATALEG (
    ERRCODE             VARCHAR2(12)    NOT NULL, 
    ERRDESC             VARCHAR2(4000)  NOT NULL UNIQUE,
    CONSTRAINT GR_ERRORS_CAT_PK PRIMARY KEY (ERRCODE)
);

-- procediment per emmagatzemar els logs
-- sera cridat per tots els procediments que creen dades
CREATE OR REPLACE PROCEDURE PRC_LOG (
    NOMPRC  IN  VARCHAR2,
    PIN     IN  VARCHAR2 DEFAULT NULL,
    POUT    IN  VARCHAR2 DEFAULT NULL,
    RSP     IN  VARCHAR2 DEFAULT NULL,
    RSPDESC IN  VARCHAR2 DEFAULT NULL
) IS 
BEGIN

    INSERT INTO GR_LOG (ID, PROCEDIMENT, PAR_INPUT, PAR_OUTPUT, RSP, RSPDESC)
    VALUES
    (SQ_LOG.NEXTVAL, NOMPRC, PIN, POUT, RSP, RSPDESC);
  
END PRC_LOG;
/

-- funció per obtenir la descripció d'un error.
-- sera cridat per tots els procediments que creen dades, per obtenir l'error controlat
CREATE OR REPLACE FUNCTION FNT_GET_ERRDESC (PI_ERRCOD IN VARCHAR2) RETURN VARCHAR2 IS
  VRET GR_ERRORS_CATALEG.ERRDESC%TYPE := NULL;
BEGIN
  
  SELECT ERRDESC
  INTO VRET
  FROM GR_ERRORS_CATALEG
  WHERE ERRCODE = PI_ERRCOD;
  
  RETURN VRET;

EXCEPTION
  WHEN OTHERS THEN 
    RETURN VRET;
  
END FNT_GET_ERRDESC;
/
