/* trigger per calcular l'impacte d'un risc quan es crea */

CREATE OR REPLACE TRIGGER TRG_GR_RISC_BEFORE_INSERT_2
BEFORE INSERT ON GR_RISC FOR EACH ROW

DECLARE
    v_criticitat number(3) := 0;
    v_importancia number(3) := 0;
BEGIN
  IF :new.id_criticitat is not null THEN
    -- segur que existeix, hi ha una FK
    select valor 
    into v_criticitat
    from GR_CRITICITAT
    WHERE ID = :new.id_criticitat;
  END IF;
  
  IF :new.id_categoria is not null THEN
    -- unicament existeix un registre actiu a gr_categoria_importancia
    -- tot i aixi, es controla la excepció per si no esta creada la relació
    begin
        select GR_IMPORTANCIA.valor
        into v_importancia
        from GR_CATEGORIA_IMPORTANCIA, GR_IMPORTANCIA
        where GR_IMPORTANCIA.id = gr_categoria_importancia.id_importancia
        and GR_CATEGORIA_IMPORTANCIA.id_categoria = :new.id_categoria
        and GR_CATEGORIA_IMPORTANCIA.ddesactivat is null;    
    exception
      when others then
        null;
    end;
  
  END IF;
  
  :new.impacte := v_importancia * v_criticitat;

END;
/



/* trigger per calcular l'impacte d'un risc quan es modifica */
CREATE OR REPLACE TRIGGER TRG_GR_RISC_BEFORE_UPDATE_2
BEFORE UPDATE ON GR_RISC FOR EACH ROW

DECLARE
    v_criticitat number(3) := 0;
    v_importancia number(3) := 0;
BEGIN
  IF :new.id_criticitat is not null THEN
    -- segur que existeix, hi ha una FK
    select valor 
    into v_criticitat
    from GR_CRITICITAT
    WHERE ID = :new.id_criticitat;
  END IF;
  
  IF :new.id_categoria is not null THEN
    -- unicament existeix un registre actiu a gr_categoria_importancia
    -- tot i aixi, es controla la excepció per si no esta creada la relació
    begin
        select GR_IMPORTANCIA.valor
        into v_importancia
        from GR_CATEGORIA_IMPORTANCIA, GR_IMPORTANCIA
        where GR_IMPORTANCIA.id = gr_categoria_importancia.id_importancia
        and GR_CATEGORIA_IMPORTANCIA.id_categoria = :new.id_categoria
        and GR_CATEGORIA_IMPORTANCIA.ddesactivat is null;    
    exception
      when others then
        null;
    end;
  
  END IF;
  
  :new.impacte := v_importancia * v_criticitat;


END;
/

-- a partir de la 11g
-- triggers compuestos.
/* calcul de l'estat d'un risc, a partir de l'estat de totes les seves accions assignades 
*/
CREATE OR REPLACE TRIGGER TRG_GR_ACCIO_BEFORE_UPDATE_2
FOR UPDATE ON GR_ACCIO COMPOUND TRIGGER

    V_ESTAT_MINIM NUMBER(9) DEFAULT NULL;
    v_id_risc number;
    v_data_tancat date;

    -- s'executa despres de cada fila actualitzada
    AFTER EACH ROW IS
        BEGIN
            v_id_risc := :new.id_risc;
            v_data_tancat := :new.dtancament;
    END AFTER EACH ROW;

    -- s'executa despres de cada sentencia actualitzada
    AFTER STATEMENT IS
        BEGIN
          SELECT MIN(ID_ESTATFINAL)
          INTO V_ESTAT_MINIM
          FROM GR_ACCIO, GR_ESTATMITIGACIO
          WHERE GR_ACCIO.ID_RISC = v_id_risc
          AND GR_ESTATMITIGACIO.ID = GR_ACCIO.ID_ESTATMITIGACIO
          AND GR_ESTATMITIGACIO.DBAIXA IS NULL
          AND GR_ESTATMITIGACIO.ID_ESTATFINAL IS NOT NULL;
          
          IF V_ESTAT_MINIM IS NOT NULL THEN
            UPDATE GR_RISC
            SET ID_ESTAT = V_ESTAT_MINIM
            WHERE ID = v_id_risc;
    
            IF V_ESTAT_MINIM IN (2,3) THEN
              UPDATE GR_RISC
              SET DFI = v_data_tancat
              WHERE ID = v_id_risc;
            END IF;
          END IF;
          
    END AFTER STATEMENT;
    
END TRG_GR_ACCIO_BEFORE_UPDATE_2;
/



/* ********************************* */
/* CREAR ELS TRIGGERS INSERT/UPDATE/DELETE DE TOTES LES TAULES 
   CREAR CONTROL: UNICAMENT L'USUARI uriscdades POT MODIFICAR DADES
*/

declare
  cursor c_crea_trg is
      select a.constraint_name, b.column_name, a.table_name
      from user_constraints a, user_cons_columns b
      where a.constraint_type = 'P' 
      and a.constraint_name = b.constraint_name
      and a.table_name not in ('GR_LOG'); -- la taula GR_LOG no te control

begin
  for reg_trg in c_crea_trg loop 
    -- TRG INSERT
    execute immediate (
    --DBMS_OUTPUT.PUT_LINE(
'CREATE OR REPLACE TRIGGER TRG_' || reg_trg.table_name || '_INSERT ' || CHR(10) || 
'  BEFORE INSERT ON ' || reg_trg.table_name || ' FOR EACH ROW ' || CHR(10) || 
'DECLARE ' || CHR(10) || 
'  V_USER VARCHAR2(200); ' || CHR(10) || 
'BEGIN ' || CHR(10) || 
'  SELECT USER INTO V_USER FROM DUAL; ' || CHR(10) || 
'  IF NOT V_USER = ''URISCDADES'' THEN ' || CHR(10) || 
'    raise_application_error(-20001,''Usuari sense permis''); ' || CHR(10) || 
'  END IF; ' || CHR(10) || 
'END;'
                      );
                      
    -- TRG UPDATE
    execute immediate (
    --DBMS_OUTPUT.PUT_LINE(
'CREATE OR REPLACE TRIGGER TRG_' || reg_trg.table_name || '_UPDATE ' || CHR(10) || 
'  BEFORE UPDATE ON ' || reg_trg.table_name || ' FOR EACH ROW ' || CHR(10) || 
'DECLARE ' || CHR(10) || 
'  V_USER VARCHAR2(200); ' || CHR(10) || 
'BEGIN ' || CHR(10) || 
'  SELECT USER INTO V_USER FROM DUAL; ' || CHR(10) || 
'  IF NOT V_USER = ''URISCDADES'' THEN ' || CHR(10) || 
'    raise_application_error(-20001,''Usuari sense permis''); ' || CHR(10) || 
'  END IF; ' || CHR(10) || 
'END;'
                      );

    -- TRG DELETE
    execute immediate (
    --DBMS_OUTPUT.PUT_LINE(
'CREATE OR REPLACE TRIGGER TRG_' || reg_trg.table_name || '_DELETE ' || CHR(10) || 
'  BEFORE DELETE ON ' || reg_trg.table_name || ' FOR EACH ROW ' || CHR(10) || 
'DECLARE ' || CHR(10) || 
'  V_USER VARCHAR2(200); ' || CHR(10) || 
'BEGIN ' || CHR(10) || 
'  SELECT USER INTO V_USER FROM DUAL; ' || CHR(10) || 
'  IF NOT V_USER = ''URISCDADES'' THEN ' || CHR(10) || 
'    raise_application_error(-20001,''Usuari sense permis''); ' || CHR(10) || 
'  END IF; ' || CHR(10) || 
'END;'
                      );

                      
  end loop;
end;
/
