
/* ****************************************************** */
/* ******************** PACKAGE BODY ******************** */
CREATE OR REPLACE PACKAGE BODY PKG_GR_DADES IS

  /* ******************
     Alta/Baixa/Modificació de dades taula GR_DEPARTAMENT 
     IN_ABM: A(lta), B(aixa), M(odificació)
     IN_COD: Codi del registre, si nou registre = Null
     IN_DESC: Descripció del registre
     RSP: Codi de sortida del procediment
  ****************** */
  PROCEDURE P_DEPARTAMENT (IN_ABM VARCHAR2, 
                           IN_COD NUMBER,
                           IN_DESC VARCHAR2,
                           RSP OUT VARCHAR2) IS
                           
    VAUX DATE;
    VAUX_COUNT VARCHAR2(2000);
    
    VL_ERRSQLCOD NUMBER;
    VL_ERRSQLMSG VARCHAR2(255);
    
    ERR_CTRL_PARAM EXCEPTION; 
    ERR_CTRL_PARAM_NO_AUTORITZAT EXCEPTION;
    ERR_CTRL_ACCIO_NO_AUTORITZAT EXCEPTION;
    
    VL_ERRCODE GR_LOG.RSP%TYPE;
    VL_ERRDESC GR_LOG.RSPDESC%TYPE;
    
  BEGIN
   
    IF IN_ABM NOT IN ('A','B','M') THEN
        RAISE ERR_CTRL_PARAM;
    ELSIF IN_ABM IN ('B') AND IN_COD IS NULL THEN
        RAISE ERR_CTRL_PARAM;
    ELSIF IN_ABM IN ('M') THEN
        RAISE ERR_CTRL_PARAM_NO_AUTORITZAT;
    END IF;
  
    -- Alta de registre
    IF IN_ABM = 'A' THEN
    
        -- Alta d'un registre nou
        IF IN_COD IS NULL THEN
            dbms_output.put_line('1');
            SELECT count(1)
            INTO VAUX_COUNT
            FROM GR_DEPARTAMENT
            WHERE UPPER(DESCRIPCIO) = UPPER(IN_DESC);
            
            IF VAUX_COUNT = 0 THEN
                INSERT INTO GR_DEPARTAMENT (ID, DESCRIPCIO, DBAIXA) VALUES 
                (SQ_DEPARTAMENT.NEXTVAL, IN_DESC, NULL);
            ELSE 
                -- no autortizat, la descripcio ja existeix
                RAISE ERR_CTRL_ACCIO_NO_AUTORITZAT;
            END IF;
        
        -- Activació d'un registre previament desactivat
        ELSE 
            BEGIN
                SELECT DBAIXA
                INTO VAUX
                FROM GR_DEPARTAMENT
                WHERE ID = IN_COD
                AND DBAIXA IS NOT NULL
                FOR UPDATE;
                UPDATE GR_DEPARTAMENT SET DBAIXA = NULL WHERE ID = IN_COD;
            EXCEPTION
                WHEN NO_DATA_FOUND THEN
                    -- el registre ja esta donat d'alta o no existeix
                    RAISE ERR_CTRL_ACCIO_NO_AUTORITZAT;
            END;
        END IF;

    
    -- baixa de registre, bloqueig de registre per modificar
    ELSIF IN_ABM = 'B' THEN
        BEGIN
            SELECT DBAIXA
            INTO VAUX
            FROM GR_DEPARTAMENT
            WHERE ID = IN_COD
            AND DBAIXA IS NULL
            FOR UPDATE;
            UPDATE GR_DEPARTAMENT SET DBAIXA = SYSDATE WHERE ID = IN_COD;
        EXCEPTION 
            WHEN OTHERS THEN
                -- el registre no esta donat d'alta o no existeix
                RAISE ERR_CTRL_ACCIO_NO_AUTORITZAT;
        END;
        
    END IF;   


    VL_ERRCODE := 'OK';
    VL_ERRDESC := NULL;
    RSP := VL_ERRCODE;
  
    
    PRC_LOG(NOMPRC => 'P_DEPARTAMENT',
            PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_COD=' || IN_COD || ', ' || 'IN_DESC=' || IN_DESC,
            RSP    => VL_ERRCODE,
            RSPDESC=> VL_ERRDESC);

  EXCEPTION
    WHEN ERR_CTRL_PARAM THEN
      VL_ERRCODE := 'ERR_001';
      VL_ERRDESC := FNT_GET_ERRDESC(VL_ERRCODE);
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_DEPARTAMENT',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_COD=' || IN_COD || ', ' || 'IN_DESC=' || IN_DESC,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);

    WHEN ERR_CTRL_PARAM_NO_AUTORITZAT THEN
      VL_ERRCODE := 'ERR_002';
      VL_ERRDESC := FNT_GET_ERRDESC(VL_ERRCODE);
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_DEPARTAMENT',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_COD=' || IN_COD || ', ' || 'IN_DESC=' || IN_DESC,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);
    
    WHEN ERR_CTRL_ACCIO_NO_AUTORITZAT THEN
      VL_ERRCODE := 'ERR_003';
      VL_ERRDESC := FNT_GET_ERRDESC(VL_ERRCODE);
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_DEPARTAMENT',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_COD=' || IN_COD || ', ' || 'IN_DESC=' || IN_DESC,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);
    
    
    WHEN OTHERS THEN
      VL_ERRSQLCOD := SQLCODE;
      VL_ERRSQLMSG := SQLERRM;
      VL_ERRCODE := 'ERR_999';
      VL_ERRDESC := VL_ERRSQLCOD || ':' || VL_ERRSQLMSG;
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_DEPARTAMENT',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_COD=' || IN_COD || ', ' || 'IN_DESC=' || IN_DESC,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);
      

  END P_DEPARTAMENT;



  /* ******************
     Alta/Baixa/Modificació de dades taula GR_DEPARTAMENT 
     IN_ABM: A(lta), B(aixa), M(odificació)
     IN_COD: Codi del registre, si nou registre = Null
     IN_NOM, IN_COG1, IN_COG2, IN_DOC: Dades de la persona
     IN_IDDEPT: Departament a la que pertany la persona
     RSP: Codi de sortida del procediment
  ****************** */
  PROCEDURE P_PERSONA (IN_ABM  VARCHAR2, 
                       IN_COD  NUMBER,
                       IN_NOM  VARCHAR2,
                       IN_COG1 VARCHAR2,
                       IN_COG2 VARCHAR2,
                       IN_DOC  VARCHAR2,
                       IN_IDDEPT NUMBER,
                       RSP OUT VARCHAR2) IS
                           
    VAUX DATE;
    VAUX_COUNT VARCHAR2(2000);
    
    VL_ERRSQLCOD NUMBER;
    VL_ERRSQLMSG VARCHAR2(255);
    
    ERR_CTRL_PARAM EXCEPTION; 
    ERR_CTRL_PARAM_NO_AUTORITZAT EXCEPTION;
    ERR_CTRL_ACCIO_NO_AUTORITZAT EXCEPTION;
    
    VL_ERRCODE GR_LOG.RSP%TYPE;
    VL_ERRDESC GR_LOG.RSPDESC%TYPE;
    
  BEGIN
   
    IF IN_ABM NOT IN ('A','B','M') THEN
        RAISE ERR_CTRL_PARAM;
    ELSIF IN_ABM IN ('B') AND IN_COD IS NULL THEN
        RAISE ERR_CTRL_PARAM;
    END IF;
  
    -- Alta de registre
    IF IN_ABM = 'A' THEN
    
        -- Alta d'un registre nou
        IF IN_COD IS NULL THEN
            -- es comproba que no existeixi el DOC
            SELECT count(1)
            INTO VAUX_COUNT
            FROM GR_PERSONA
            WHERE UPPER(DOCUMENT) = UPPER(IN_DOC);
            
            IF VAUX_COUNT = 0 THEN
                INSERT INTO GR_PERSONA (ID, NOM, COGNOM1, COGNOM2, DOCUMENT, DBAIXA, ID_DEPARTAMENT) VALUES 
                (SQ_PERSONA.NEXTVAL, IN_NOM, IN_COG1, IN_COG2, IN_DOC, NULL, IN_IDDEPT);
            ELSE 
                -- no autortizat, la descripcio ja existeix
                RAISE ERR_CTRL_ACCIO_NO_AUTORITZAT;
            END IF;
        
        -- Activació d'un registre previament desactivat
        ELSE 
            BEGIN
                SELECT DBAIXA
                INTO VAUX
                FROM GR_PERSONA
                WHERE ID = IN_COD
                AND DBAIXA IS NOT NULL
                FOR UPDATE;
                UPDATE GR_PERSONA SET DBAIXA = NULL WHERE ID = IN_COD;
            EXCEPTION
                WHEN NO_DATA_FOUND THEN
                    -- el registre ja esta donat d'alta o no existeix
                    RAISE ERR_CTRL_ACCIO_NO_AUTORITZAT;
            END;
        END IF;

    
    -- baixa de registre, bloqueig de registre per modificar
    ELSIF IN_ABM = 'B' THEN
        BEGIN
            SELECT DBAIXA
            INTO VAUX
            FROM GR_PERSONA
            WHERE ID = IN_COD
            AND DBAIXA IS NULL
            FOR UPDATE;
            UPDATE GR_PERSONA SET DBAIXA = SYSDATE WHERE ID = IN_COD;
        EXCEPTION 
            WHEN OTHERS THEN
                -- el registre no esta donat d'alta o no existeix
                RAISE ERR_CTRL_ACCIO_NO_AUTORITZAT;
        END;

    -- modificació del registre, es permet modificar les dades personals i el departament assignat
    ELSIF IN_ABM = 'M' THEN
    
        SELECT count(1)
        INTO VAUX_COUNT
        FROM GR_PERSONA
        WHERE ID = IN_COD;
        
        IF VAUX_COUNT = 0 THEN
            -- no autortizat, el registre no existeix
            RAISE ERR_CTRL_ACCIO_NO_AUTORITZAT;
        ELSE 
            UPDATE GR_PERSONA
            SET COGNOM1 = IN_NOM
              , COGNOM2 = IN_COG1
              , DOCUMENT = IN_DOC
              , ID_DEPARTAMENT = IN_IDDEPT
            WHERE ID = IN_COD; 
        END IF;
        
       
    END IF;   


    VL_ERRCODE := 'OK';
    VL_ERRDESC := NULL;
    RSP := VL_ERRCODE;
      
   
    PRC_LOG(NOMPRC => 'P_PERSONA',
            PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_COD=' || IN_COD || ', ' || 'IN_NOM=' || IN_NOM || ', ' ||
                      'IN_COG1=' || IN_COG1 || ', ' || 'IN_COG2=' || IN_COG2 || ', ' || IN_COG2 || ', '|| 
                      'IN_DOC=' || IN_DOC || ', ' || 'IN_IDDEPT=' || IN_IDDEPT,
            RSP    => VL_ERRCODE,
            RSPDESC=> VL_ERRDESC);

  EXCEPTION
    WHEN ERR_CTRL_PARAM THEN
      VL_ERRCODE := 'ERR_001';
      VL_ERRDESC := FNT_GET_ERRDESC(VL_ERRCODE);
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_PERSONA',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_COD=' || IN_COD || ', ' || 'IN_NOM=' || IN_NOM || ', ' ||
                      'IN_COG1=' || IN_COG1 || ', ' || 'IN_COG2=' || IN_COG2 || ', ' || IN_COG2 || ', '|| 
                      'IN_DOC=' || IN_DOC || ', ' || 'IN_IDDEPT=' || IN_IDDEPT,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);

    WHEN ERR_CTRL_PARAM_NO_AUTORITZAT THEN
      VL_ERRCODE := 'ERR_002';
      VL_ERRDESC := FNT_GET_ERRDESC(VL_ERRCODE);
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_PERSONA',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_COD=' || IN_COD || ', ' || 'IN_NOM=' || IN_NOM || ', ' ||
                      'IN_COG1=' || IN_COG1 || ', ' || 'IN_COG2=' || IN_COG2 || ', ' || IN_COG2 || ', '|| 
                      'IN_DOC=' || IN_DOC || ', ' || 'IN_IDDEPT=' || IN_IDDEPT,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);
    
    WHEN ERR_CTRL_ACCIO_NO_AUTORITZAT THEN
      VL_ERRCODE := 'ERR_003';
      VL_ERRDESC := FNT_GET_ERRDESC(VL_ERRCODE);
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_DEPARTAMENT',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_COD=' || IN_COD || ', ' || 'IN_NOM=' || IN_NOM || ', ' ||
                      'IN_COG1=' || IN_COG1 || ', ' || 'IN_COG2=' || IN_COG2 || ', ' || IN_COG2 || ', '|| 
                      'IN_DOC=' || IN_DOC || ', ' || 'IN_IDDEPT=' || IN_IDDEPT,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);
    
    
    WHEN OTHERS THEN
      VL_ERRSQLCOD := SQLCODE;
      VL_ERRSQLMSG := SQLERRM;
      VL_ERRCODE := 'ERR_999';
      VL_ERRDESC := VL_ERRSQLCOD || ':' || VL_ERRSQLMSG;
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_PERSONA',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_COD=' || IN_COD || ', ' || 'IN_NOM=' || IN_NOM || ', ' ||
                      'IN_COG1=' || IN_COG1 || ', ' || 'IN_COG2=' || IN_COG2 || ', ' || IN_COG2 || ', '|| 
                      'IN_DOC=' || IN_DOC || ', ' || 'IN_IDDEPT=' || IN_IDDEPT,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);
      

  END P_PERSONA;


  /* ******************
     Alta/Baixa/Modificació de dades taula GR_CATEGORIA 
     IN_ABM: A(lta), B(aixa), M(odificació)
     IN_COD: Codi del registre, si nou registre = Null
     IN_DESC: Descripció del registre
     RSP: Codi de sortida del procediment
  ****************** */
  PROCEDURE P_CATEGORIA (IN_ABM VARCHAR2, 
                         IN_COD NUMBER,
                         IN_DESC VARCHAR2,
                         IN_RESP VARCHAR2,
                         RSP OUT VARCHAR2) IS
                           
    VAUX DATE;
    VAUX_COUNT VARCHAR2(2000);
    
    VL_ERRSQLCOD NUMBER;
    VL_ERRSQLMSG VARCHAR2(255);
    
    ERR_CTRL_PARAM EXCEPTION; 
    ERR_CTRL_PARAM_NO_AUTORITZAT EXCEPTION;
    ERR_CTRL_ACCIO_NO_AUTORITZAT EXCEPTION;
    
    VL_ERRCODE GR_LOG.RSP%TYPE;
    VL_ERRDESC GR_LOG.RSPDESC%TYPE;
    
  BEGIN
   
    IF IN_ABM NOT IN ('A','B','M') THEN
        RAISE ERR_CTRL_PARAM;
    ELSIF IN_ABM IN ('B') AND IN_COD IS NULL THEN
        RAISE ERR_CTRL_PARAM;
    --ELSIF IN_ABM IN ('M') THEN
    --    RAISE ERR_CTRL_PARAM_NO_AUTORITZAT;
    END IF;
  
    -- Alta de registre
    IF IN_ABM = 'A' THEN
    
        -- Alta d'un registre nou
        IF IN_COD IS NULL THEN
            dbms_output.put_line('1');
            SELECT count(1)
            INTO VAUX_COUNT
            FROM GR_CATEGORIA
            WHERE UPPER(DESCRIPCIO) = UPPER(IN_DESC);
            
            IF VAUX_COUNT = 0 THEN
                INSERT INTO GR_CATEGORIA (ID, DESCRIPCIO, DBAIXA, ID_RESPONSABLE) VALUES 
                (SQ_CATEGORIA.NEXTVAL, IN_DESC, NULL, IN_RESP);
            ELSE 
                -- no autortizat, la descripcio ja existeix
                RAISE ERR_CTRL_ACCIO_NO_AUTORITZAT;
            END IF;
        
        -- Activació d'un registre previament desactivat
        ELSE 
            BEGIN
                SELECT DBAIXA
                INTO VAUX
                FROM GR_CATEGORIA
                WHERE ID = IN_COD
                AND DBAIXA IS NOT NULL
                FOR UPDATE;
                UPDATE GR_CATEGORIA SET DBAIXA = NULL WHERE ID = IN_COD;
            EXCEPTION
                WHEN NO_DATA_FOUND THEN
                    -- el registre ja esta donat d'alta o no existeix
                    RAISE ERR_CTRL_ACCIO_NO_AUTORITZAT;
            END;
        END IF;

    
    -- baixa de registre, bloqueig de registre per modificar
    ELSIF IN_ABM = 'B' THEN
        BEGIN
            SELECT DBAIXA
            INTO VAUX
            FROM GR_CATEGORIA
            WHERE ID = IN_COD
            AND DBAIXA IS NULL
            FOR UPDATE;
            UPDATE GR_CATEGORIA SET DBAIXA = SYSDATE WHERE ID = IN_COD;
        EXCEPTION 
            WHEN OTHERS THEN
                -- el registre no esta donat d'alta o no existeix
                RAISE ERR_CTRL_ACCIO_NO_AUTORITZAT;
        END;
        
    ELSIF IN_ABM = 'M' THEN
        UPDATE GR_CATEGORIA SET ID_RESPONSABLE = IN_RESP WHERE ID = IN_COD;

    END IF;   


    VL_ERRCODE := 'OK';
    VL_ERRDESC := NULL;
    RSP := VL_ERRCODE;
      
    
    PRC_LOG(NOMPRC => 'P_CATEGORIA',
            PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_COD=' || IN_COD || ', ' || 'IN_DESC=' || IN_DESC || ', ' || 'IN_RESP=' || IN_RESP,
            RSP    => VL_ERRCODE,
            RSPDESC=> VL_ERRDESC);

  EXCEPTION
    WHEN ERR_CTRL_PARAM THEN
      VL_ERRCODE := 'ERR_001';
      VL_ERRDESC := FNT_GET_ERRDESC(VL_ERRCODE);
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_CATEGORIA',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_COD=' || IN_COD || ', ' || 'IN_DESC=' || IN_DESC || ', ' || 'IN_RESP=' || IN_RESP,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);

    WHEN ERR_CTRL_PARAM_NO_AUTORITZAT THEN
      VL_ERRCODE := 'ERR_002';
      VL_ERRDESC := FNT_GET_ERRDESC(VL_ERRCODE);
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_CATEGORIA',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_COD=' || IN_COD || ', ' || 'IN_DESC=' || IN_DESC || ', ' || 'IN_RESP=' || IN_RESP,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);
    
    WHEN ERR_CTRL_ACCIO_NO_AUTORITZAT THEN
      VL_ERRCODE := 'ERR_003';
      VL_ERRDESC := FNT_GET_ERRDESC(VL_ERRCODE);
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_CATEGORIA',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_COD=' || IN_COD || ', ' || 'IN_DESC=' || IN_DESC || ', ' || 'IN_RESP=' || IN_RESP,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);
    
    
    WHEN OTHERS THEN
      VL_ERRSQLCOD := SQLCODE;
      VL_ERRSQLMSG := SQLERRM;
      VL_ERRCODE := 'ERR_999';
      VL_ERRDESC := VL_ERRSQLCOD || ':' || VL_ERRSQLMSG;
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_CATEGORIA',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_COD=' || IN_COD || ', ' || 'IN_DESC=' || IN_DESC || ', ' || 'IN_RESP=' || IN_RESP,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);
      

  END P_CATEGORIA;


  /* ******************
     Manteniment de la Taula GR_DEPARTAMENT_CATEGORIA, es lliguen els departaments amb les categories.
     Alta/Baixa/Modificació de dades taula GR_CATEGORIA 
     IN_ABM: A(lta), B(aixa), M(odificació)
     IN_IDDEPT: Codi del departament 
     IN_IDCAT: Codi de la categoria
     RSP: Codi de sortida del procediment
  ****************** */
  PROCEDURE P_DEPARTAMENT_CATEGORIA (IN_ABM VARCHAR2, 
                                     IN_IDDEPT NUMBER,
                                     IN_IDCAT  NUMBER,
                                     RSP OUT VARCHAR2) IS
                           
    VAUX DATE;
    VAUX_COUNT VARCHAR2(2000);
    
    VL_ERRSQLCOD NUMBER;
    VL_ERRSQLMSG VARCHAR2(255);
    
    ERR_CTRL_PARAM EXCEPTION; 
    ERR_CTRL_PARAM_NO_AUTORITZAT EXCEPTION;
    ERR_CTRL_ACCIO_NO_AUTORITZAT EXCEPTION;
    ERR_CTRL_ACCIO_NO_AUTORITZAT_2 EXCEPTION;
    
    VL_ERRCODE GR_LOG.RSP%TYPE;
    VL_ERRDESC GR_LOG.RSPDESC%TYPE;
    
  BEGIN
   
    IF IN_ABM NOT IN ('A','B','M') THEN
        RAISE ERR_CTRL_PARAM;
    ELSIF IN_IDDEPT IS NULL OR IN_IDCAT IS NULL THEN
        RAISE ERR_CTRL_PARAM;
    ELSIF IN_ABM IN ('M') THEN
        RAISE ERR_CTRL_PARAM_NO_AUTORITZAT;
    END IF;
  
    -- Alta de registre
    IF IN_ABM = 'A' THEN

        BEGIN
            SELECT DBAIXA
            INTO VAUX
            FROM GR_DEPARTAMENT_CATEGORIA
            WHERE ID_DEPARTAMENT = IN_IDDEPT
            AND ID_CATEGORIA = IN_IDCAT;
            
            IF VAUX IS NULL THEN
                RAISE ERR_CTRL_ACCIO_NO_AUTORITZAT;
            ELSE
                UPDATE GR_DEPARTAMENT_CATEGORIA 
                SET DBAIXA = NULL
                WHERE ID_DEPARTAMENT = IN_IDDEPT
                AND ID_CATEGORIA = IN_IDCAT;
            END IF;
            
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
                INSERT INTO GR_DEPARTAMENT_CATEGORIA (ID, ID_DEPARTAMENT, ID_CATEGORIA) VALUES 
                (SQ_DEPARTAMENT_CATEGORIA.NEXTVAL, IN_IDDEPT, IN_IDCAT);
        END;

    -- baixa de registre, bloqueig de registre per modificar
    ELSIF IN_ABM = 'B' THEN
        BEGIN
            SELECT DBAIXA
            INTO VAUX
            FROM GR_DEPARTAMENT_CATEGORIA
            WHERE ID_DEPARTAMENT = IN_IDDEPT
            AND ID_CATEGORIA = IN_IDCAT;
            
            IF VAUX IS NULL THEN
                UPDATE GR_DEPARTAMENT_CATEGORIA 
                SET DBAIXA = SYSDATE
                WHERE ID_DEPARTAMENT = IN_IDDEPT
                AND ID_CATEGORIA = IN_IDCAT;
                
            ELSE
                RAISE ERR_CTRL_ACCIO_NO_AUTORITZAT;
            END IF;
            
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
                RAISE ERR_CTRL_ACCIO_NO_AUTORITZAT_2;
        END;
            
    END IF;   


    VL_ERRCODE := 'OK';
    VL_ERRDESC := NULL;
    RSP := VL_ERRCODE;
      
    
    PRC_LOG(NOMPRC => 'P_DEPARTAMENT_CATEGORIA',
            PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_IDDEPT=' || IN_IDDEPT || ', ' || 'IN_IDCAT=' || IN_IDCAT,
            RSP    => VL_ERRCODE,
            RSPDESC=> VL_ERRDESC);

  EXCEPTION
    WHEN ERR_CTRL_PARAM THEN
      VL_ERRCODE := 'ERR_001';
      VL_ERRDESC := FNT_GET_ERRDESC(VL_ERRCODE);
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_DEPARTAMENT_CATEGORIA',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_IDDEPT=' || IN_IDDEPT || ', ' || 'IN_IDCAT=' || IN_IDCAT,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);

    WHEN ERR_CTRL_PARAM_NO_AUTORITZAT THEN
      VL_ERRCODE := 'ERR_002';
      VL_ERRDESC := FNT_GET_ERRDESC(VL_ERRCODE);
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_DEPARTAMENT_CATEGORIA',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_IDDEPT=' || IN_IDDEPT || ', ' || 'IN_IDCAT=' || IN_IDCAT,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);
    
    WHEN ERR_CTRL_ACCIO_NO_AUTORITZAT THEN
      VL_ERRCODE := 'ERR_003';
      VL_ERRDESC := FNT_GET_ERRDESC(VL_ERRCODE);
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_DEPARTAMENT_CATEGORIA',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_IDDEPT=' || IN_IDDEPT || ', ' || 'IN_IDCAT=' || IN_IDCAT,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);

    WHEN ERR_CTRL_ACCIO_NO_AUTORITZAT_2 THEN
      VL_ERRCODE := 'ERR_005';
      VL_ERRDESC := FNT_GET_ERRDESC(VL_ERRCODE);
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_DEPARTAMENT_CATEGORIA',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_IDDEPT=' || IN_IDDEPT || ', ' || 'IN_IDCAT=' || IN_IDCAT,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);    
    
    WHEN OTHERS THEN
      VL_ERRSQLCOD := SQLCODE;
      VL_ERRSQLMSG := SQLERRM;
      VL_ERRCODE := 'ERR_999';
      VL_ERRDESC := VL_ERRSQLCOD || ':' || VL_ERRSQLMSG;
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_DEPARTAMENT_CATEGORIA',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_IDDEPT=' || IN_IDDEPT || ', ' || 'IN_IDCAT=' || IN_IDCAT,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);
      

  END P_DEPARTAMENT_CATEGORIA;


  /* ******************
     Manteniment de la Taula GR_CATEGORIA_IMPORTANCIA, es lliguen les categories amb el seu nivell d'importancia
     Alta/Baixa/Modificació de dades taula GR_CATEGORIA 
     IN_ABM: A(lta), B(aixa), M(odificació)
     IN_IDDEPT: Codi del departament 
     IN_IDCAT: Codi de la categoria
     RSP: Codi de sortida del procediment
  ****************** */
  PROCEDURE P_CATEGORIA_IMPORTANCIA (IN_ABM VARCHAR2, 
                                     IN_IDCAT  NUMBER,
                                     IN_IDIMP  NUMBER,
                                     RSP OUT VARCHAR2) IS
                           
    VAUX DATE;
    VAUX_COUNT NUMBER;
    
    VL_ERRSQLCOD NUMBER;
    VL_ERRSQLMSG VARCHAR2(255);
    
    ERR_CTRL_PARAM EXCEPTION; 
    ERR_CTRL_PARAM_NO_AUTORITZAT EXCEPTION;
    ERR_CTRL_ACCIO_NO_AUTORITZAT EXCEPTION;
    ERR_CTRL_ACCIO_NO_AUTORITZAT_2 EXCEPTION;
    
    VL_ERRCODE GR_LOG.RSP%TYPE;
    VL_ERRDESC GR_LOG.RSPDESC%TYPE;
    
  BEGIN
   
    IF IN_ABM NOT IN ('A','B','M') THEN
        RAISE ERR_CTRL_PARAM;
    ELSIF IN_IDCAT IS NULL OR IN_IDIMP IS NULL THEN
        RAISE ERR_CTRL_PARAM;
    ELSIF IN_ABM IN ('B','M') THEN
        RAISE ERR_CTRL_PARAM_NO_AUTORITZAT;
    END IF;
  
    --unicament es permet el parametre A(lta)
  
    SELECT COUNT(1)
    INTO VAUX_COUNT
    FROM GR_CATEGORIA_IMPORTANCIA
    WHERE ID_IMPORTANCIA = IN_IDIMP
    AND ID_CATEGORIA = IN_IDCAT
    AND DDESACTIVAT IS NULL;
    
    IF VAUX_COUNT > 0 THEN
        RAISE ERR_CTRL_ACCIO_NO_AUTORITZAT;
    ELSE
        dbms_output.put_line('1');
        -- es desactiven tots
        UPDATE GR_CATEGORIA_IMPORTANCIA 
        SET DDESACTIVAT = SYSDATE
        WHERE ID_CATEGORIA = IN_IDCAT
        --AND ID_IMPORTANCIA = IN_IDIMP
        AND DDESACTIVAT IS NULL;
        dbms_output.put_line('2');
        -- es crea el nou registre
        INSERT INTO GR_CATEGORIA_IMPORTANCIA (ID, ID_IMPORTANCIA, ID_CATEGORIA, DACTIU) VALUES 
        (SQ_CATEGORIA_IMPORTANCIA.NEXTVAL, IN_IDIMP, IN_IDCAT, SYSDATE);
        dbms_output.put_line('3');
    END IF;
        
    VL_ERRCODE := 'OK';
    VL_ERRDESC := NULL;
    RSP := VL_ERRCODE;
    
  
    PRC_LOG(NOMPRC => 'P_CATEGORIA_IMPORTANCIA',
            PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_IDCAT=' || IN_IDCAT || ', ' || 'IN_IDIMP=' || IN_IDIMP,
            RSP    => VL_ERRCODE,
            RSPDESC=> VL_ERRDESC);

  EXCEPTION
    WHEN ERR_CTRL_PARAM THEN
      VL_ERRCODE := 'ERR_001';
      VL_ERRDESC := FNT_GET_ERRDESC(VL_ERRCODE);
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_CATEGORIA_IMPORTANCIA',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_IDCAT=' || IN_IDCAT || ', ' || 'IN_IDIMP=' || IN_IDIMP,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);

    WHEN ERR_CTRL_PARAM_NO_AUTORITZAT THEN
      VL_ERRCODE := 'ERR_002';
      VL_ERRDESC := FNT_GET_ERRDESC(VL_ERRCODE);
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_CATEGORIA_IMPORTANCIA',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_IDCAT=' || IN_IDCAT || ', ' || 'IN_IDIMP=' || IN_IDIMP,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);
    
    WHEN ERR_CTRL_ACCIO_NO_AUTORITZAT THEN
      VL_ERRCODE := 'ERR_003';
      VL_ERRDESC := FNT_GET_ERRDESC(VL_ERRCODE);
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_CATEGORIA_IMPORTANCIA',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_IDCAT=' || IN_IDCAT || ', ' || 'IN_IDIMP=' || IN_IDIMP,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);

    WHEN OTHERS THEN
      VL_ERRSQLCOD := SQLCODE;
      VL_ERRSQLMSG := SQLERRM;
      VL_ERRCODE := 'ERR_999';
      VL_ERRDESC := VL_ERRSQLCOD || ':' || VL_ERRSQLMSG;
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_CATEGORIA_IMPORTANCIA',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_IDCAT=' || IN_IDCAT || ', ' || 'IN_IDIMP=' || IN_IDIMP,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);
      

  END P_CATEGORIA_IMPORTANCIA;



  /* ******************
     Alta/Baixa/Modificació de dades taula GR_CONTROLINTERN 
     IN_ABM: A(lta), B(aixa), M(odificació)
     IN_COD: Codi del registre, si nou registre = Null
     IN_DESC: Descripció de l'equip intern
     RSP: Codi de sortida del procediment
  ****************** */
  PROCEDURE P_CONTROLINTERN (IN_ABM VARCHAR2, 
                             IN_COD NUMBER,
                             IN_DESC VARCHAR2,
                             RSP OUT VARCHAR2) IS
                           
    VAUX DATE;
    VAUX_COUNT NUMBER;
    
    VL_ERRSQLCOD NUMBER;
    VL_ERRSQLMSG VARCHAR2(255);
    
    ERR_CTRL_PARAM EXCEPTION; 
    ERR_CTRL_PARAM_NO_AUTORITZAT EXCEPTION;
    ERR_CTRL_ACCIO_NO_AUTORITZAT EXCEPTION;
    
    VL_ERRCODE GR_LOG.RSP%TYPE;
    VL_ERRDESC GR_LOG.RSPDESC%TYPE;
    
  BEGIN
   
    IF IN_ABM NOT IN ('A','B','M') THEN
        RAISE ERR_CTRL_PARAM;
    ELSIF IN_ABM IN ('B') AND IN_COD IS NULL THEN
        RAISE ERR_CTRL_PARAM;
    ELSIF IN_ABM IN ('M') THEN
        RAISE ERR_CTRL_PARAM_NO_AUTORITZAT;
    END IF;
  
    -- Alta de registre
    IF IN_ABM = 'A' THEN
    
        -- Alta d'un registre nou
        IF IN_COD IS NULL THEN
            dbms_output.put_line('1');
            SELECT count(1)
            INTO VAUX_COUNT
            FROM GR_CONTROLINTERN
            WHERE UPPER(DESCRIPCIO) = UPPER(IN_DESC);
            
            IF VAUX_COUNT = 0 THEN
                INSERT INTO GR_CONTROLINTERN (ID, DESCRIPCIO) VALUES 
                (SQ_CONTROLINTERN.NEXTVAL, IN_DESC);
            ELSE 
                -- no autortizat, la descripcio ja existeix
                RAISE ERR_CTRL_ACCIO_NO_AUTORITZAT;
            END IF;
        
        -- Activació d'un registre previament desactivat
        ELSE 
            BEGIN
                SELECT DBAIXA
                INTO VAUX
                FROM GR_CONTROLINTERN
                WHERE ID = IN_COD
                AND DBAIXA IS NOT NULL
                FOR UPDATE;
                UPDATE GR_CONTROLINTERN SET DBAIXA = NULL WHERE ID = IN_COD;
            EXCEPTION
                WHEN NO_DATA_FOUND THEN
                    -- el registre ja esta donat d'alta o no existeix
                    RAISE ERR_CTRL_ACCIO_NO_AUTORITZAT;
            END;
        END IF;

    
    -- baixa de registre, bloqueig de registre per modificar
    ELSIF IN_ABM = 'B' THEN
        BEGIN
            SELECT DBAIXA
            INTO VAUX
            FROM GR_CONTROLINTERN
            WHERE ID = IN_COD
            AND DBAIXA IS NULL
            FOR UPDATE;
            UPDATE GR_CONTROLINTERN SET DBAIXA = SYSDATE WHERE ID = IN_COD;
        EXCEPTION 
            WHEN OTHERS THEN
                -- el registre no esta donat d'alta o no existeix
                RAISE ERR_CTRL_ACCIO_NO_AUTORITZAT;
        END;
        
    END IF;   


    VL_ERRCODE := 'OK';
    VL_ERRDESC := NULL;
    RSP := VL_ERRCODE;
      
    
    PRC_LOG(NOMPRC => 'P_CONTROLINTERN',
            PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_COD=' || IN_COD || ', ' || 'IN_DESC=' || IN_DESC,
            RSP    => VL_ERRCODE,
            RSPDESC=> VL_ERRDESC);

  EXCEPTION
    WHEN ERR_CTRL_PARAM THEN
      VL_ERRCODE := 'ERR_001';
      VL_ERRDESC := FNT_GET_ERRDESC(VL_ERRCODE);
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_CONTROLINTERN',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_COD=' || IN_COD || ', ' || 'IN_DESC=' || IN_DESC,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);

    WHEN ERR_CTRL_PARAM_NO_AUTORITZAT THEN
      VL_ERRCODE := 'ERR_002';
      VL_ERRDESC := FNT_GET_ERRDESC(VL_ERRCODE);
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_CONTROLINTERN',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_COD=' || IN_COD || ', ' || 'IN_DESC=' || IN_DESC,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);
    
    WHEN ERR_CTRL_ACCIO_NO_AUTORITZAT THEN
      VL_ERRCODE := 'ERR_003';
      VL_ERRDESC := FNT_GET_ERRDESC(VL_ERRCODE);
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_CONTROLINTERN',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_COD=' || IN_COD || ', ' || 'IN_DESC=' || IN_DESC,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);
    
    
    WHEN OTHERS THEN
      VL_ERRSQLCOD := SQLCODE;
      VL_ERRSQLMSG := SQLERRM;
      VL_ERRCODE := 'ERR_999';
      VL_ERRDESC := VL_ERRSQLCOD || ':' || VL_ERRSQLMSG;
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_CONTROLINTERN',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_COD=' || IN_COD || ', ' || 'IN_DESC=' || IN_DESC,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);
      

  END P_CONTROLINTERN;

  PROCEDURE P_CONTROLINTERN_PERSONA (IN_ABM    VARCHAR2, 
                                     IN_IDCI   NUMBER,
                                     IN_IDPER  NUMBER,
                                     RSP OUT VARCHAR2) IS
                           
    VAUX DATE;
    VAUX_COUNT NUMBER;
    
    VL_ERRSQLCOD NUMBER;
    VL_ERRSQLMSG VARCHAR2(255);
    
    ERR_CTRL_PARAM EXCEPTION; 
    ERR_CTRL_PARAM_NO_AUTORITZAT EXCEPTION;
    ERR_CTRL_ACCIO_NO_AUTORITZAT EXCEPTION;
    ERR_CTRL_ACCIO_NO_AUTORITZAT_2 EXCEPTION;
    
    VL_ERRCODE GR_LOG.RSP%TYPE;
    VL_ERRDESC GR_LOG.RSPDESC%TYPE;
    
  BEGIN
   
    IF IN_ABM NOT IN ('A','B','M') THEN
        RAISE ERR_CTRL_PARAM;
    ELSIF IN_IDCI IS NULL OR IN_IDPER IS NULL THEN
        RAISE ERR_CTRL_PARAM;
    ELSIF IN_ABM IN ('M') THEN
        RAISE ERR_CTRL_PARAM_NO_AUTORITZAT;
    END IF;
  

    IF IN_ABM = 'A' THEN

        BEGIN
            SELECT DBAIXA
            INTO VAUX
            FROM GR_CONTROLINTERN_EQUIP
            WHERE ID_CONTROLINTERN = IN_IDCI
            AND ID_PERSONA = IN_IDPER;
            
            IF VAUX IS NOT NULL THEN
                UPDATE GR_CONTROLINTERN_EQUIP
                SET DBAIXA = NULL
                WHERE ID_CONTROLINTERN = IN_IDCI
                AND ID_PERSONA = IN_IDPER;
            ELSE
                RAISE ERR_CTRL_ACCIO_NO_AUTORITZAT;
            END IF;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
                INSERT INTO GR_CONTROLINTERN_EQUIP (ID_CONTROLINTERN, ID_PERSONA, DALTA) VALUES 
                (IN_IDCI, IN_IDPER, SYSDATE);
        END;
        
    ELSIF IN_ABM = 'B' THEN
        BEGIN
            SELECT DBAIXA
            INTO VAUX
            FROM GR_CONTROLINTERN_EQUIP
            WHERE ID_CONTROLINTERN = IN_IDCI
            AND ID_PERSONA = IN_IDPER
            AND DBAIXA IS NULL;
            
            UPDATE GR_CONTROLINTERN_EQUIP
            SET DBAIXA = SYSDATE
            WHERE ID_CONTROLINTERN = IN_IDCI
            AND ID_PERSONA = IN_IDPER;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
                RAISE ERR_CTRL_ACCIO_NO_AUTORITZAT_2;
        END;
        
    END IF;

        
    VL_ERRCODE := 'OK';
    VL_ERRDESC := NULL;
    RSP := VL_ERRCODE;
      
    PRC_LOG(NOMPRC => 'P_CONTROLINTERN_PERSONA',
            PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_IDCI=' || IN_IDCI || ', ' || 'IN_IDPER=' || IN_IDPER,
            RSP    => VL_ERRCODE,
            RSPDESC=> VL_ERRDESC);

  EXCEPTION
    WHEN ERR_CTRL_PARAM THEN
      VL_ERRCODE := 'ERR_001';
      VL_ERRDESC := FNT_GET_ERRDESC(VL_ERRCODE);
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_CONTROLINTERN_PERSONA',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_IDCI=' || IN_IDCI || ', ' || 'IN_IDPER=' || IN_IDPER,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);

    WHEN ERR_CTRL_PARAM_NO_AUTORITZAT THEN
      VL_ERRCODE := 'ERR_002';
      VL_ERRDESC := FNT_GET_ERRDESC(VL_ERRCODE);
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_CONTROLINTERN_PERSONA',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_IDCI=' || IN_IDCI || ', ' || 'IN_IDPER=' || IN_IDPER,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);
    
    WHEN ERR_CTRL_ACCIO_NO_AUTORITZAT THEN
      VL_ERRCODE := 'ERR_003';
      VL_ERRDESC := FNT_GET_ERRDESC(VL_ERRCODE);
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_CONTROLINTERN_PERSONA',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_IDCI=' || IN_IDCI || ', ' || 'IN_IDPER=' || IN_IDPER,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);

    WHEN ERR_CTRL_ACCIO_NO_AUTORITZAT_2 THEN
      VL_ERRCODE := 'ERR_005';
      VL_ERRDESC := FNT_GET_ERRDESC(VL_ERRCODE);
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_CONTROLINTERN_PERSONA',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_IDCI=' || IN_IDCI || ', ' || 'IN_IDPER=' || IN_IDPER,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC); 

    WHEN OTHERS THEN
      VL_ERRSQLCOD := SQLCODE;
      VL_ERRSQLMSG := SQLERRM;
      VL_ERRCODE := 'ERR_999';
      VL_ERRDESC := VL_ERRSQLCOD || ':' || VL_ERRSQLMSG;
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_CONTROLINTERN_PERSONA',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_IDCI=' || IN_IDCI || ', ' || 'IN_IDPER=' || IN_IDPER,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);
    

  END P_CONTROLINTERN_PERSONA;


  /* ******************
     Es relacionen tots els departaments d'una categoria en concret, amb un codi de control intern.
     
     Alta/Baixa/Modificació de dades taula GR_CONTROLINTERN 
     IN_ABM: A(lta), B(aixa), M(odificació)
     IN_IDCI: Codi del registre de control intern
     IN_IDCAT: Codi de la categoria
     RSP: Codi de sortida del procediment
  ****************** */
  PROCEDURE P_CONTROLINTERN_DPTOS_IMPLICATS (IN_ABM    VARCHAR2, 
                                             IN_IDCI   NUMBER,
                                             IN_IDCAT  NUMBER,
                                             RSP OUT VARCHAR2) IS

    VAUX_IDCI  NUMBER;
    VAUX_COUNT NUMBER;
    
    VL_ERRSQLCOD NUMBER;
    VL_ERRSQLMSG VARCHAR2(255);
    
    ERR_CTRL_PARAM EXCEPTION; 
    ERR_CTRL_PARAM_NO_AUTORITZAT EXCEPTION;
    ERR_CTRL_REGISTRE EXCEPTION;
    ERR_CTRL_REGISTRE_CATEGORIA EXCEPTION;

    VL_ERR_N1 VARCHAR2(50);    
    VL_ERRCODE GR_LOG.RSP%TYPE;
    VL_ERRDESC GR_LOG.RSPDESC%TYPE;
                                             
  BEGIN
  
    IF IN_ABM NOT IN ('A','B','M') THEN
        RAISE ERR_CTRL_PARAM;
    ELSIF IN_IDCI IS NULL OR IN_IDCAT IS NULL THEN
        RAISE ERR_CTRL_PARAM;
    ELSIF IN_ABM IN ('M') THEN
        RAISE ERR_CTRL_PARAM_NO_AUTORITZAT;
    END IF;
    
    BEGIN
        SELECT ID
        INTO VAUX_IDCI
        FROM GR_CONTROLINTERN
        WHERE ID = IN_IDCI
        AND DBAIXA IS NULL;
    EXCEPTION
        WHEN NO_DATA_FOUND THEN
            VL_ERR_N1 := 'GR_CONTROLINTERN';
            RAISE ERR_CTRL_REGISTRE;
    END;
    
    SELECT COUNT(1)
    INTO VAUX_COUNT
    FROM GR_DEPARTAMENT_CATEGORIA
    WHERE ID_CATEGORIA = IN_IDCAT;
    IF VAUX_COUNT = 0 THEN
        RAISE ERR_CTRL_REGISTRE_CATEGORIA;
    END IF;
    
    
    DELETE FROM GR_CONTROLINTERN_DPTOS_IMPLICATS
    WHERE ID_CONTROLINTERN = IN_IDCI
    AND ID_DEPARTAMENT_CATEGORIA IN (SELECT ID FROM GR_DEPARTAMENT_CATEGORIA WHERE ID_CATEGORIA = IN_IDCAT);
    
    IF IN_ABM = 'A' THEN
        INSERT INTO GR_CONTROLINTERN_DPTOS_IMPLICATS 
        (ID_CONTROLINTERN, ID_DEPARTAMENT_CATEGORIA)
        (SELECT IN_IDCI, ID
         FROM GR_DEPARTAMENT_CATEGORIA
         WHERE ID_CATEGORIA = IN_IDCAT
        );
    END IF;
      
  
    VL_ERRCODE := 'OK';
    VL_ERRDESC := NULL;
    RSP := VL_ERRCODE;
      
    PRC_LOG(NOMPRC => 'P_CONTROLINTERN_DPTOS_IMPLICATS',
            PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_IDCI=' || IN_IDCI || ', ' || 'IN_IDCAT=' || IN_IDCAT,
            RSP    => VL_ERRCODE,
            RSPDESC=> VL_ERRDESC);

  
  EXCEPTION
    WHEN ERR_CTRL_PARAM THEN
      VL_ERRCODE := 'ERR_001';
      VL_ERRDESC := FNT_GET_ERRDESC(VL_ERRCODE);
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_CONTROLINTERN_DPTOS_IMPLICATS',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_IDCI=' || IN_IDCI || ', ' || 'IN_IDCAT=' || IN_IDCAT,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);

    WHEN ERR_CTRL_PARAM_NO_AUTORITZAT THEN
      VL_ERRCODE := 'ERR_002';
      VL_ERRDESC := FNT_GET_ERRDESC(VL_ERRCODE);
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_CONTROLINTERN_DPTOS_IMPLICATS',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_IDCI=' || IN_IDCI || ', ' || 'IN_IDCAT=' || IN_IDCAT,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);
    
    WHEN ERR_CTRL_REGISTRE THEN
      VL_ERRCODE := 'ERR_006';
      VL_ERRDESC := REPLACE(FNT_GET_ERRDESC(VL_ERRCODE),'#N1#',VL_ERR_N1);
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_CONTROLINTERN_DPTOS_IMPLICATS',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_IDCI=' || IN_IDCI || ', ' || 'IN_IDCAT=' || IN_IDCAT,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);

    WHEN ERR_CTRL_REGISTRE_CATEGORIA THEN
      VL_ERRCODE := 'ERR_007';
      VL_ERRDESC := FNT_GET_ERRDESC(VL_ERRCODE);
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_CONTROLINTERN_DPTOS_IMPLICATS',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_IDCI=' || IN_IDCI || ', ' || 'IN_IDCAT=' || IN_IDCAT,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);

    WHEN OTHERS THEN
      VL_ERRSQLCOD := SQLCODE;
      VL_ERRSQLMSG := SQLERRM;
      VL_ERRCODE := 'ERR_999';
      VL_ERRDESC := VL_ERRSQLCOD || ':' || VL_ERRSQLMSG;
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_CONTROLINTERN_DPTOS_IMPLICATS',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_IDCI=' || IN_IDCI || ', ' || 'IN_IDCAT=' || IN_IDCAT,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);
    

  
  END P_CONTROLINTERN_DPTOS_IMPLICATS;


  /* ******************
     A(lta) - B(aixa) d'un CONTROL relacionat amb:
        ControlIntern
        Departament que el Revisa
        Departament que l'executa
        Tipus de Control
        
     IN_ABM: A(lta), B(aixa)
     IN_IDCTRL: id control intern
     IN_DESCCTRL: descripció del control intern
     IN_TIPCTRL: id tipus de control
     IN_IDCI: Codi del registre de control intern
     IN_IDDEPT_REVISIO: Codi del departament de la revisio
     IN_IDDEPT_EXECUCIO: Codi del departament de l'execució
     RSP: Codi de sortida del procediment
  ****************** */
  PROCEDURE P_CONTROL_CONTROLINTERN (IN_ABM    VARCHAR2, 
                                     IN_IDCTRL NUMBER,
                                     IN_DESCCTRL VARCHAR2,
                                     IN_DATA DATE,
                                     IN_TIPCTRL NUMBER,
                                     IN_IDCI   NUMBER,
                                     IN_IDDEPT_REVISIO  NUMBER,
                                     IN_IDDEPT_EXECUCIO  NUMBER,
                                     RSP OUT VARCHAR2) IS
                                     
    VAUX_IDCI  NUMBER;
    VAUX_COUNT NUMBER;
    
    VL_ERRSQLCOD NUMBER;
    VL_ERRSQLMSG VARCHAR2(255);
    
    ERR_CTRL_PARAM EXCEPTION; 
    ERR_CTRL_PARAM_NO_AUTORITZAT EXCEPTION;
    ERR_CTRL_REGISTRE EXCEPTION;
    ERR_CTRL_REGISTRE_CATEGORIA EXCEPTION;
    ERR_CTRL_ACCIO_NO_AUTORITZAT EXCEPTION;
    ERR_CTRL_ACCIO_NO_AUTORITZAT_2 EXCEPTION;

    VL_ERR_N1 VARCHAR2(50);    
    VL_ERRCODE GR_LOG.RSP%TYPE;
    VL_ERRDESC GR_LOG.RSPDESC%TYPE;
                                        
                                     
  BEGIN
  
    
    IF IN_ABM NOT IN ('A','B','M') THEN
        RAISE ERR_CTRL_PARAM;
    ELSIF IN_IDCI IS NULL OR IN_TIPCTRL IS NULL THEN
        RAISE ERR_CTRL_PARAM;
    ELSIF IN_ABM IN ('M') THEN
        RAISE ERR_CTRL_PARAM_NO_AUTORITZAT;
    END IF;
    
    IF IN_ABM = 'A' THEN
        SELECT COUNT(1) INTO VAUX_COUNT FROM GR_TIPUSCONTROL WHERE ID = IN_TIPCTRL;
        IF VAUX_COUNT = 0 THEN 
            VL_ERR_N1:= 'GR_TIPUSCONTROL';
            RAISE ERR_CTRL_REGISTRE;
        END IF;
    
        SELECT COUNT(1) INTO VAUX_COUNT FROM GR_CONTROLINTERN WHERE ID = IN_IDCI AND DBAIXA IS NULL;
        IF VAUX_COUNT = 0 THEN 
            VL_ERR_N1:= 'GR_CONTROLINTERN';
            RAISE ERR_CTRL_REGISTRE;
        END IF;
        
        SELECT COUNT(1) INTO VAUX_COUNT FROM GR_DEPARTAMENT WHERE ID = IN_IDDEPT_REVISIO AND DBAIXA IS NULL;
        IF VAUX_COUNT = 0 THEN 
            VL_ERR_N1:= 'GR_DEPARTAMENT (REVISIO)';
            RAISE ERR_CTRL_REGISTRE;
        END IF;
        
        SELECT COUNT(1) INTO VAUX_COUNT FROM GR_DEPARTAMENT WHERE ID = IN_IDDEPT_EXECUCIO AND DBAIXA IS NULL;
        IF VAUX_COUNT = 0 THEN 
            VL_ERR_N1:= 'GR_DEPARTAMENT (EXECUCIO)';
            RAISE ERR_CTRL_REGISTRE;
        END IF;
        
        IF IN_IDCTRL IS NULL THEN
            INSERT INTO GR_CONTROL
            (ID, DESCRIPCIO, DCREACIO, DBAIXA, ID_CONTROLINTERN, ID_DEPARTAMENTREVISIO, ID_DEPARTAMENTEXECUCIO, ID_TIPUSCONTROL)
            VALUES
            (SQ_CONTROL.NEXTVAL, IN_DESCCTRL, IN_DATA, NULL, IN_IDCI, IN_IDDEPT_REVISIO, IN_IDDEPT_EXECUCIO, IN_TIPCTRL);
            
        ELSE 
            SELECT COUNT(1) INTO VAUX_COUNT FROM GR_CONTROL WHERE ID = IN_IDCTRL AND DBAIXA IS NOT NULL;
            IF VAUX_COUNT = 0 THEN 
                RAISE ERR_CTRL_ACCIO_NO_AUTORITZAT;
            END IF;
            UPDATE GR_CONTROL
            SET DBAIXA = NULL
            WHERE ID = IN_IDCTRL;
        
        END IF;
        
        
        
    ELSIF IN_ABM = 'B' THEN
        SELECT COUNT(1) INTO VAUX_COUNT FROM GR_CONTROL WHERE ID = IN_IDCTRL AND DBAIXA IS NULL;
        IF VAUX_COUNT = 0 THEN 
            RAISE ERR_CTRL_ACCIO_NO_AUTORITZAT_2;
        END IF;
        UPDATE GR_CONTROL
        SET DBAIXA = SYSDATE
        WHERE ID = IN_IDCTRL;
        
    END IF;


    VL_ERRCODE := 'OK';
    VL_ERRDESC := NULL;
    RSP := VL_ERRCODE;
      
    PRC_LOG(NOMPRC => 'P_CONTROL_CONTROLINTERN',
            PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_IDCTRL=' || IN_IDCTRL || ', ' || 
                      'IN_DESCCTRL=' || IN_DESCCTRL || ', ' || 'IN_DATA=' || TO_CHAR(IN_DATA,'DD/MM/YYYY') || ', ' || 
                      'IN_TIPCTRL=' || IN_TIPCTRL || ', ' || 'IN_IDCI=' || IN_IDCI || ', ' ||                       
                      'IN_IDDEPT_REVISIO=' || IN_IDDEPT_REVISIO || ', ' || 'IN_IDDEPT_EXECUCIO=' || IN_IDDEPT_EXECUCIO,
            RSP    => VL_ERRCODE,
            RSPDESC=> VL_ERRDESC);
    
    EXCEPTION
    WHEN ERR_CTRL_PARAM THEN
      VL_ERRCODE := 'ERR_001';
      VL_ERRDESC := FNT_GET_ERRDESC(VL_ERRCODE);
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_CONTROL_CONTROLINTERN',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_IDCTRL=' || IN_IDCTRL || ', ' || 
                        'IN_DESCCTRL=' || IN_DESCCTRL || ', ' || 'IN_DATA=' || TO_CHAR(IN_DATA,'DD/MM/YYYY') || ', ' || 
                        'IN_TIPCTRL=' || IN_TIPCTRL || ', ' || 'IN_IDCI=' || IN_IDCI || ', ' ||                       
                        'IN_IDDEPT_REVISIO=' || IN_IDDEPT_REVISIO || ', ' || 'IN_IDDEPT_EXECUCIO=' || IN_IDDEPT_EXECUCIO,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);

    WHEN ERR_CTRL_PARAM_NO_AUTORITZAT THEN
      VL_ERRCODE := 'ERR_002';
      VL_ERRDESC := FNT_GET_ERRDESC(VL_ERRCODE);
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_CONTROL_CONTROLINTERN',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_IDCTRL=' || IN_IDCTRL || ', ' || 
                        'IN_DESCCTRL=' || IN_DESCCTRL || ', ' || 'IN_DATA=' || TO_CHAR(IN_DATA,'DD/MM/YYYY') || ', ' || 
                        'IN_TIPCTRL=' || IN_TIPCTRL || ', ' || 'IN_IDCI=' || IN_IDCI || ', ' ||                       
                        'IN_IDDEPT_REVISIO=' || IN_IDDEPT_REVISIO || ', ' || 'IN_IDDEPT_EXECUCIO=' || IN_IDDEPT_EXECUCIO,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);
    
    WHEN ERR_CTRL_REGISTRE THEN
      VL_ERRCODE := 'ERR_006';
      VL_ERRDESC := REPLACE(FNT_GET_ERRDESC(VL_ERRCODE),'#N1#',VL_ERR_N1);
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_CONTROL_CONTROLINTERN',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_IDCTRL=' || IN_IDCTRL || ', ' || 
                        'IN_DESCCTRL=' || IN_DESCCTRL || ', ' || 'IN_DATA=' || TO_CHAR(IN_DATA,'DD/MM/YYYY') || ', ' || 
                        'IN_TIPCTRL=' || IN_TIPCTRL || ', ' || 'IN_IDCI=' || IN_IDCI || ', ' ||                       
                        'IN_IDDEPT_REVISIO=' || IN_IDDEPT_REVISIO || ', ' || 'IN_IDDEPT_EXECUCIO=' || IN_IDDEPT_EXECUCIO,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);

    WHEN ERR_CTRL_REGISTRE_CATEGORIA THEN
      VL_ERRCODE := 'ERR_007';
      VL_ERRDESC := FNT_GET_ERRDESC(VL_ERRCODE);
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_CONTROL_CONTROLINTERN',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_IDCTRL=' || IN_IDCTRL || ', ' || 
                        'IN_DESCCTRL=' || IN_DESCCTRL || ', ' || 'IN_DATA=' || TO_CHAR(IN_DATA,'DD/MM/YYYY') || ', ' || 
                        'IN_TIPCTRL=' || IN_TIPCTRL || ', ' || 'IN_IDCI=' || IN_IDCI || ', ' ||                       
                        'IN_IDDEPT_REVISIO=' || IN_IDDEPT_REVISIO || ', ' || 'IN_IDDEPT_EXECUCIO=' || IN_IDDEPT_EXECUCIO,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);
              
    
    WHEN ERR_CTRL_ACCIO_NO_AUTORITZAT THEN
      VL_ERRCODE := 'ERR_003';
      VL_ERRDESC := FNT_GET_ERRDESC(VL_ERRCODE);
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_CONTROL_CONTROLINTERN',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_IDCTRL=' || IN_IDCTRL || ', ' || 
                        'IN_DESCCTRL=' || IN_DESCCTRL || ', ' || 'IN_DATA=' || TO_CHAR(IN_DATA,'DD/MM/YYYY') || ', ' || 
                        'IN_TIPCTRL=' || IN_TIPCTRL || ', ' || 'IN_IDCI=' || IN_IDCI || ', ' ||                       
                        'IN_IDDEPT_REVISIO=' || IN_IDDEPT_REVISIO || ', ' || 'IN_IDDEPT_EXECUCIO=' || IN_IDDEPT_EXECUCIO,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);
    
              
    WHEN ERR_CTRL_ACCIO_NO_AUTORITZAT_2 THEN
      VL_ERRCODE := 'ERR_005';
      VL_ERRDESC := FNT_GET_ERRDESC(VL_ERRCODE);
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_CONTROL_CONTROLINTERN',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_IDCTRL=' || IN_IDCTRL || ', ' || 
                        'IN_DESCCTRL=' || IN_DESCCTRL || ', ' || 'IN_DATA=' || TO_CHAR(IN_DATA,'DD/MM/YYYY') || ', ' || 
                        'IN_TIPCTRL=' || IN_TIPCTRL || ', ' || 'IN_IDCI=' || IN_IDCI || ', ' ||                       
                        'IN_IDDEPT_REVISIO=' || IN_IDDEPT_REVISIO || ', ' || 'IN_IDDEPT_EXECUCIO=' || IN_IDDEPT_EXECUCIO,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC); 

    WHEN OTHERS THEN
      VL_ERRSQLCOD := SQLCODE;
      VL_ERRSQLMSG := SQLERRM;
      VL_ERRCODE := 'ERR_999';
      VL_ERRDESC := VL_ERRSQLCOD || ':' || VL_ERRSQLMSG;
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_CONTROL_CONTROLINTERN',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_IDCTRL=' || IN_IDCTRL || ', ' || 
                        'IN_DESCCTRL=' || IN_DESCCTRL || ', ' || 'IN_DATA=' || TO_CHAR(IN_DATA,'DD/MM/YYYY') || ', ' || 
                        'IN_TIPCTRL=' || IN_TIPCTRL || ', ' || 'IN_IDCI=' || IN_IDCI || ', ' ||                       
                        'IN_IDDEPT_REVISIO=' || IN_IDDEPT_REVISIO || ', ' || 'IN_IDDEPT_EXECUCIO=' || IN_IDDEPT_EXECUCIO,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);
    
  
  END P_CONTROL_CONTROLINTERN;


  /* ******************
     Alta/Baixa/Modificació de CAMPANYES 
     IN_ABM: A(lta), B(aixa), M(odificació)
     IN_COD: Codi de la campanya, si nou registre = Null
     IN_DESC: Descripció de la campanya
     IN_DINI: Data inici de la campanya
     IN_DFI: Data fi de la campanya
     RSP: Codi de sortida del procediment
  ****************** */
  PROCEDURE P_CAMPANYA (IN_ABM VARCHAR2, 
                        IN_COD NUMBER,
                        IN_DESC VARCHAR2,
                        IN_DINI DATE,
                        IN_DFI DATE,
                        IN_LIDER NUMBER,
                        RSP OUT VARCHAR2) IS

    VAUX DATE;
    VAUX_COUNT VARCHAR2(2000);
    
    VL_ERRSQLCOD NUMBER;
    VL_ERRSQLMSG VARCHAR2(255);
    
    ERR_CTRL_PARAM EXCEPTION; 
    ERR_CTRL_PARAM_NO_AUTORITZAT EXCEPTION;
    ERR_CTRL_ACCIO_NO_AUTORITZAT_2 EXCEPTION;
    ERR_CTRL_JA_EXISTEIX EXCEPTION;
    
    VL_ERRCODE GR_LOG.RSP%TYPE;
    VL_ERRDESC GR_LOG.RSPDESC%TYPE;
    
  BEGIN
   
    IF IN_ABM NOT IN ('A','B','M') THEN
        RAISE ERR_CTRL_PARAM;
    ELSIF IN_ABM IN ('A') AND IN_COD IS NOT NULL THEN
        RAISE ERR_CTRL_PARAM;
    ELSIF IN_ABM IN ('B', 'M') AND IN_COD IS NULL THEN
        RAISE ERR_CTRL_PARAM;
    END IF;
  
    -- Alta de registre
    IF IN_ABM = 'A' THEN
    
        SELECT count(1)
        INTO VAUX_COUNT
        FROM GR_CAMPANYA
        WHERE UPPER(DESCRIPCIO) = UPPER(IN_DESC);
        
        IF VAUX_COUNT = 0 THEN
            INSERT INTO GR_CAMPANYA (ID, DESCRIPCIO, DINICI, DFI, PERSONA_LIDER_ID) VALUES 
            (SQ_CAMPANYA.NEXTVAL, IN_DESC, IN_DINI, IN_DFI, IN_LIDER);
        ELSE 
            -- no autortizat, la descripcio ja existeix
            RAISE ERR_CTRL_JA_EXISTEIX;
        END IF;
        

    ELSIF IN_ABM = 'M' THEN
    
            SELECT count(1)
            INTO VAUX_COUNT
            FROM GR_CAMPANYA
            WHERE ID <> IN_COD
            AND UPPER(DESCRIPCIO) = UPPER(IN_DESC);
            
            IF VAUX_COUNT = 0 THEN
                UPDATE GR_CAMPANYA 
                SET DESCRIPCIO = IN_DESC
                  , DINICI = IN_DINI
                  , DFI = IN_DFI
                  , PERSONA_LIDER_ID = IN_LIDER
                WHERE ID = IN_COD;
            ELSE 
                -- no autortizat, la descripcio ja existeix
                RAISE ERR_CTRL_JA_EXISTEIX;
            END IF;

    ELSIF IN_ABM = 'B' THEN
    
        SELECT count(1)
        INTO VAUX_COUNT
        FROM GR_CAMPANYA
        WHERE ID = IN_COD;
        
        IF VAUX_COUNT = 0 THEN
            RAISE ERR_CTRL_ACCIO_NO_AUTORITZAT_2;
        END IF;
        
        DELETE FROM GR_CAMPANYA_CONTROL
        WHERE ID_CAMPANYA = IN_COD;
        
        DELETE FROM GR_CAMPANYA
        WHERE ID = IN_COD;
    
    END IF;   


    VL_ERRCODE := 'OK';
    VL_ERRDESC := NULL;
    RSP := VL_ERRCODE;
        
    PRC_LOG(NOMPRC => 'P_CAMPANYA',
            PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_COD=' || IN_COD || ', ' || 'IN_DESC=' || IN_DESC ||
                      'IN_DINI=' || IN_DINI || ', ' || 'IN_DFI=' || IN_DFI,
            RSP    => VL_ERRCODE,
            RSPDESC=> VL_ERRDESC);

  EXCEPTION
    WHEN ERR_CTRL_PARAM THEN
      VL_ERRCODE := 'ERR_001';
      VL_ERRDESC := FNT_GET_ERRDESC(VL_ERRCODE);
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_CAMPANYA',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_COD=' || IN_COD || ', ' || 'IN_DESC=' || IN_DESC ||
                        'IN_DINI=' || IN_DINI || ', ' || 'IN_DFI=' || IN_DFI,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);

    WHEN ERR_CTRL_PARAM_NO_AUTORITZAT THEN
      VL_ERRCODE := 'ERR_002';
      VL_ERRDESC := FNT_GET_ERRDESC(VL_ERRCODE);
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_CAMPANYA',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_COD=' || IN_COD || ', ' || 'IN_DESC=' || IN_DESC ||
                        'IN_DINI=' || IN_DINI || ', ' || 'IN_DFI=' || IN_DFI,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);
    
    WHEN ERR_CTRL_JA_EXISTEIX THEN
      VL_ERRCODE := 'ERR_004';
      VL_ERRDESC := FNT_GET_ERRDESC(VL_ERRCODE);
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_CAMPANYA',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_COD=' || IN_COD || ', ' || 'IN_DESC=' || IN_DESC ||
                        'IN_DINI=' || IN_DINI || ', ' || 'IN_DFI=' || IN_DFI,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);
    
    WHEN ERR_CTRL_ACCIO_NO_AUTORITZAT_2 THEN
      VL_ERRCODE := 'ERR_005';
      VL_ERRDESC := FNT_GET_ERRDESC(VL_ERRCODE);
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_CAMPANYA',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_COD=' || IN_COD || ', ' || 'IN_DESC=' || IN_DESC ||
                        'IN_DINI=' || IN_DINI || ', ' || 'IN_DFI=' || IN_DFI,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);  
              
    WHEN OTHERS THEN
      VL_ERRSQLCOD := SQLCODE;
      VL_ERRSQLMSG := SQLERRM;
      VL_ERRCODE := 'ERR_999';
      VL_ERRDESC := VL_ERRSQLCOD || ':' || VL_ERRSQLMSG;
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_CAMPANYA',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_COD=' || IN_COD || ', ' || 'IN_DESC=' || IN_DESC ||
                        'IN_DINI=' || IN_DINI || ', ' || 'IN_DFI=' || IN_DFI,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);
      

  END P_CAMPANYA;

  /* ******************
     Alta/Baixa/Modificació de CAMPANYES I CONTROLS
     IN_ABM: A(lta), B(aixa), M(odificació)
     IN_CAMP: Codi de la campanya
     IN_CTRL: Codi del control
     RSP: Codi de sortida del procediment
  ****************** */
  PROCEDURE P_CAMPANYA_CONTROL (IN_ABM VARCHAR2, 
                                IN_CAMP NUMBER,
                                IN_CTRL NUMBER,
                                RSP OUT VARCHAR2) IS
                                
    VAUX_COUNT VARCHAR2(2000);
    
    VL_ERRSQLCOD NUMBER;
    VL_ERRSQLMSG VARCHAR2(255);
    VL_ERR_N1 VARCHAR2(50);
    
    ERR_CTRL_PARAM EXCEPTION; 
    ERR_CTRL_REGISTRE EXCEPTION; 
   
    VL_ERRCODE GR_LOG.RSP%TYPE;
    VL_ERRDESC GR_LOG.RSPDESC%TYPE;
                                
  BEGIN
  
    IF IN_ABM NOT IN ('A','B') THEN
        RAISE ERR_CTRL_PARAM;
    ELSIF IN_CAMP IS NULL OR IN_CTRL IS NULL THEN
        RAISE ERR_CTRL_PARAM;
    END IF;
  
    SELECT COUNT(1)
    INTO VAUX_COUNT
    FROM GR_CONTROL
    WHERE ID = IN_CTRL
    AND DBAIXA IS NULL;
    IF VAUX_COUNT = 0 THEN
        VL_ERR_N1:= 'GR_CONTROL';
        RAISE ERR_CTRL_REGISTRE;
    END IF;
    
    SELECT COUNT(1)
    INTO VAUX_COUNT
    FROM GR_CAMPANYA
    WHERE ID = IN_CAMP;
    IF VAUX_COUNT = 0 THEN
        VL_ERR_N1:= 'GR_CAMPANYA';
        RAISE ERR_CTRL_REGISTRE;
    END IF;
    
    DELETE FROM GR_CAMPANYA_CONTROL
    WHERE ID_CAMPANYA = IN_CAMP
    AND ID_CONTROL = IN_CTRL;
    
    INSERT INTO GR_CAMPANYA_CONTROL
    (ID, ID_CAMPANYA, ID_CONTROL) VALUES
    (SQ_CAMPANYA_CONTROL.NEXTVAL, IN_CAMP, IN_CTRL);
    

    VL_ERRCODE := 'OK';
    VL_ERRDESC := NULL;
    RSP := VL_ERRCODE;
        
    PRC_LOG(NOMPRC => 'P_CAMPANYA_CONTROL',
            PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_CAMP=' || IN_CAMP || ', ' || 'IN_CTRL=' || IN_CTRL,
            RSP    => VL_ERRCODE,
            RSPDESC=> VL_ERRDESC);

  
  EXCEPTION
    WHEN ERR_CTRL_PARAM THEN
      VL_ERRCODE := 'ERR_001';
      VL_ERRDESC := FNT_GET_ERRDESC(VL_ERRCODE);
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_CAMPANYA_CONTROL',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_CAMP=' || IN_CAMP || ', ' || 'IN_CTRL=' || IN_CTRL,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);

    WHEN ERR_CTRL_REGISTRE THEN
      VL_ERRCODE := 'ERR_006';
      VL_ERRDESC := REPLACE(FNT_GET_ERRDESC(VL_ERRCODE),'#N1#',VL_ERR_N1);
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_CAMPANYA_CONTROL',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_CAMP=' || IN_CAMP || ', ' || 'IN_CTRL=' || IN_CTRL,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);

    WHEN OTHERS THEN
      VL_ERRSQLCOD := SQLCODE;
      VL_ERRSQLMSG := SQLERRM;
      VL_ERRCODE := 'ERR_999';
      VL_ERRDESC := VL_ERRSQLCOD || ':' || VL_ERRSQLMSG;
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_CAMPANYA_CONTROL',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_CAMP=' || IN_CAMP || ', ' || 'IN_CTRL=' || IN_CTRL,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);
      
  
  END P_CAMPANYA_CONTROL;



  /* ******************
     Alta/Baixa/Modificació de RISCOS 
     IN_ABM: A(lta), B(aixa), M(odificació)
     IN_COD: Codi del RISC, si nou registre = Null
     IN_DESC: Descripció del Risc
     IN_DCREA: Data inici de la Risc
     IN_CAT: Id Categoria
     IN_ORIG: Id Origen
     IN_CRIT: Id Criticitat
     IN_ESTAT: Id Estat
     IN_REVEXT: Id Revisió Externa
     IN_ACCIO: ID Accio
     IN_REV: Id Revisió
     IN_CTRL: Id Control
     IN_PERSORESP: Id Persona Responsable
     RSP: Codi de sortida del procediment
  ****************** */
  PROCEDURE P_RISC (IN_ABM VARCHAR2, 
                    IN_COD NUMBER,
                    IN_DESC VARCHAR2,
                    IN_DCREA DATE,
                    IN_DFI DATE,
                    IN_CAT NUMBER,
                    IN_ORIG NUMBER,
                    IN_CRIT NUMBER,
                    IN_ESTAT NUMBER,
                    IN_REVEXT NUMBER,
                    IN_ACCIO NUMBER,
                    IN_REV NUMBER,
                    IN_CTRL NUMBER,
                    IN_PERSORESP NUMBER,
                    RSP OUT VARCHAR2) IS

    VAUX_COUNT VARCHAR2(2000);
    
    VL_ERRSQLCOD NUMBER;
    VL_ERRSQLMSG VARCHAR2(255);
    VL_ERR_N1 VARCHAR2(50);
    
    ERR_CTRL_PARAM EXCEPTION; 
    ERR_CTRL_REGISTRE EXCEPTION;
    
    VL_ERRCODE GR_LOG.RSP%TYPE;
    VL_ERRDESC GR_LOG.RSPDESC%TYPE;
    
  BEGIN
   
    -- la baixa no esta permesa
    IF IN_ABM NOT IN ('A','M') THEN
        RAISE ERR_CTRL_PARAM;
    ELSIF IN_ABM IN ('A') AND IN_COD IS NOT NULL THEN
        RAISE ERR_CTRL_PARAM;
    ELSIF IN_ABM IN ('M') AND IN_COD IS NULL THEN
        RAISE ERR_CTRL_PARAM;
    END IF;
  
    IF IN_ABM IN ('M') THEN
        SELECT COUNT(1) INTO VAUX_COUNT FROM GR_RISC WHERE ID = IN_COD;
        IF VAUX_COUNT = 0 THEN
            VL_ERR_N1:= 'GR_RISC';
            RAISE ERR_CTRL_REGISTRE;
        END IF;
    
    END IF;

    IF IN_ABM IN ('A','M') THEN
        IF IN_CAT IS NOT NULL THEN
            SELECT COUNT(1) INTO VAUX_COUNT FROM GR_CATEGORIA WHERE ID = IN_CAT;
            IF VAUX_COUNT = 0 THEN
                VL_ERR_N1:= 'GR_CATEGORIA';
                RAISE ERR_CTRL_REGISTRE;
            END IF;
        END IF;
        IF IN_ORIG IS NOT NULL THEN
            SELECT COUNT(1) INTO VAUX_COUNT FROM GR_ORIGEN WHERE ID = IN_ORIG;
            IF VAUX_COUNT = 0 THEN
                VL_ERR_N1:= 'GR_ORIGEN';
                RAISE ERR_CTRL_REGISTRE;
            END IF;
        END IF;
        IF IN_CRIT IS NOT NULL THEN
            SELECT COUNT(1) INTO VAUX_COUNT FROM GR_CRITICITAT WHERE ID = IN_CRIT;
            IF VAUX_COUNT = 0 THEN
                VL_ERR_N1:= 'GR_CRITICITAT';
                RAISE ERR_CTRL_REGISTRE;
            END IF;
        END IF;
        IF IN_ESTAT IS NOT NULL THEN
            SELECT COUNT(1) INTO VAUX_COUNT FROM GR_ESTAT WHERE ID = IN_ESTAT;
            IF VAUX_COUNT = 0 THEN
                VL_ERR_N1:= 'GR_ESTAT';
                RAISE ERR_CTRL_REGISTRE;
            END IF;
        END IF;
        IF IN_REVEXT IS NOT NULL THEN
            SELECT COUNT(1) INTO VAUX_COUNT FROM GR_REVISIOEXTERNA WHERE ID = IN_REVEXT;
            IF VAUX_COUNT = 0 THEN
                VL_ERR_N1:= 'GR_REVISIOEXTERNA';
                RAISE ERR_CTRL_REGISTRE;
            END IF;
        END IF;
        IF IN_ACCIO IS NOT NULL THEN
            SELECT COUNT(1) INTO VAUX_COUNT FROM GR_ACCIO WHERE ID = IN_ACCIO;
            IF VAUX_COUNT = 0 THEN
                VL_ERR_N1:= 'GR_ACCIO';
                RAISE ERR_CTRL_REGISTRE;
            END IF;
        END IF;
        IF IN_REV IS NOT NULL THEN
            SELECT COUNT(1) INTO VAUX_COUNT FROM GR_REVISIO WHERE ID = IN_REV;
            IF VAUX_COUNT = 0 THEN
                VL_ERR_N1:= 'GR_REVISIO';
                RAISE ERR_CTRL_REGISTRE;
            END IF;
        END IF;
        IF IN_CTRL IS NOT NULL THEN
            SELECT COUNT(1) INTO VAUX_COUNT FROM GR_CONTROL WHERE ID = IN_CTRL;
            IF VAUX_COUNT = 0 THEN
                VL_ERR_N1:= 'GR_CONTROL';
                RAISE ERR_CTRL_REGISTRE;
            END IF;
        END IF;
        IF IN_PERSORESP IS NOT NULL THEN
            SELECT COUNT(1) INTO VAUX_COUNT FROM GR_PERSONA WHERE ID = IN_PERSORESP;
            IF VAUX_COUNT = 0 THEN
                VL_ERR_N1:= 'GR_PERSONA';
                RAISE ERR_CTRL_REGISTRE;
            END IF;
        END IF;
    END IF;

    IF IN_ABM IN ('A') THEN
        INSERT INTO GR_RISC (
          ID, DCREACIO, DFI, DESCRIPCIO, ID_CATEGORIA, ID_ORIGEN, ID_CRITICITAT, 
          ID_ESTAT, ID_REVISIOEXTERNA, ID_ACCIO, ID_REVISIO, ID_CONTROL, ID_PERSONARESPONSABLE)
        VALUES (
          SQ_RISC.NEXTVAL, IN_DCREA, IN_DFI, IN_DESC, IN_CAT, IN_ORIG, IN_CRIT,
          IN_ESTAT, IN_REVEXT, IN_ACCIO, IN_REV, IN_CTRL, IN_PERSORESP
        );
        
    ELSIF IN_ABM IN ('M') THEN
        UPDATE GR_RISC
        SET DESCRIPCIO = IN_DESC
          , ID_CATEGORIA = IN_CAT
          , ID_ORIGEN = IN_ORIG
          , ID_CRITICITAT = IN_CRIT
          , ID_ESTAT = IN_ESTAT
          , ID_REVISIOEXTERNA = IN_REVEXT
          , ID_ACCIO = IN_ACCIO
          , ID_REVISIO = IN_REV
          , ID_CONTROL = IN_CTRL
          , ID_PERSONARESPONSABLE = IN_PERSORESP
          , DFI = IN_DFI
        WHERE ID = IN_COD;
          
    END IF;

    VL_ERRCODE := 'OK';
    VL_ERRDESC := NULL;
    RSP := VL_ERRCODE;


    
    PRC_LOG(NOMPRC => 'P_RISC',
            PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_COD=' || IN_COD || ', ' || 'IN_DESC=' || IN_DESC || ', ' ||
                      'IN_CAT=' || IN_CAT || ', ' || 'IN_ORIG=' || IN_ORIG || ', ' || 'IN_CRIT=' || IN_CRIT || ', ' || 
                      'IN_ESTAT=' || IN_ESTAT || ', ' || 'IN_REVEXT=' || IN_REVEXT ||', ' || 'IN_ACCIO=' || IN_ACCIO || ', ' || 
                      'IN_REV=' || IN_REV || ', ' || 'IN_CTRL=' || IN_CTRL || ', ' || 'IN_PERSORESP=' || IN_PERSORESP,
            RSP    => VL_ERRCODE,
            RSPDESC=> VL_ERRDESC);

  EXCEPTION
    WHEN ERR_CTRL_PARAM THEN
      VL_ERRCODE := 'ERR_001';
      VL_ERRDESC := FNT_GET_ERRDESC(VL_ERRCODE);
      RSP := VL_ERRCODE;
        
      PRC_LOG(NOMPRC => 'P_RISC',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_COD=' || IN_COD || ', ' || 'IN_DESC=' || IN_DESC || ', ' ||
                        'IN_CAT=' || IN_CAT || ', ' || 'IN_ORIG=' || IN_ORIG || ', ' || 'IN_CRIT=' || IN_CRIT || ', ' || 
                        'IN_ESTAT=' || IN_ESTAT || ', ' || 'IN_REVEXT=' || IN_REVEXT ||', ' || 'IN_ACCIO=' || IN_ACCIO || ', ' || 
                        'IN_REV=' || IN_REV || ', ' || 'IN_CTRL=' || IN_CTRL || ', ' || 'IN_PERSORESP=' || IN_PERSORESP,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);

    WHEN ERR_CTRL_REGISTRE THEN
      VL_ERRCODE := 'ERR_006';
      VL_ERRDESC := REPLACE(FNT_GET_ERRDESC(VL_ERRCODE),'#N1#',VL_ERR_N1);
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_RISC',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_COD=' || IN_COD || ', ' || 'IN_DESC=' || IN_DESC || ', ' ||
                        'IN_CAT=' || IN_CAT || ', ' || 'IN_ORIG=' || IN_ORIG || ', ' || 'IN_CRIT=' || IN_CRIT || ', ' || 
                        'IN_ESTAT=' || IN_ESTAT || ', ' || 'IN_REVEXT=' || IN_REVEXT ||', ' || 'IN_ACCIO=' || IN_ACCIO || ', ' || 
                        'IN_REV=' || IN_REV || ', ' || 'IN_CTRL=' || IN_CTRL || ', ' || 'IN_PERSORESP=' || IN_PERSORESP,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);
              
    WHEN OTHERS THEN
      VL_ERRSQLCOD := SQLCODE;
      VL_ERRSQLMSG := SQLERRM;
      VL_ERRCODE := 'ERR_999';
      VL_ERRDESC := VL_ERRSQLCOD || ':' || VL_ERRSQLMSG;
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_RISC',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_COD=' || IN_COD || ', ' || 'IN_DESC=' || IN_DESC || ', ' ||
                        'IN_CAT=' || IN_CAT || ', ' || 'IN_ORIG=' || IN_ORIG || ', ' || 'IN_CRIT=' || IN_CRIT || ', ' || 
                        'IN_ESTAT=' || IN_ESTAT || ', ' || 'IN_REVEXT=' || IN_REVEXT ||', ' || 'IN_ACCIO=' || IN_ACCIO || ', ' || 
                        'IN_REV=' || IN_REV || ', ' || 'IN_CTRL=' || IN_CTRL || ', ' || 'IN_PERSORESP=' || IN_PERSORESP,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);
      

  END P_RISC;


  /* ******************
     Alta/Baixa/Modificació de AUDITORIAINTERNA 
     IN_ABM: A(lta), B(aixa), M(odificació)
     IN_COD: Codi del RISC, si nou registre = Null
     IN_DESC: Descripció del Risc
     RSP: Codi de sortida del procediment
  ****************** */
  PROCEDURE P_AUDITORIAINTERNA (IN_ABM VARCHAR2, 
                                IN_COD NUMBER,
                                IN_DESC VARCHAR2,
                                RSP OUT VARCHAR2) IS
                                
    VAUX DATE;
    VAUX_COUNT VARCHAR2(2000);
    
    VL_ERRSQLCOD NUMBER;
    VL_ERRSQLMSG VARCHAR2(255);
    
    ERR_CTRL_PARAM EXCEPTION; 
    ERR_CTRL_PARAM_NO_AUTORITZAT EXCEPTION;
    ERR_CTRL_ACCIO_NO_AUTORITZAT EXCEPTION;
    ERR_CTRL_JA_EXISTEIX EXCEPTION;
    
    VL_ERRCODE GR_LOG.RSP%TYPE;
    VL_ERRDESC GR_LOG.RSPDESC%TYPE;
    
  BEGIN
   
    IF IN_ABM NOT IN ('A','B','M') THEN
        RAISE ERR_CTRL_PARAM;
    ELSIF IN_ABM IN ('B') AND IN_COD IS NULL THEN
        RAISE ERR_CTRL_PARAM;
    ELSIF IN_ABM IN ('M') THEN
        -- NO ES PERMET LA MODIFICACIO
        RAISE ERR_CTRL_PARAM;
        
    END IF;
  
    -- Alta de registre
    IF IN_ABM = 'A' THEN
    
        -- Alta d'un registre nou
        IF IN_COD IS NULL THEN
            SELECT count(1)
            INTO VAUX_COUNT
            FROM GR_AUDITINTERNA
            WHERE UPPER(DESCRIPCIO) = UPPER(IN_DESC);
            
            IF VAUX_COUNT = 0 THEN
                INSERT INTO GR_AUDITINTERNA (ID, DESCRIPCIO, DBAIXA) VALUES 
                (SQ_AUDITINTERNA.NEXTVAL, IN_DESC, NULL);
            ELSE 
                -- no autortizat, la descripcio ja existeix
                RAISE ERR_CTRL_JA_EXISTEIX;
            END IF;
        
        -- Activació d'un registre previament desactivat
        ELSE 
            BEGIN
                SELECT DBAIXA
                INTO VAUX
                FROM GR_AUDITINTERNA
                WHERE ID = IN_COD
                AND DBAIXA IS NOT NULL
                FOR UPDATE;
                UPDATE GR_AUDITINTERNA SET DBAIXA = NULL WHERE ID = IN_COD;
            EXCEPTION
                WHEN NO_DATA_FOUND THEN
                    -- el registre ja esta donat d'alta o no existeix
                    RAISE ERR_CTRL_ACCIO_NO_AUTORITZAT;
            END;
        END IF;

    
    -- baixa de registre, bloqueig de registre per modificar
    ELSIF IN_ABM = 'B' THEN
        BEGIN
            SELECT DBAIXA
            INTO VAUX
            FROM GR_AUDITINTERNA
            WHERE ID = IN_COD
            AND DBAIXA IS NULL
            FOR UPDATE;
            UPDATE GR_AUDITINTERNA SET DBAIXA = SYSDATE WHERE ID = IN_COD;
        EXCEPTION 
            WHEN OTHERS THEN
                -- el registre no esta donat d'alta o no existeix
                RAISE ERR_CTRL_ACCIO_NO_AUTORITZAT;
        END;
        
    --ELSIF IN_ABM = 'M' THEN
    --    UPDATE GR_AUDITINTERNA SET  WHERE ID = IN_COD;

    END IF;   


    VL_ERRCODE := 'OK';
    VL_ERRDESC := NULL;
    RSP := VL_ERRCODE;
  
    
    PRC_LOG(NOMPRC => 'P_AUDITORIAINTERNA',
            PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_COD=' || IN_COD || ', ' || 'IN_DESC=' || IN_DESC ,
            RSP    => VL_ERRCODE,
            RSPDESC=> VL_ERRDESC);

  EXCEPTION
    WHEN ERR_CTRL_PARAM THEN
      VL_ERRCODE := 'ERR_001';
      VL_ERRDESC := FNT_GET_ERRDESC(VL_ERRCODE);
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_AUDITORIAINTERNA',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_COD=' || IN_COD || ', ' || 'IN_DESC=' || IN_DESC ,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);

    WHEN ERR_CTRL_PARAM_NO_AUTORITZAT THEN
      VL_ERRCODE := 'ERR_002';
      VL_ERRDESC := FNT_GET_ERRDESC(VL_ERRCODE);
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_AUDITORIAINTERNA',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_COD=' || IN_COD || ', ' || 'IN_DESC=' || IN_DESC ,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);
    
    WHEN ERR_CTRL_ACCIO_NO_AUTORITZAT THEN
      VL_ERRCODE := 'ERR_003';
      VL_ERRDESC := FNT_GET_ERRDESC(VL_ERRCODE);
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_AUDITORIAINTERNA',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_COD=' || IN_COD || ', ' || 'IN_DESC=' || IN_DESC ,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);
              
    WHEN ERR_CTRL_JA_EXISTEIX THEN
      VL_ERRCODE := 'ERR_004';
      VL_ERRDESC := FNT_GET_ERRDESC(VL_ERRCODE);
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_AUDITORIAINTERNA',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_COD=' || IN_COD || ', ' || 'IN_DESC=' || IN_DESC ,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);
    
    WHEN OTHERS THEN
      VL_ERRSQLCOD := SQLCODE;
      VL_ERRSQLMSG := SQLERRM;
      VL_ERRCODE := 'ERR_999';
      VL_ERRDESC := VL_ERRSQLCOD || ':' || VL_ERRSQLMSG;
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_AUDITORIAINTERNA',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_COD=' || IN_COD || ', ' || 'IN_DESC=' || IN_DESC ,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);

  
  
  
  END P_AUDITORIAINTERNA;
  


  /* ******************
     Es relacionen els equips auditoria intern amb les persones (formen l'equip) 
     Alta/Baixa/Modificació de dades taula GR_CONTROLINTERN 
     IN_ABM: A(lta), B(aixa), M(odificació)
     IN_IDAI: Codi del registre de auditoria interna
     IN_IDPER: Codi de la persona
     RSP: Codi de sortida del procediment
  ****************** */
  PROCEDURE P_AUDITORIAINTERNA_PERSONA (IN_ABM    VARCHAR2, 
                                        IN_IDAI   NUMBER,
                                        IN_IDPER  NUMBER,
                                        RSP OUT VARCHAR2) IS

    VAUX_ID    NUMBER;
    VAUX_BAIXA DATE;
    VAUX_EXISTEIX NUMBER;
    
    VL_ERRSQLCOD NUMBER;
    VL_ERRSQLMSG VARCHAR2(255);
    
    ERR_CTRL_PARAM EXCEPTION; 
    ERR_CTRL_PARAM_NO_AUTORITZAT EXCEPTION;
    ERR_CTRL_ACCIO_NO_AUTORITZAT EXCEPTION;
    ERR_CTRL_ACCIO_NO_AUTORITZAT_2 EXCEPTION;
    ERR_CTRL_REGISTRE EXCEPTION;
    ERR_CTRL_REGISTRE_CATEGORIA EXCEPTION;

    VL_ERR_N1 VARCHAR2(50);    
    VL_ERRCODE GR_LOG.RSP%TYPE;
    VL_ERRDESC GR_LOG.RSPDESC%TYPE;
                                             
  BEGIN
  
    IF IN_ABM NOT IN ('A','B','M') THEN
        RAISE ERR_CTRL_PARAM;
    ELSIF IN_IDAI IS NULL OR IN_IDPER IS NULL THEN
        RAISE ERR_CTRL_PARAM;
    ELSIF IN_ABM IN ('M') THEN
        RAISE ERR_CTRL_PARAM_NO_AUTORITZAT;
    END IF;
    
    BEGIN
        SELECT ID
        INTO VAUX_ID
        FROM GR_AUDITINTERNA
        WHERE ID = IN_IDAI
        AND DBAIXA IS NULL;
    EXCEPTION
        WHEN NO_DATA_FOUND THEN
            VL_ERR_N1 := 'GR_AUDITINTERNA';
            RAISE ERR_CTRL_REGISTRE;
    END;
    
    BEGIN
        SELECT ID
        INTO VAUX_ID
        FROM GR_PERSONA
        WHERE ID = IN_IDPER;
    EXCEPTION
        WHEN NO_DATA_FOUND THEN
            VL_ERR_N1 := 'GR_PERSONA';
            RAISE ERR_CTRL_REGISTRE;
    END;

    BEGIN    
        SELECT DBAIXA
        INTO VAUX_BAIXA
        FROM GR_AUDITINTERNA_EQUIP
        WHERE ID_AUDITINTERNA = IN_IDAI
        AND ID_PERSONA = IN_IDPER;
        VAUX_EXISTEIX := 1;
    EXCEPTION
        WHEN NO_DATA_FOUND THEN
            VAUX_EXISTEIX := 0;
    END;
           
    IF IN_ABM = 'A' THEN
    
        IF VAUX_EXISTEIX = 0 THEN
            INSERT INTO GR_AUDITINTERNA_EQUIP
            (ID_AUDITINTERNA, ID_PERSONA, DALTA, DBAIXA) VALUES
            (IN_IDAI, IN_IDPER, SYSDATE, NULL);
        ELSIF VAUX_EXISTEIX = 1 AND VAUX_BAIXA IS NOT NULL THEN
          -- ALTA REGISTRE
          UPDATE GR_AUDITINTERNA_EQUIP
          SET DBAIXA = NULL
          WHERE ID_AUDITINTERNA = IN_IDAI
          AND ID_PERSONA = IN_IDPER;
        ELSIF VAUX_EXISTEIX = 1 AND VAUX_BAIXA IS NULL THEN
            RAISE ERR_CTRL_ACCIO_NO_AUTORITZAT;
        END IF;
    
    ELSIF IN_ABM = 'B' THEN
        IF VAUX_EXISTEIX = 0 THEN
            RAISE ERR_CTRL_ACCIO_NO_AUTORITZAT_2;
        ELSIF VAUX_EXISTEIX = 1 AND VAUX_BAIXA IS NOT NULL THEN
            RAISE ERR_CTRL_ACCIO_NO_AUTORITZAT_2;
        ELSIF VAUX_EXISTEIX = 1 AND VAUX_BAIXA IS NULL THEN
          -- ALTA REGISTRE
          UPDATE GR_AUDITINTERNA_EQUIP
          SET DBAIXA = SYSDATE
          WHERE ID_AUDITINTERNA = IN_IDAI
          AND ID_PERSONA = IN_IDPER;
        END IF;
        
    END IF;
  
    VL_ERRCODE := 'OK';
    VL_ERRDESC := NULL;
    RSP := VL_ERRCODE;
        
    PRC_LOG(NOMPRC => 'P_AUDITORIAINTERNA_PERSONA',
            PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_IDAI=' || IN_IDAI || ', ' || 'IN_IDPER=' || IN_IDPER,
            RSP    => VL_ERRCODE,
            RSPDESC=> VL_ERRDESC);
  
  EXCEPTION
    WHEN ERR_CTRL_PARAM THEN
      VL_ERRCODE := 'ERR_001';
      VL_ERRDESC := FNT_GET_ERRDESC(VL_ERRCODE);
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_AUDITORIAINTERNA_PERSONA',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_IDAI=' || IN_IDAI || ', ' || 'IN_IDPER=' || IN_IDPER,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);

    WHEN ERR_CTRL_PARAM_NO_AUTORITZAT THEN
      VL_ERRCODE := 'ERR_002';
      VL_ERRDESC := FNT_GET_ERRDESC(VL_ERRCODE);
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_AUDITORIAINTERNA_PERSONA',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_IDAI=' || IN_IDAI || ', ' || 'IN_IDPER=' || IN_IDPER,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);
    
    WHEN ERR_CTRL_REGISTRE THEN
      VL_ERRCODE := 'ERR_006';
      VL_ERRDESC := REPLACE(FNT_GET_ERRDESC(VL_ERRCODE),'#N1#',VL_ERR_N1);
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_AUDITORIAINTERNA_PERSONA',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_IDAI=' || IN_IDAI || ', ' || 'IN_IDPER=' || IN_IDPER,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);

    WHEN ERR_CTRL_REGISTRE_CATEGORIA THEN
      VL_ERRCODE := 'ERR_007';
      VL_ERRDESC := FNT_GET_ERRDESC(VL_ERRCODE);
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_AUDITORIAINTERNA_PERSONA',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_IDAI=' || IN_IDAI || ', ' || 'IN_IDPER=' || IN_IDPER,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);
              
    WHEN ERR_CTRL_ACCIO_NO_AUTORITZAT THEN
      VL_ERRCODE := 'ERR_003';
      VL_ERRDESC := FNT_GET_ERRDESC(VL_ERRCODE);
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_AUDITORIAINTERNA_PERSONA',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_IDAI=' || IN_IDAI || ', ' || 'IN_IDPER=' || IN_IDPER,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);
              
    WHEN ERR_CTRL_ACCIO_NO_AUTORITZAT_2 THEN
      VL_ERRCODE := 'ERR_005';
      VL_ERRDESC := FNT_GET_ERRDESC(VL_ERRCODE);
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_AUDITORIAINTERNA_PERSONA',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_IDAI=' || IN_IDAI || ', ' || 'IN_IDPER=' || IN_IDPER,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);  
              
    WHEN OTHERS THEN
      VL_ERRSQLCOD := SQLCODE;
      VL_ERRSQLMSG := SQLERRM;
      VL_ERRCODE := 'ERR_999';
      VL_ERRDESC := VL_ERRSQLCOD || ':' || VL_ERRSQLMSG;
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_AUDITORIAINTERNA_PERSONA',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_IDAI=' || IN_IDAI || ', ' || 'IN_IDPER=' || IN_IDPER,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);
    

  
  END P_AUDITORIAINTERNA_PERSONA;



  /* ******************
     Alta/Baixa/Modificació de REVISIONS 
     IN_ABM: A(lta), B(aixa), M(odificació)
     IN_COD: Codi del REVISIO, si nou registre = Null
     IN_DINI: DATA INICI
     IN_DFI: DATA FI
     IN_DESC: Descripció del Risc
     IN_OBJECTIU: Objectiu de la revisió
     IN_DFI: DATA AVIS
     IN_DPTO: Id Departament implicat
     IN_AUDITINTERNA: Id Audit Interna
     IN_REVISIOEXTERNA: Id Revisio externa
     RSP: Codi de sortida del procediment
  ****************** */
  PROCEDURE P_REVISIO(IN_ABM VARCHAR2, 
                      IN_COD NUMBER,
                      IN_DINI DATE,
                      IN_DFI DATE,
                      IN_OBJECTIU VARCHAR2,
                      IN_DAVIS DATE,
                      IN_DPTO NUMBER,
                      IN_AUDITINTERNA NUMBER,
                      IN_REVISIOEXTERNA NUMBER,
                      RSP OUT VARCHAR2) IS

    VAUX_COUNT VARCHAR2(2000);
    
    VL_ERRSQLCOD NUMBER;
    VL_ERRSQLMSG VARCHAR2(255);
    VL_ERR_N1 VARCHAR2(50);
    
    ERR_CTRL_PARAM EXCEPTION; 
    ERR_CTRL_ACCIO_NO_AUTORITZAT EXCEPTION;
    ERR_CTRL_REGISTRE EXCEPTION;
    ERR_CTRL_REGISTRE_2 EXCEPTION;
    
    VL_ERRCODE GR_LOG.RSP%TYPE;
    VL_ERRDESC GR_LOG.RSPDESC%TYPE;
    
  BEGIN
   
    -- la baixa no esta permesa
    IF IN_ABM NOT IN ('A','M','B') THEN
        RAISE ERR_CTRL_PARAM;
    ELSIF IN_ABM IN ('M','B') AND IN_COD IS NULL THEN
        RAISE ERR_CTRL_PARAM;
    END IF;
  
    IF IN_ABM IN ('B') THEN
        SELECT COUNT(1) INTO VAUX_COUNT FROM GR_REVISIO WHERE ID = IN_COD AND DBAIXA IS NULL;
        IF VAUX_COUNT = 0 THEN
            VL_ERR_N1:= 'GR_REVISIO';
            RAISE ERR_CTRL_REGISTRE;
        END IF;
        
        UPDATE GR_REVISIO
        SET DBAIXA = SYSDATE
        WHERE ID = IN_COD;    
        
    ELSIF IN_ABM IN ('M') THEN
        SELECT COUNT(1) INTO VAUX_COUNT FROM GR_REVISIO WHERE ID = IN_COD AND DBAIXA IS NULL;
        IF VAUX_COUNT = 0 THEN
            RAISE ERR_CTRL_REGISTRE_2;
        END IF;
        
        SELECT COUNT(1) INTO VAUX_COUNT FROM GR_DEPARTAMENT WHERE ID = IN_DPTO AND DBAIXA IS NULL;
        IF VAUX_COUNT = 0 THEN
            VL_ERR_N1:= 'GR_DEPARTAMENT';
            RAISE ERR_CTRL_REGISTRE;
        END IF;

        SELECT COUNT(1) INTO VAUX_COUNT FROM GR_AUDITINTERNA WHERE ID = IN_AUDITINTERNA AND DBAIXA IS NULL;
        IF VAUX_COUNT = 0 THEN
            VL_ERR_N1:= 'GR_AUDITINTERNA';
            RAISE ERR_CTRL_REGISTRE;
        END IF;
        
        IF IN_REVISIOEXTERNA IS NOT NULL THEN
            SELECT COUNT(1) INTO VAUX_COUNT FROM GR_REVISIOEXTERNA WHERE ID = IN_REVISIOEXTERNA;
            IF VAUX_COUNT = 0 THEN
                VL_ERR_N1:= 'GR_REVISIOEXTERNA';
                RAISE ERR_CTRL_REGISTRE;
            END IF;
        END IF;
        
        UPDATE GR_REVISIO
        SET DINICI = DECODE(IN_DINI,NULL,DINICI,IN_DINI)
          , DFI = DECODE(IN_DFI,NULL,DFI,IN_DFI)
          , OBJECTIU = DECODE(IN_OBJECTIU,NULL,OBJECTIU,IN_OBJECTIU)
          , DAVIS = DECODE(IN_DAVIS,NULL,DAVIS,IN_DAVIS)
          , ID_DEPARTAMENTIMPLICAT = DECODE(IN_DPTO,NULL,ID_DEPARTAMENTIMPLICAT,IN_DPTO)
          , ID_AUDITINTERNA = DECODE(IN_AUDITINTERNA,NULL,ID_AUDITINTERNA,IN_AUDITINTERNA)
          , ID_REVISIOEXTERNA = DECODE(IN_REVISIOEXTERNA,NULL,ID_REVISIOEXTERNA,IN_REVISIOEXTERNA)
        WHERE ID = IN_COD;
        
    ELSIF IN_ABM IN ('A') THEN
        IF IN_COD IS NOT NULL THEN
            SELECT COUNT(1) INTO VAUX_COUNT FROM GR_REVISIO WHERE ID = IN_COD AND DBAIXA IS NOT NULL;
            IF VAUX_COUNT = 0 THEN
                RAISE ERR_CTRL_ACCIO_NO_AUTORITZAT;                
            END IF;
            
            UPDATE GR_REVISIO
            SET DBAIXA = NULL
            WHERE ID = IN_COD;
            
        ELSE 
            SELECT COUNT(1) INTO VAUX_COUNT FROM GR_DEPARTAMENT WHERE ID = IN_DPTO AND DBAIXA IS NULL;
            IF VAUX_COUNT = 0 THEN
                VL_ERR_N1:= 'GR_DEPARTAMENT';
                RAISE ERR_CTRL_REGISTRE;
            END IF;
    
            SELECT COUNT(1) INTO VAUX_COUNT FROM GR_AUDITINTERNA WHERE ID = IN_AUDITINTERNA AND DBAIXA IS NULL;
            IF VAUX_COUNT = 0 THEN
                VL_ERR_N1:= 'GR_AUDITINTERNA';
                RAISE ERR_CTRL_REGISTRE;
            END IF; 
            
            IF IN_REVISIOEXTERNA IS NOT NULL THEN
                SELECT COUNT(1) INTO VAUX_COUNT FROM GR_REVISIOEXTERNA WHERE ID = IN_REVISIOEXTERNA;
                IF VAUX_COUNT = 0 THEN
                    VL_ERR_N1:= 'GR_REVISIOEXTERNA';
                    RAISE ERR_CTRL_REGISTRE;
                END IF;
            END IF;
            
            INSERT INTO GR_REVISIO
            (ID, DINICI, DFI, OBJECTIU, DAVIS, ID_DEPARTAMENTIMPLICAT, ID_AUDITINTERNA, DBAIXA, ID_REVISIOEXTERNA) VALUES 
            (SQ_REVISIO.NEXTVAL, IN_DINI, IN_DFI, IN_OBJECTIU, IN_DAVIS, IN_DPTO, IN_AUDITINTERNA, NULL, IN_REVISIOEXTERNA);

        END IF;
        
    END IF;


    VL_ERRCODE := 'OK';
    VL_ERRDESC := NULL;
    RSP := VL_ERRCODE;
        
    
    PRC_LOG(NOMPRC => 'P_REVISIO',
            PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_COD=' || IN_COD || ', ' || 'IN_DINI=' || IN_DINI || ', ' ||
                      'IN_DFI=' || IN_DFI || ', ' || 'IN_OBJECTIU=' || IN_OBJECTIU || ', ' || 'IN_DAVIS=' || IN_DAVIS || ', ' || 
                      'IN_DPTO=' || IN_DPTO || ', ' || 'IN_AUDITINTERNA=' || IN_AUDITINTERNA,
            RSP    => VL_ERRCODE,
            RSPDESC=> VL_ERRDESC);

  EXCEPTION
    WHEN ERR_CTRL_PARAM THEN
      VL_ERRCODE := 'ERR_001';
      VL_ERRDESC := FNT_GET_ERRDESC(VL_ERRCODE);
      RSP := VL_ERRCODE;
        
      PRC_LOG(NOMPRC => 'P_REVISIO',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_COD=' || IN_COD || ', ' || 'IN_DINI=' || IN_DINI || ', ' ||
                      'IN_DFI=' || IN_DFI || ', ' || 'IN_OBJECTIU=' || IN_OBJECTIU || ', ' || 'IN_DAVIS=' || IN_DAVIS || ', ' || 
                      'IN_DPTO=' || IN_DPTO || ', ' || 'IN_AUDITINTERNA=' || IN_AUDITINTERNA,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);
              
    WHEN ERR_CTRL_ACCIO_NO_AUTORITZAT THEN
      VL_ERRCODE := 'ERR_003';
      VL_ERRDESC := FNT_GET_ERRDESC(VL_ERRCODE);
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_REVISIO',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_COD=' || IN_COD || ', ' || 'IN_DINI=' || IN_DINI || ', ' ||
                      'IN_DFI=' || IN_DFI || ', ' || 'IN_OBJECTIU=' || IN_OBJECTIU || ', ' || 'IN_DAVIS=' || IN_DAVIS || ', ' || 
                      'IN_DPTO=' || IN_DPTO || ', ' || 'IN_AUDITINTERNA=' || IN_AUDITINTERNA,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);

    WHEN ERR_CTRL_REGISTRE THEN
      VL_ERRCODE := 'ERR_006';
      VL_ERRDESC := REPLACE(FNT_GET_ERRDESC(VL_ERRCODE),'#N1#',VL_ERR_N1);
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_REVISIO',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_COD=' || IN_COD || ', ' || 'IN_DINI=' || IN_DINI || ', ' ||
                      'IN_DFI=' || IN_DFI || ', ' || 'IN_OBJECTIU=' || IN_OBJECTIU || ', ' || 'IN_DAVIS=' || IN_DAVIS || ', ' || 
                      'IN_DPTO=' || IN_DPTO || ', ' || 'IN_AUDITINTERNA=' || IN_AUDITINTERNA,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);
              
    WHEN ERR_CTRL_REGISTRE_2 THEN
      VL_ERRCODE := 'ERR_008';
      VL_ERRDESC := FNT_GET_ERRDESC(VL_ERRCODE);
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_REVISIO',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_COD=' || IN_COD || ', ' || 'IN_DINI=' || IN_DINI || ', ' ||
                      'IN_DFI=' || IN_DFI || ', ' || 'IN_OBJECTIU=' || IN_OBJECTIU || ', ' || 'IN_DAVIS=' || IN_DAVIS || ', ' || 
                      'IN_DPTO=' || IN_DPTO || ', ' || 'IN_AUDITINTERNA=' || IN_AUDITINTERNA,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);
              
    WHEN OTHERS THEN
      VL_ERRSQLCOD := SQLCODE;
      VL_ERRSQLMSG := SQLERRM;
      VL_ERRCODE := 'ERR_999';
      VL_ERRDESC := VL_ERRSQLCOD || ':' || VL_ERRSQLMSG;
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_REVISIO',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_COD=' || IN_COD || ', ' || 'IN_DINI=' || IN_DINI || ', ' ||
                      'IN_DFI=' || IN_DFI || ', ' || 'IN_OBJECTIU=' || IN_OBJECTIU || ', ' || 'IN_DAVIS=' || IN_DAVIS || ', ' || 
                      'IN_DPTO=' || IN_DPTO || ', ' || 'IN_AUDITINTERNA=' || IN_AUDITINTERNA,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);


  END P_REVISIO;




  /* ******************
     Es relacionen les revisions amb les persones, forman les persones implicades de la revisió
     Alta/Baixa/Modificació de dades taula GR_CONTROLINTERN 
     IN_ABM: A(lta), B(aixa), M(odificació)
     IN_IDAI: Codi del registre de auditoria interna
     IN_IDPER: Codi de la persona
     RSP: Codi de sortida del procediment
  ****************** */
  PROCEDURE P_REVISIO_PERSONA (IN_ABM    VARCHAR2, 
                               IN_IDREV  NUMBER,
                               IN_IDPER  NUMBER,
                               RSP OUT VARCHAR2) IS
                           
    VAUX DATE;
    VAUX_COUNT NUMBER;
    
    VL_ERRSQLCOD NUMBER;
    VL_ERRSQLMSG VARCHAR2(255);
    
    ERR_CTRL_PARAM EXCEPTION; 
    ERR_CTRL_PARAM_NO_AUTORITZAT EXCEPTION;
    ERR_CTRL_ACCIO_NO_AUTORITZAT EXCEPTION;
    ERR_CTRL_ACCIO_NO_AUTORITZAT_2 EXCEPTION;
    
    VL_ERRCODE GR_LOG.RSP%TYPE;
    VL_ERRDESC GR_LOG.RSPDESC%TYPE;
    
  BEGIN
   
    IF IN_ABM NOT IN ('A','B','M') THEN
        RAISE ERR_CTRL_PARAM;
    ELSIF IN_IDREV IS NULL OR IN_IDPER IS NULL THEN
        RAISE ERR_CTRL_PARAM;
    ELSIF IN_ABM IN ('M') THEN
        RAISE ERR_CTRL_PARAM_NO_AUTORITZAT;
    END IF;
  

    IF IN_ABM = 'A' THEN

        BEGIN
            SELECT DBAIXA
            INTO VAUX
            FROM GR_REVISIO_PERSONA
            WHERE ID_REVISIO = IN_IDREV
            AND ID_PERSONA = IN_IDPER;
            
            IF VAUX IS NOT NULL THEN
                UPDATE GR_REVISIO_PERSONA
                SET DBAIXA = NULL
                WHERE ID_REVISIO = IN_IDREV
                AND ID_PERSONA = IN_IDPER;
            ELSE
                RAISE ERR_CTRL_ACCIO_NO_AUTORITZAT;
            END IF;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
                INSERT INTO GR_REVISIO_PERSONA (ID_REVISIO, ID_PERSONA, DALTA) VALUES 
                (IN_IDREV, IN_IDPER, SYSDATE);
        END;
        
    ELSIF IN_ABM = 'B' THEN
        BEGIN
            SELECT DBAIXA
            INTO VAUX
            FROM GR_REVISIO_PERSONA
            WHERE ID_REVISIO = IN_IDREV
            AND ID_PERSONA = IN_IDPER
            AND DBAIXA IS NULL;
            
            UPDATE GR_REVISIO_PERSONA
            SET DBAIXA = SYSDATE
            WHERE ID_REVISIO = IN_IDREV
            AND ID_PERSONA = IN_IDPER;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
                RAISE ERR_CTRL_ACCIO_NO_AUTORITZAT_2;
        END;
        
    END IF;

        
    VL_ERRCODE := 'OK';
    VL_ERRDESC := NULL;
    RSP := VL_ERRCODE;
      
    PRC_LOG(NOMPRC => 'P_REVISIO_PERSONA',
            PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_IDREV=' || IN_IDREV || ', ' || 'IN_IDPER=' || IN_IDPER,
            RSP    => VL_ERRCODE,
            RSPDESC=> VL_ERRDESC);

  EXCEPTION
    WHEN ERR_CTRL_PARAM THEN
      VL_ERRCODE := 'ERR_001';
      VL_ERRDESC := FNT_GET_ERRDESC(VL_ERRCODE);
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_REVISIO_PERSONA',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_IDREV=' || IN_IDREV || ', ' || 'IN_IDPER=' || IN_IDPER,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);

    WHEN ERR_CTRL_PARAM_NO_AUTORITZAT THEN
      VL_ERRCODE := 'ERR_002';
      VL_ERRDESC := FNT_GET_ERRDESC(VL_ERRCODE);
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_REVISIO_PERSONA',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_IDREV=' || IN_IDREV || ', ' || 'IN_IDPER=' || IN_IDPER,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);
    
    WHEN ERR_CTRL_ACCIO_NO_AUTORITZAT THEN
      VL_ERRCODE := 'ERR_003';
      VL_ERRDESC := FNT_GET_ERRDESC(VL_ERRCODE);
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_REVISIO_PERSONA',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_IDREV=' || IN_IDREV || ', ' || 'IN_IDPER=' || IN_IDPER,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);

    WHEN ERR_CTRL_ACCIO_NO_AUTORITZAT_2 THEN
      VL_ERRCODE := 'ERR_005';
      VL_ERRDESC := FNT_GET_ERRDESC(VL_ERRCODE);
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_REVISIO_PERSONA',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_IDREV=' || IN_IDREV || ', ' || 'IN_IDPER=' || IN_IDPER,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC); 

    WHEN OTHERS THEN
      VL_ERRSQLCOD := SQLCODE;
      VL_ERRSQLMSG := SQLERRM;
      VL_ERRCODE := 'ERR_999';
      VL_ERRDESC := VL_ERRSQLCOD || ':' || VL_ERRSQLMSG;
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_REVISIO_PERSONA',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_IDREV=' || IN_IDREV || ', ' || 'IN_IDPER=' || IN_IDPER,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);
    

  END P_REVISIO_PERSONA;


  /* ******************
     Alta/Baixa/Modificació de ACCIONS sobre els riscos 
     IN_ABM: A(lta), B(aixa), M(odificació)
     IN_COD: Codi de la ACCIO, si nou registre = Null
     IN_DESC: Descripció del Risc
     RSP: Codi de sortida del procediment
  ****************** */
  PROCEDURE P_ACCIO (IN_ABM VARCHAR2, 
                     IN_COD NUMBER,
                     IN_DESC VARCHAR2,
                     IN_NOM VARCHAR2,
                     IN_DCREA DATE,
                     IN_DTANCA DATE,
                     IN_IDESTATMITIGACIO NUMBER,
                     IN_IDRESPONSABLE NUMBER,
                     IN_IDRISC NUMBER,
                     RSP OUT VARCHAR2) IS
                                
    VAUX DATE;
    VAUX_COUNT VARCHAR2(2000);
    
    VL_ERR_N1 VARCHAR2(50);    
    VL_ERRSQLCOD NUMBER;
    VL_ERRSQLMSG VARCHAR2(255);
    
    ERR_CTRL_PARAM EXCEPTION; 
    ERR_CTRL_PARAM_NO_AUTORITZAT EXCEPTION;
    ERR_CTRL_ACCIO_NO_AUTORITZAT EXCEPTION;
    ERR_CTRL_ACCIO_NO_AUTORITZAT_2 EXCEPTION;
    ERR_CTRL_JA_EXISTEIX EXCEPTION;
    ERR_CTRL_REGISTRE EXCEPTION;
    ERR_CTRL_REGISTRE_2 EXCEPTION;
    
    VL_ERRCODE GR_LOG.RSP%TYPE;
    VL_ERRDESC GR_LOG.RSPDESC%TYPE;
    
  BEGIN
   
    IF IN_ABM NOT IN ('A','B','M') THEN
        RAISE ERR_CTRL_PARAM;
    ELSIF IN_ABM IN ('B','M') AND IN_COD IS NULL THEN
        RAISE ERR_CTRL_PARAM;
    END IF;    
    
    SELECT COUNT(1) INTO VAUX_COUNT FROM GR_ESTATMITIGACIO WHERE ID = IN_IDESTATMITIGACIO AND DBAIXA IS NULL;
    IF VAUX_COUNT = 0 THEN
        VL_ERRCODE:= 'GR_ESTATMITIGACIO';
        RAISE ERR_CTRL_ACCIO_NO_AUTORITZAT;                
    END IF;
    
    IF IN_IDRESPONSABLE IS NOT NULL THEN 
        SELECT COUNT(1) INTO VAUX_COUNT FROM GR_PERSONA WHERE ID = IN_IDRESPONSABLE;
        IF VAUX_COUNT = 0 THEN
            VL_ERRCODE:= 'GR_PERSONA';
            RAISE ERR_CTRL_ACCIO_NO_AUTORITZAT;                
        END IF;
    END IF;
  
    SELECT COUNT(1) INTO VAUX_COUNT FROM GR_RISC WHERE ID = IN_IDRISC;
    IF VAUX_COUNT = 0 THEN
        VL_ERRCODE:= 'GR_RISC';
        RAISE ERR_CTRL_ACCIO_NO_AUTORITZAT;                
    END IF;
  
    -- Alta de registre
    IF IN_ABM = 'A' THEN
        -- Alta d'un registre nou
        IF IN_COD IS NULL THEN
            INSERT INTO GR_ACCIO (ID, DESCRIPCIO, DCREACIO, DBAIXA, NOM, ID_ESTATMITIGACIO, ID_PERSONARESPONSABLE, ID_RISC, DTANCAMENT) VALUES 
            (SQ_ACCIO.NEXTVAL, IN_DESC, IN_DCREA, NULL, IN_NOM, IN_IDESTATMITIGACIO, IN_IDRESPONSABLE, IN_IDRISC, IN_DTANCA);
        
        -- Activació d'un registre previament desactivat
        ELSE 
            SELECT COUNT(1)
            INTO VAUX_COUNT
            FROM GR_ACCIO
            WHERE ID = IN_COD
            AND DBAIXA IS NOT NULL;                
            IF VAUX_COUNT = 0 THEN
                RAISE ERR_CTRL_ACCIO_NO_AUTORITZAT;
            END IF;                
            UPDATE GR_ACCIO SET DBAIXA = NULL WHERE ID = IN_COD;
        END IF;
    
    -- baixa de registre, 
    ELSIF IN_ABM = 'B' THEN
        SELECT COUNT(1)
        INTO VAUX_COUNT
        FROM GR_ACCIO
        WHERE ID = IN_COD
        AND DBAIXA IS NULL;
        IF VAUX_COUNT = 0 THEN
            RAISE ERR_CTRL_ACCIO_NO_AUTORITZAT_2;
        END IF;                
        
        UPDATE GR_ACCIO SET DBAIXA = SYSDATE WHERE ID = IN_COD;
        
    -- baixa de registre, 
    ELSIF IN_ABM = 'M' THEN
        SELECT COUNT(1)
        INTO VAUX_COUNT
        FROM GR_ACCIO
        WHERE ID = IN_COD
        AND DBAIXA IS NULL;                
        IF VAUX_COUNT = 0 THEN
            RAISE ERR_CTRL_REGISTRE_2;
        END IF; 
        
        UPDATE GR_ACCIO
        SET DESCRIPCIO = IN_DESC
          , NOM = IN_NOM
          , ID_ESTATMITIGACIO = IN_IDESTATMITIGACIO
          , ID_PERSONARESPONSABLE = IN_IDRESPONSABLE
          , ID_RISC = IN_IDRISC
          , DCREACIO = IN_DCREA
          , DTANCAMENT = IN_DTANCA
        WHERE ID = IN_COD;

    END IF;   

    VL_ERRCODE := 'OK';
    VL_ERRDESC := NULL;
    RSP := VL_ERRCODE;
        
    PRC_LOG(NOMPRC => 'P_ACCIO',
            PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_COD=' || IN_COD || ', ' || 'IN_DESC=' || IN_DESC || ', ' || 
                      'IN_NOM=' || IN_NOM || ', ' || 'IN_DCREA=' || TO_CHAR(IN_DCREA,'DD/MM/YYYY') || ', ' ||
                      'IN_DTANCA=' || TO_CHAR(IN_DTANCA,'DD/MM/YYYY') || ', ' ||
                      'IN_IDESTATMITIGACIO=' || IN_IDESTATMITIGACIO || ', ' || 
                      'IN_IDRESPONSABLE=' || IN_IDRESPONSABLE || ', ' || 'IN_IDRISC=' || IN_IDRISC,
            RSP    => VL_ERRCODE,
            RSPDESC=> VL_ERRDESC);

  EXCEPTION
    WHEN ERR_CTRL_PARAM THEN
      VL_ERRCODE := 'ERR_001';
      VL_ERRDESC := FNT_GET_ERRDESC(VL_ERRCODE);
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_ACCIO',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_COD=' || IN_COD || ', ' || 'IN_DESC=' || IN_DESC || ', ' || 
                        'IN_NOM=' || IN_NOM || ', ' || 'IN_DCREA=' || TO_CHAR(IN_DCREA,'DD/MM/YYYY') || ', ' ||
                        'IN_DTANCA=' || TO_CHAR(IN_DTANCA,'DD/MM/YYYY') || ', ' ||
                        'IN_IDESTATMITIGACIO=' || IN_IDESTATMITIGACIO || ', ' || 
                        'IN_IDRESPONSABLE=' || IN_IDRESPONSABLE || ', ' || 'IN_IDRISC=' || IN_IDRISC,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);

    WHEN ERR_CTRL_PARAM_NO_AUTORITZAT THEN
      VL_ERRCODE := 'ERR_002';
      VL_ERRDESC := FNT_GET_ERRDESC(VL_ERRCODE);
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_ACCIO',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_COD=' || IN_COD || ', ' || 'IN_DESC=' || IN_DESC || ', ' || 
                        'IN_NOM=' || IN_NOM || ', ' || 'IN_DCREA=' || TO_CHAR(IN_DCREA,'DD/MM/YYYY') || ', ' ||
                        'IN_DTANCA=' || TO_CHAR(IN_DTANCA,'DD/MM/YYYY') || ', ' ||
                        'IN_IDESTATMITIGACIO=' || IN_IDESTATMITIGACIO || ', ' || 
                        'IN_IDRESPONSABLE=' || IN_IDRESPONSABLE || ', ' || 'IN_IDRISC=' || IN_IDRISC,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);
    
    WHEN ERR_CTRL_ACCIO_NO_AUTORITZAT THEN
      VL_ERRCODE := 'ERR_003';
      VL_ERRDESC := FNT_GET_ERRDESC(VL_ERRCODE);
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_ACCIO',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_COD=' || IN_COD || ', ' || 'IN_DESC=' || IN_DESC || ', ' || 
                        'IN_NOM=' || IN_NOM || ', ' || 'IN_DCREA=' || TO_CHAR(IN_DCREA,'DD/MM/YYYY') || ', ' ||
                        'IN_DTANCA=' || TO_CHAR(IN_DTANCA,'DD/MM/YYYY') || ', ' ||
                        'IN_IDESTATMITIGACIO=' || IN_IDESTATMITIGACIO || ', ' || 
                        'IN_IDRESPONSABLE=' || IN_IDRESPONSABLE || ', ' || 'IN_IDRISC=' || IN_IDRISC,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);
              
    WHEN ERR_CTRL_JA_EXISTEIX THEN
      VL_ERRCODE := 'ERR_004';
      VL_ERRDESC := FNT_GET_ERRDESC(VL_ERRCODE);
      RSP := VL_ERRCODE;
      PRC_LOG(NOMPRC => 'P_ACCIO',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_COD=' || IN_COD || ', ' || 'IN_DESC=' || IN_DESC || ', ' || 
                        'IN_NOM=' || IN_NOM || ', ' || 'IN_DCREA=' || TO_CHAR(IN_DCREA,'DD/MM/YYYY') || ', ' ||
                        'IN_DTANCA=' || TO_CHAR(IN_DTANCA,'DD/MM/YYYY') || ', ' ||
                        'IN_IDESTATMITIGACIO=' || IN_IDESTATMITIGACIO || ', ' || 
                        'IN_IDRESPONSABLE=' || IN_IDRESPONSABLE || ', ' || 'IN_IDRISC=' || IN_IDRISC,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);
    
    WHEN ERR_CTRL_ACCIO_NO_AUTORITZAT_2 THEN
      VL_ERRCODE := 'ERR_005';
      VL_ERRDESC := FNT_GET_ERRDESC(VL_ERRCODE);
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_ACCIO',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_COD=' || IN_COD || ', ' || 'IN_DESC=' || IN_DESC || ', ' || 
                        'IN_NOM=' || IN_NOM || ', ' || 'IN_DCREA=' || TO_CHAR(IN_DCREA,'DD/MM/YYYY') || ', ' ||
                        'IN_DTANCA=' || TO_CHAR(IN_DTANCA,'DD/MM/YYYY') || ', ' ||
                        'IN_IDESTATMITIGACIO=' || IN_IDESTATMITIGACIO || ', ' || 
                        'IN_IDRESPONSABLE=' || IN_IDRESPONSABLE || ', ' || 'IN_IDRISC=' || IN_IDRISC,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC); 
    
    WHEN ERR_CTRL_REGISTRE THEN
      VL_ERRCODE := 'ERR_006';
      VL_ERRDESC := REPLACE(FNT_GET_ERRDESC(VL_ERRCODE),'#N1#',VL_ERR_N1);
      RSP := VL_ERRCODE;
      PRC_LOG(NOMPRC => 'P_ACCIO',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_COD=' || IN_COD || ', ' || 'IN_DESC=' || IN_DESC || ', ' || 
                        'IN_NOM=' || IN_NOM || ', ' || 'IN_DCREA=' || TO_CHAR(IN_DCREA,'DD/MM/YYYY') || ', ' ||
                        'IN_DTANCA=' || TO_CHAR(IN_DTANCA,'DD/MM/YYYY') || ', ' ||
                        'IN_IDESTATMITIGACIO=' || IN_IDESTATMITIGACIO || ', ' || 
                        'IN_IDRESPONSABLE=' || IN_IDRESPONSABLE || ', ' || 'IN_IDRISC=' || IN_IDRISC,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);
    
    WHEN ERR_CTRL_REGISTRE_2 THEN
      VL_ERRCODE := 'ERR_008';
      VL_ERRDESC := FNT_GET_ERRDESC(VL_ERRCODE);
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_ACCIO',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_COD=' || IN_COD || ', ' || 'IN_DESC=' || IN_DESC || ', ' || 
                        'IN_NOM=' || IN_NOM || ', ' || 'IN_DCREA=' || TO_CHAR(IN_DCREA,'DD/MM/YYYY') || ', ' ||
                        'IN_DTANCA=' || TO_CHAR(IN_DTANCA,'DD/MM/YYYY') || ', ' ||
                        'IN_IDESTATMITIGACIO=' || IN_IDESTATMITIGACIO || ', ' || 
                        'IN_IDRESPONSABLE=' || IN_IDRESPONSABLE || ', ' || 'IN_IDRISC=' || IN_IDRISC,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);
    
    WHEN OTHERS THEN
      VL_ERRSQLCOD := SQLCODE;
      VL_ERRSQLMSG := SQLERRM;
      VL_ERRCODE := 'ERR_999';
      VL_ERRDESC := VL_ERRSQLCOD || ':' || VL_ERRSQLMSG;
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_ACCIO',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_COD=' || IN_COD || ', ' || 'IN_DESC=' || IN_DESC || ', ' || 
                        'IN_NOM=' || IN_NOM || ', ' || 'IN_DCREA=' || TO_CHAR(IN_DCREA,'DD/MM/YYYY') || ', ' ||
                        'IN_DTANCA=' || TO_CHAR(IN_DTANCA,'DD/MM/YYYY') || ', ' ||
                        'IN_IDESTATMITIGACIO=' || IN_IDESTATMITIGACIO || ', ' || 
                        'IN_IDRESPONSABLE=' || IN_IDRESPONSABLE || ', ' || 'IN_IDRISC=' || IN_IDRISC,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);

  
  
  
  END P_ACCIO;
  

  /* ******************
     Alta/Baixa/Modificació de P_AUDITEXTERNA 
     IN_ABM: A(lta), B(aixa), M(odificació)
     IN_COD: Codi de la empresa d'auditoria externa
     IN_NOMEMPRESA: Nom de l'empresa
     RSP: Codi de sortida del procediment
  ****************** */
  PROCEDURE P_AUDITEXTERNA (IN_ABM VARCHAR2, 
                            IN_COD NUMBER,
                            IN_NOMEMPRESA VARCHAR2,
                            RSP OUT VARCHAR2) IS
                                
    VAUX DATE;
    VAUX_COUNT VARCHAR2(2000);
    
    VL_ERR_N1 VARCHAR2(50);    
    VL_ERRSQLCOD NUMBER;
    VL_ERRSQLMSG VARCHAR2(255);
    
    ERR_CTRL_PARAM EXCEPTION; 
    ERR_CTRL_PARAM_NO_AUTORITZAT EXCEPTION;
    ERR_CTRL_ACCIO_NO_AUTORITZAT EXCEPTION;
    ERR_CTRL_ACCIO_NO_AUTORITZAT_2 EXCEPTION;
    ERR_CTRL_JA_EXISTEIX EXCEPTION;
    ERR_CTRL_REGISTRE EXCEPTION;
    ERR_CTRL_REGISTRE_2 EXCEPTION;
    
    VL_ERRCODE GR_LOG.RSP%TYPE;
    VL_ERRDESC GR_LOG.RSPDESC%TYPE;
    
  BEGIN
   
    IF IN_ABM NOT IN ('A','B','M') THEN
        RAISE ERR_CTRL_PARAM;
    ELSIF IN_ABM IN ('B','M') AND IN_COD IS NULL THEN
        RAISE ERR_CTRL_PARAM;
    END IF;    
    
    BEGIN
      SELECT DBAIXA
      INTO VAUX 
      FROM GR_AUDITEXTERNA 
      WHERE ID = IN_COD;
      VAUX_COUNT := 1;
    EXCEPTION
      WHEN NO_DATA_FOUND THEN
        VAUX_COUNT := 0;
      WHEN OTHERS THEN
        VAUX_COUNT := 0;
    END;
        
    
    -- Alta de registre
    IF IN_ABM = 'A' THEN
        -- Alta d'un registre nou
        IF IN_COD IS NULL THEN
            INSERT INTO GR_AUDITEXTERNA (ID, NOMEMPRESA, DBAIXA) VALUES 
            (SQ_AUDITEXTERNA.NEXTVAL, IN_NOMEMPRESA, NULL);
        
        -- Activació d'un registre previament desactivat
        ELSIF VAUX_COUNT = 1 AND VAUX IS NOT NULL THEN
            UPDATE GR_AUDITEXTERNA SET DBAIXA = NULL WHERE ID = IN_COD;
        
        ELSE 
            RAISE ERR_CTRL_ACCIO_NO_AUTORITZAT; 

        END IF;
    
    -- baixa de registre, 
    ELSIF IN_ABM = 'B' THEN
        IF VAUX_COUNT = 1 AND VAUX IS NULL THEN
            UPDATE GR_AUDITEXTERNA SET DBAIXA = SYSDATE WHERE ID = IN_COD;
        ELSE 
            RAISE ERR_CTRL_ACCIO_NO_AUTORITZAT; 
        END IF;

    ELSIF IN_ABM = 'M' THEN
        IF VAUX_COUNT = 1 AND VAUX IS NULL THEN
            UPDATE GR_AUDITEXTERNA SET NOMEMPRESA = IN_NOMEMPRESA WHERE ID = IN_COD;
        ELSE 
            RAISE ERR_CTRL_REGISTRE_2; 
        END IF;

    END IF;   

    VL_ERRCODE := 'OK';
    VL_ERRDESC := NULL;
    RSP := VL_ERRCODE;
        
    PRC_LOG(NOMPRC => 'P_AUDITEXTERNA',
            PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_COD=' || IN_COD || ', ' || 'IN_NOMEMPRESA=' || IN_NOMEMPRESA,
            RSP    => VL_ERRCODE,
            RSPDESC=> VL_ERRDESC);

  EXCEPTION
    WHEN ERR_CTRL_PARAM THEN
      VL_ERRCODE := 'ERR_001';
      VL_ERRDESC := FNT_GET_ERRDESC(VL_ERRCODE);
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_AUDITEXTERNA',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_COD=' || IN_COD || ', ' || 'IN_NOMEMPRESA=' || IN_NOMEMPRESA,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);

    WHEN ERR_CTRL_PARAM_NO_AUTORITZAT THEN
      VL_ERRCODE := 'ERR_002';
      VL_ERRDESC := FNT_GET_ERRDESC(VL_ERRCODE);
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_AUDITEXTERNA',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_COD=' || IN_COD || ', ' || 'IN_NOMEMPRESA=' || IN_NOMEMPRESA,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);
    
    WHEN ERR_CTRL_ACCIO_NO_AUTORITZAT THEN
      VL_ERRCODE := 'ERR_003';
      VL_ERRDESC := FNT_GET_ERRDESC(VL_ERRCODE);
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_AUDITEXTERNA',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_COD=' || IN_COD || ', ' || 'IN_NOMEMPRESA=' || IN_NOMEMPRESA,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);
              
    WHEN ERR_CTRL_JA_EXISTEIX THEN
      VL_ERRCODE := 'ERR_004';
      VL_ERRDESC := FNT_GET_ERRDESC(VL_ERRCODE);
      RSP := VL_ERRCODE;
      PRC_LOG(NOMPRC => 'P_AUDITEXTERNA',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_COD=' || IN_COD || ', ' || 'IN_NOMEMPRESA=' || IN_NOMEMPRESA,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);
    
    WHEN ERR_CTRL_ACCIO_NO_AUTORITZAT_2 THEN
      VL_ERRCODE := 'ERR_005';
      VL_ERRDESC := FNT_GET_ERRDESC(VL_ERRCODE);
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_AUDITEXTERNA',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_COD=' || IN_COD || ', ' || 'IN_NOMEMPRESA=' || IN_NOMEMPRESA,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC); 
    
    WHEN ERR_CTRL_REGISTRE THEN
      VL_ERRCODE := 'ERR_006';
      VL_ERRDESC := REPLACE(FNT_GET_ERRDESC(VL_ERRCODE),'#N1#',VL_ERR_N1);
      RSP := VL_ERRCODE;
      PRC_LOG(NOMPRC => 'P_AUDITEXTERNA',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_COD=' || IN_COD || ', ' || 'IN_NOMEMPRESA=' || IN_NOMEMPRESA,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);
    
    WHEN ERR_CTRL_REGISTRE_2 THEN
      VL_ERRCODE := 'ERR_008';
      VL_ERRDESC := FNT_GET_ERRDESC(VL_ERRCODE);
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_AUDITEXTERNA',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_COD=' || IN_COD || ', ' || 'IN_NOMEMPRESA=' || IN_NOMEMPRESA,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);
    
    WHEN OTHERS THEN
      VL_ERRSQLCOD := SQLCODE;
      VL_ERRSQLMSG := SQLERRM;
      VL_ERRCODE := 'ERR_999';
      VL_ERRDESC := VL_ERRSQLCOD || ':' || VL_ERRSQLMSG;
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_AUDITEXTERNA',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_COD=' || IN_COD || ', ' || 'IN_NOMEMPRESA=' || IN_NOMEMPRESA,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);

  
  
  END P_AUDITEXTERNA;
  

  /* ******************
     Alta/Baixa/Modificació de dades taula GR_DEPARTAMENT 
     IN_ABM: A(lta), B(aixa), M(odificació)
     IN_COD: Codi del registre, si nou registre = Null
     IN_NOM, IN_COG1, IN_COG2, IN_DOC: Dades de la persona
     IN_IDDEPT: Departament a la que pertany la persona
     RSP: Codi de sortida del procediment
  ****************** */
  PROCEDURE P_PERSONAEXTERNA (IN_ABM  VARCHAR2, 
                       IN_COD  NUMBER,
                       IN_NOM  VARCHAR2,
                       IN_COG1 VARCHAR2,
                       IN_COG2 VARCHAR2,
                       IN_DOC VARCHAR2,
                       IN_IDAUDIT NUMBER,
                       RSP OUT VARCHAR2) IS
                           
    VAUX DATE;
    VAUX_COUNT VARCHAR2(2000);
    
    VL_ERRSQLCOD NUMBER;
    VL_ERRSQLMSG VARCHAR2(255);
    
    ERR_CTRL_PARAM EXCEPTION; 
    ERR_CTRL_PARAM_NO_AUTORITZAT EXCEPTION;
    ERR_CTRL_ACCIO_NO_AUTORITZAT EXCEPTION;
    
    VL_ERRCODE GR_LOG.RSP%TYPE;
    VL_ERRDESC GR_LOG.RSPDESC%TYPE;
    
  BEGIN
   
    IF IN_ABM NOT IN ('A','B','M') THEN
        RAISE ERR_CTRL_PARAM;
    ELSIF IN_ABM IN ('B') AND IN_COD IS NULL THEN
        RAISE ERR_CTRL_PARAM;
    END IF;
  
    -- Alta de registre
    IF IN_ABM = 'A' THEN
    
        -- Alta d'un registre nou
        IF IN_COD IS NULL THEN
            -- es comproba que no existeixi el DOC
            SELECT count(1)
            INTO VAUX_COUNT
            FROM GR_PERSONAEXTERNA
            WHERE UPPER(DOCUMENT) = UPPER(IN_DOC);
            
            IF VAUX_COUNT = 0 THEN
                INSERT INTO GR_PERSONAEXTERNA (ID, NOM, COGNOM1, COGNOM2, DOCUMENT, DBAIXA, ID_AUDITEXTERNA) VALUES 
                (SQ_PERSONAEXTERNA.NEXTVAL, IN_NOM, IN_COG1, IN_COG2, IN_DOC, NULL, IN_IDAUDIT);
            ELSE 
                -- no autortizat, la descripcio ja existeix
                RAISE ERR_CTRL_ACCIO_NO_AUTORITZAT;
            END IF;
        
        -- Activació d'un registre previament desactivat
        ELSE 
            BEGIN
                SELECT DBAIXA
                INTO VAUX
                FROM GR_PERSONAEXTERNA
                WHERE ID = IN_COD
                AND DBAIXA IS NOT NULL
                FOR UPDATE;
                UPDATE GR_PERSONAEXTERNA SET DBAIXA = NULL WHERE ID = IN_COD;
            EXCEPTION
                WHEN NO_DATA_FOUND THEN
                    -- el registre ja esta donat d'alta o no existeix
                    RAISE ERR_CTRL_ACCIO_NO_AUTORITZAT;
            END;
        END IF;

    
    -- baixa de registre, bloqueig de registre per modificar
    ELSIF IN_ABM = 'B' THEN
        BEGIN
            SELECT DBAIXA
            INTO VAUX
            FROM GR_PERSONAEXTERNA
            WHERE ID = IN_COD
            AND DBAIXA IS NULL
            FOR UPDATE;
            UPDATE GR_PERSONAEXTERNA SET DBAIXA = SYSDATE WHERE ID = IN_COD;
        EXCEPTION 
            WHEN OTHERS THEN
                -- el registre no esta donat d'alta o no existeix
                RAISE ERR_CTRL_ACCIO_NO_AUTORITZAT;
        END;

    -- modificació del registre, es permet modificar les dades personals i el departament assignat
    ELSIF IN_ABM = 'M' THEN
    
        SELECT count(1)
        INTO VAUX_COUNT
        FROM GR_PERSONAEXTERNA
        WHERE ID = IN_COD;
        
        IF VAUX_COUNT = 0 THEN
            -- no autortizat, el registre no existeix
            RAISE ERR_CTRL_ACCIO_NO_AUTORITZAT;
        ELSE 
            UPDATE GR_PERSONAEXTERNA
            SET COGNOM1 = IN_NOM
              , COGNOM2 = IN_COG1
              , DOCUMENT = IN_DOC
              , ID_AUDITEXTERNA = IN_IDAUDIT
            WHERE ID = IN_COD; 
        END IF;
        
       
    END IF;   


    VL_ERRCODE := 'OK';
    VL_ERRDESC := NULL;
    RSP := VL_ERRCODE;
      
   
    PRC_LOG(NOMPRC => 'P_PERSONAEXTERNA',
            PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_COD=' || IN_COD || ', ' || 'IN_NOM=' || IN_NOM || ', ' ||
                      'IN_COG1=' || IN_COG1 || ', ' || 'IN_COG2=' || IN_COG2 || ', ' || IN_COG2 || ', '|| 
                      'IN_DOC=' || IN_DOC || ', ' || 'IN_IDAUDIT=' || IN_IDAUDIT,
            RSP    => VL_ERRCODE,
            RSPDESC=> VL_ERRDESC);

  EXCEPTION
    WHEN ERR_CTRL_PARAM THEN
      VL_ERRCODE := 'ERR_001';
      VL_ERRDESC := FNT_GET_ERRDESC(VL_ERRCODE);
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_PERSONAEXTERNA',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_COD=' || IN_COD || ', ' || 'IN_NOM=' || IN_NOM || ', ' ||
                        'IN_COG1=' || IN_COG1 || ', ' || 'IN_COG2=' || IN_COG2 || ', ' || IN_COG2 || ', '|| 
                        'IN_DOC=' || IN_DOC || ', ' || 'IN_IDAUDIT=' || IN_IDAUDIT,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);

    WHEN ERR_CTRL_PARAM_NO_AUTORITZAT THEN
      VL_ERRCODE := 'ERR_002';
      VL_ERRDESC := FNT_GET_ERRDESC(VL_ERRCODE);
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_PERSONAEXTERNA',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_COD=' || IN_COD || ', ' || 'IN_NOM=' || IN_NOM || ', ' ||
                        'IN_COG1=' || IN_COG1 || ', ' || 'IN_COG2=' || IN_COG2 || ', ' || IN_COG2 || ', '|| 
                        'IN_DOC=' || IN_DOC || ', ' || 'IN_IDAUDIT=' || IN_IDAUDIT,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);
    
    WHEN ERR_CTRL_ACCIO_NO_AUTORITZAT THEN
      VL_ERRCODE := 'ERR_003';
      VL_ERRDESC := FNT_GET_ERRDESC(VL_ERRCODE);
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_PERSONAEXTERNA',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_COD=' || IN_COD || ', ' || 'IN_NOM=' || IN_NOM || ', ' ||
                        'IN_COG1=' || IN_COG1 || ', ' || 'IN_COG2=' || IN_COG2 || ', ' || IN_COG2 || ', '|| 
                        'IN_DOC=' || IN_DOC || ', ' || 'IN_IDAUDIT=' || IN_IDAUDIT,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);
    
    
    WHEN OTHERS THEN
      VL_ERRSQLCOD := SQLCODE;
      VL_ERRSQLMSG := SQLERRM;
      VL_ERRCODE := 'ERR_999';
      VL_ERRDESC := VL_ERRSQLCOD || ':' || VL_ERRSQLMSG;
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_PERSONAEXTERNA',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_COD=' || IN_COD || ', ' || 'IN_NOM=' || IN_NOM || ', ' ||
                        'IN_COG1=' || IN_COG1 || ', ' || 'IN_COG2=' || IN_COG2 || ', ' || IN_COG2 || ', '|| 
                        'IN_DOC=' || IN_DOC || ', ' || 'IN_IDAUDIT=' || IN_IDAUDIT,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);
      

  END P_PERSONAEXTERNA;


  /* ******************
     Alta/Baixa/Modificació de dades taula GR_REVISIOEXTERNA
     IN_ABM: A(lta), B(aixa), M(odificació)
     IN_COD: Codi del registre, si nou registre = Null
     IN_INICI: Data d'inici de la revisió
     IN_FI: Data fi de la revisió
     IN_COST: cost de la revisió
     IN_IDAUDITEXTERNA: Identificador de la auditoria externa a la que pertany la revisió
     RSP: Codi de sortida del procediment
  ****************** */
  PROCEDURE P_REVISIOEXTERNA (IN_ABM  VARCHAR2, 
                       IN_COD  NUMBER,
                       IN_INICI DATE,
                       IN_FI DATE,
                       IN_COST NUMBER,
                       IN_IDAUDITEXTERNA NUMBER,
                       RSP OUT VARCHAR2) IS
                           
    VAUX DATE;
    VAUX_COUNT NUMBER;
    
    VL_ERRSQLCOD NUMBER;
    VL_ERRSQLMSG VARCHAR2(255);
    
    ERR_CTRL_PARAM EXCEPTION; 
    ERR_CTRL_PARAM_NO_AUTORITZAT EXCEPTION;
    ERR_CTRL_ACCIO_NO_AUTORITZAT EXCEPTION;
    ERR_CTRL_REGISTRE EXCEPTION; 
    
    VL_ERR_N1 VARCHAR2(50);
    VL_ERRCODE GR_LOG.RSP%TYPE;
    VL_ERRDESC GR_LOG.RSPDESC%TYPE;
    
  BEGIN
  
    IF IN_ABM NOT IN ('A','M') THEN
        RAISE ERR_CTRL_PARAM;
    ELSIF IN_ABM IN ('A') AND IN_COD IS NOT NULL THEN
        RAISE ERR_CTRL_PARAM;
    ELSIF IN_ABM IN ('M') AND IN_COD IS NULL THEN
        RAISE ERR_CTRL_PARAM;
    END IF;
  
  
    SELECT COUNT(1) INTO VAUX_COUNT FROM GR_AUDITEXTERNA WHERE ID = IN_IDAUDITEXTERNA AND DBAIXA IS NULL;
    IF VAUX_COUNT = 0 THEN
      VL_ERR_N1 := 'GR_AUDITEXTERNA';
      RAISE ERR_CTRL_ACCIO_NO_AUTORITZAT;
    END IF;

    -- Alta de registre
    IF IN_ABM = 'A' THEN
      INSERT INTO GR_REVISIOEXTERNA
      (ID, DINICI, DFI, COST, ID_AUDITEXTERNA) VALUES
      (SQ_REVISIOEXTERNA.NEXTVAL, IN_INICI, IN_FI, IN_COST, IN_IDAUDITEXTERNA);
    ELSIF IN_ABM = 'M' THEN
      UPDATE GR_REVISIOEXTERNA
      SET DINICI = IN_INICI
        , DFI = IN_FI
        , COST = IN_COST
        , ID_AUDITEXTERNA = IN_IDAUDITEXTERNA
      WHERE ID = IN_COD;
    END IF;
    

    VL_ERRCODE := 'OK';
    VL_ERRDESC := NULL;
    RSP := VL_ERRCODE;
     
    PRC_LOG(NOMPRC => 'P_REVISIOEXTERNA',
            PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_COD=' || IN_COD || ', ' || 'IN_INICI=' || IN_INICI || ', ' ||
                      'IN_FI=' || IN_FI || ', ' || 'IN_COST=' || IN_COST || ', ' || 'IN_IDAUDITEXTERNA=' || IN_IDAUDITEXTERNA,
            RSP    => VL_ERRCODE,
            RSPDESC=> VL_ERRDESC);
    
    
  EXCEPTION
    WHEN ERR_CTRL_PARAM THEN
      VL_ERRCODE := 'ERR_001';
      VL_ERRDESC := FNT_GET_ERRDESC(VL_ERRCODE);
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_REVISIOEXTERNA',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_COD=' || IN_COD || ', ' || 'IN_INICI=' || IN_INICI || ', ' ||
                        'IN_FI=' || IN_FI || ', ' || 'IN_COST=' || IN_COST || ', ' || 'IN_IDAUDITEXTERNA=' || IN_IDAUDITEXTERNA,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);
              
    WHEN ERR_CTRL_REGISTRE THEN
      VL_ERRCODE := 'ERR_006';
      VL_ERRDESC := REPLACE(FNT_GET_ERRDESC(VL_ERRCODE),'#N1#',VL_ERR_N1);
      RSP := VL_ERRCODE;
      PRC_LOG(NOMPRC => 'P_REVISIOEXTERNA',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_COD=' || IN_COD || ', ' || 'IN_INICI=' || IN_INICI || ', ' ||
                        'IN_FI=' || IN_FI || ', ' || 'IN_COST=' || IN_COST || ', ' || 'IN_IDAUDITEXTERNA=' || IN_IDAUDITEXTERNA,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);
  
    WHEN OTHERS THEN
      VL_ERRSQLCOD := SQLCODE;
      VL_ERRSQLMSG := SQLERRM;
      VL_ERRCODE := 'ERR_999';
      VL_ERRDESC := VL_ERRSQLCOD || ':' || VL_ERRSQLMSG;
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_REVISIOEXTERNA',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_COD=' || IN_COD || ', ' || 'IN_INICI=' || IN_INICI || ', ' ||
                        'IN_FI=' || IN_FI || ', ' || 'IN_COST=' || IN_COST || ', ' || 'IN_IDAUDITEXTERNA=' || IN_IDAUDITEXTERNA,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);
  
  END P_REVISIOEXTERNA;


  /* ******************
     Alta/Baixa/Modificació de dades taula GR_REVISIOEXTERNA
     IN_ABM: A(lta), B(aixa), M(odificació)
     IN_COD: Codi del registre, si nou registre = Null
     IN_INICI: Data d'inici de la revisió
     IN_FI: Data fi de la revisió
     IN_COST: cost de la revisió
     IN_IDAUDITEXTERNA: Identificador de la auditoria externa a la que pertany la revisió
     RSP: Codi de sortida del procediment
  ****************** */
  PROCEDURE P_MOSTREIG (IN_ABM  VARCHAR2,
                        IN_INICI DATE,
                        RSP OUT VARCHAR2) IS

    VL_ERRSQLCOD NUMBER;
    VL_ERRSQLMSG VARCHAR2(255);
    
    ERR_CTRL_PARAM EXCEPTION; 
    
    VL_ERRCODE GR_LOG.RSP%TYPE;
    VL_ERRDESC GR_LOG.RSPDESC%TYPE;
    
    VL_MOSTREIG NUMBER;
    
  BEGIN
  

    IF IN_ABM NOT IN ('A') THEN
        RAISE ERR_CTRL_PARAM;
    END IF;
    
    VL_MOSTREIG:= SQ_MOSTREIG.NEXTVAL;
    
    INSERT INTO GR_MOSTREIG
    (ID, DMOSTREIG) VALUES
    (VL_MOSTREIG, IN_INICI);


    --GR_ACCIO_MOSTREIG
    INSERT INTO GR_ACCIO_MOSTREIG 
    (ID_ACCIO,DESCRIPCIO,DCREACIO,DTANCAMENT,DBAIXA,NOM
    ,ID_ESTATMITIGACIO,ID_PERSONARESPONSABLE,ID_RISC
    ,ID_MOSTREIG)
    (SELECT ID,DESCRIPCIO,DCREACIO,DTANCAMENT,DBAIXA,NOM
           ,ID_ESTATMITIGACIO,ID_PERSONARESPONSABLE,ID_RISC
           ,VL_MOSTREIG 
     FROM GR_ACCIO 
     WHERE DCREACIO <= IN_INICI);


    --GR_CONTROL_MOSTREIG
    INSERT INTO GR_CONTROL_MOSTREIG 
    (ID_CONTROL, DESCRIPCIO, DCREACIO, DBAIXA, ID_CONTROLINTERN, ID_DEPARTAMENTREVISIO,
     ID_DEPARTAMENTEXECUCIO, ID_TIPUSCONTROL, ID_MOSTREIG)
    (SELECT ID, DESCRIPCIO, DCREACIO, DBAIXA, ID_CONTROLINTERN, ID_DEPARTAMENTREVISIO,
            ID_DEPARTAMENTEXECUCIO, ID_TIPUSCONTROL, VL_MOSTREIG 
     FROM GR_CONTROL WHERE DCREACIO <= IN_INICI);
    
   
        
    --GR_REVISIOEXTERNA_MOSTREIG
    INSERT INTO GR_REVISIOEXTERNA_MOSTREIG 
    (ID_REVISIOEXTERNA, DINICI, DFI, COST, ID_AUDITEXTERNA, ID_MOSTREIG)
    (SELECT ID, DINICI, DFI, COST, ID_AUDITEXTERNA ,VL_MOSTREIG 
     FROM GR_REVISIOEXTERNA 
     WHERE DINICI <= IN_INICI);


    --GR_REVISIO_MOSTREIG
    INSERT INTO GR_REVISIO_MOSTREIG 
    (ID_REVISIO,DINICI,DFI,OBJECTIU,DAVIS,ID_DEPARTAMENTIMPLICAT,
     ID_AUDITINTERNA,DBAIXA,ID_REVISIOEXTERNA,ID_MOSTREIG)
    (SELECT ID,DINICI,DFI,OBJECTIU,DAVIS,ID_DEPARTAMENTIMPLICAT,
            ID_AUDITINTERNA,DBAIXA,ID_REVISIOEXTERNA,VL_MOSTREIG 
     FROM GR_REVISIO 
     WHERE DINICI <= IN_INICI);

    --GR_RISC_MOSTREIG
    INSERT INTO GR_RISC_MOSTREIG 
    (ID_RISC, DCREACIO, DFI, DESCRIPCIO, IMPACTE, ID_CATEGORIA, ID_ORIGEN, 
     ID_CRITICITAT, ID_ESTAT, ID_REVISIOEXTERNA, ID_ACCIO, ID_REVISIO, ID_CONTROL, 
     ID_PERSONARESPONSABLE,ID_MOSTREIG)
    (SELECT ID, DCREACIO, DFI, DESCRIPCIO, IMPACTE, ID_CATEGORIA, ID_ORIGEN,
            ID_CRITICITAT, ID_ESTAT, ID_REVISIOEXTERNA, ID_ACCIO, ID_REVISIO, ID_CONTROL, 
            ID_PERSONARESPONSABLE, VL_MOSTREIG 
     FROM GR_RISC 
     WHERE DCREACIO <= IN_INICI);

    
    VL_ERRCODE := 'OK';
    VL_ERRDESC := NULL;
    RSP := VL_ERRCODE;
      
   
    PRC_LOG(NOMPRC => 'P_MOSTREIG',
            PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_INICI=' || IN_INICI,
            RSP    => VL_ERRCODE,
            RSPDESC=> VL_ERRDESC);

    
      
  EXCEPTION
    WHEN ERR_CTRL_PARAM THEN
      VL_ERRCODE := 'ERR_001';
      VL_ERRDESC := FNT_GET_ERRDESC(VL_ERRCODE);
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_MOSTREIG',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_INICI=' || IN_INICI,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);
              

    WHEN OTHERS THEN
      VL_ERRSQLCOD := SQLCODE;
      VL_ERRSQLMSG := SQLERRM;
      VL_ERRCODE := 'ERR_999';
      VL_ERRDESC := VL_ERRSQLCOD || ':' || VL_ERRSQLMSG;
      RSP := VL_ERRCODE;
      
      PRC_LOG(NOMPRC => 'P_MOSTREIG',
              PIN    => 'IN_ABM=' || IN_ABM || ', ' || 'IN_INICI=' || IN_INICI,
              RSP    => VL_ERRCODE,
              RSPDESC=> VL_ERRDESC);
  
  
  END P_MOSTREIG;



END PKG_GR_DADES;
