CONNECT urisc_dwh/urisc_dwh

/* ****************************************************** */
/* ******************** PACKAGE BODY ******************** */
CREATE OR REPLACE PACKAGE BODY PKG_DWH_ETL IS

  /* ******************
     Inici del proces d'extraccio i transformació (ETL)
  ****************** */
  PROCEDURE P_ETL IS
    VL_MARCA_TEMPS DATE DEFAULT TO_DATE('01/01/1999 00:00:00','DD/MM/YYYY HH24:MI:SS');
    VL_ESTAT_EXECUCIO VARCHAR2(10);
    
    VL_ERRSQLCOD NUMBER;
    VL_ERRSQLMSG VARCHAR2(255);

  BEGIN

    
    BEGIN
      SELECT EXECUCIO -- ultima execució correcta
      INTO VL_MARCA_TEMPS
      FROM DWH_ETL
      WHERE ID = (SELECT MAX(ID) FROM DWH_ETL WHERE RSP = 'OK');      
    EXCEPTION
      WHEN OTHERS THEN 
        NULL;
    END;
    
    P_RISC(VL_MARCA_TEMPS);    
    P_RISC_DADES;
    P_CREAR_JSON;
    
    VL_ESTAT_EXECUCIO:= 'OK';
    
    INSERT INTO DWH_ETL
    (ID, EXECUCIO, PROCEDIMENT, MARCA_TEMPS, RSP)
    VALUES
    (SQ_DWH_ETL.NEXTVAL, SYSDATE, 'P_ETL', VL_MARCA_TEMPS, VL_ESTAT_EXECUCIO);
    
    
  EXCEPTION
    WHEN OTHERS THEN
      VL_ERRSQLCOD := SQLCODE;
      VL_ERRSQLMSG := SQLERRM;
      DBMS_OUTPUT.PUT_LINE(VL_ERRSQLCOD || ':' || VL_ERRSQLMSG);
    
      VL_ESTAT_EXECUCIO:= 'ERROR';
      INSERT INTO DWH_ETL
      (ID, EXECUCIO, PROCEDIMENT, MARCA_TEMPS, RSP)
      VALUES
      (SQ_DWH_ETL.NEXTVAL, SYSDATE, 'P_ETL', VL_MARCA_TEMPS, VL_ESTAT_EXECUCIO);
  
  END P_ETL;
  


  /* ******************
  ****************** */
  PROCEDURE P_RISC(p_marcatemps DATE) IS
    CURSOR C_RISC IS 
      select gr_risc.id RISC_ID, gr_risc.dcreacio, gr_risc.dfi, gr_risc.descripcio, gr_risc.impacte, 
             gr_estat.id estat_id, gr_estat.descripcio estat_desc,
             gr_categoria.id categoria_id, gr_categoria.descripcio categoria_desc,
             gr_origen.id origen_id, gr_origen.descripcio origen_desc,
             gr_criticitat.id criticitat_id, gr_criticitat.descripcio criticitat_desc,
             gr_persona.id responsable_id, gr_persona.nom responsable_nom, gr_persona.cognom1 responsable_cog1, gr_persona.cognom2 responsable_cog2, 
             departamentrevisio.id departamentrevisio_id, departamentrevisio.descripcio departamentrevisio_desc, 
             departamentexecucio.id departamentexecucio_id, departamentexecucio.descripcio departamentexecucio_desc,
             -- control
             gr_control.id control_id, gr_control.descripcio control_desc, gr_control.dcreacio control_dcrea,
             gr_tipuscontrol.id tipuscontrol_id, gr_tipuscontrol.descripcio tipuscontrol_desc,
             gr_control.dbaixa control_dbaixa, 
             gr_controlintern.id controlintern_id, gr_controlintern.descripcio controlintern_desc, gr_controlintern.dcreacio controlintern_dcreacio,
             -- revisio
             gr_revisio.id revisio_id, gr_revisio.objectiu revisio_objectiu,
             gr_revisio.dinici revisio_dinici, gr_revisio.dfi revisio_dfi,
             gr_revisio.davis revisio_davis, departamentimplicat.id departamentimplicat_id, departamentimplicat.descripcio departamentimplicat_desc,
             gr_auditinterna.id auditinterna_id, gr_auditinterna.descripcio auditinterna_desc,
             -- revisioexterna
             gr_revisioexterna.id revisioexterna_id, gr_revisioexterna.dinici revisioexterna_dinici, gr_revisioexterna.dfi revisioexterna_dfi, 
             gr_revisioexterna.cost revisioexterna_cost, 
             gr_auditexterna.id auditexterna_id, gr_auditexterna.nomempresa auditexterna_nomempresa,
             -- dades accions
             (select count(1) from urisc.gr_accio where gr_accio.id_risc = gr_risc.id) total_accions
      from urisc.gr_risc, urisc.gr_estat, urisc.gr_categoria, urisc.gr_origen,
           urisc.gr_criticitat, urisc.gr_persona, 
           urisc.gr_departament departamentrevisio, urisc.gr_departament departamentexecucio,
           urisc.gr_control, urisc.gr_controlintern, urisc.gr_tipuscontrol,
           urisc.gr_revisio, urisc.gr_departament departamentimplicat, urisc.gr_auditinterna,
           urisc.gr_revisioexterna, urisc.gr_auditexterna
      where gr_estat.id (+) = gr_risc.id_estat
      and gr_categoria.id (+) = gr_risc.id_categoria
      and gr_origen.id (+) = gr_risc.id_origen
      and gr_criticitat.id (+) = gr_risc.id_criticitat
      and gr_persona.id (+) = gr_risc.id_personaresponsable
      and gr_control.id (+) = gr_risc.id_control
      and gr_controlintern.id (+) = gr_control.id_controlintern
      and gr_tipuscontrol.id (+) = gr_control.id_tipuscontrol
      and departamentrevisio.id (+) = gr_control.id_departamentrevisio
      and departamentexecucio.id (+) = gr_control.id_departamentexecucio
      and gr_revisio.id (+) = gr_risc.id_revisio
      and departamentimplicat.id (+) = gr_revisio.id_departamentimplicat
      and gr_auditinterna.id (+) = gr_revisio.id_auditinterna
      and gr_revisioexterna.id (+) = gr_risc.id_revisioexterna
      and gr_auditexterna.id (+) = gr_revisioexterna.id_auditexterna
      -- control marca de temps
      AND (
        urisc.gr_risc.ddwh > p_marcatemps
        OR urisc.gr_control.ddwh > p_marcatemps
        OR urisc.gr_controlintern.ddwh > p_marcatemps
        OR urisc.gr_revisio.ddwh > p_marcatemps
        OR urisc.gr_auditinterna.ddwh > p_marcatemps
        OR urisc.gr_revisioexterna.ddwh > p_marcatemps
        OR urisc.gr_auditexterna.ddwh > p_marcatemps
      );

      VL_I NUMBER := 0;
  BEGIN
    
    FOR REG_RISC IN C_RISC LOOP
        VL_I := VL_I + 1;
    
    
        UPDATE DWH_RISC
        SET TIPUS = 'H'
        WHERE RISC_ID = REG_RISC.RISC_ID
        AND TIPUS = 'A';
        
        
        INSERT INTO DWH_RISC
        (ID, TIPUS, DVALUE, 
         RISC_ID, DCREACIO, DFI, DESCRIPCIO, IMPACTE, ESTAT_ID, ESTAT_DESC,
         CATEGORIA_ID, CATEGORIA_DESC, ORIGEN_ID, ORIGEN_DESC, CRITICITAT_ID, CRITICITAT_DESC,
         RESPONSABLE_ID, RESPONSABLE_NOM, RESPONSABLE_COG1, RESPONSABLE_COG2, DEPARTAMENTREVISIO_ID,
         DEPARTAMENTREVISIO_DESC, DEPARTAMENTEXECUCIO_ID, DEPARTAMENTEXECUCIO_DESC, CONTROL_ID,
         CONTROL_DESC, CONTROL_DCREA, TIPUSCONTROL_ID, TIPUSCONTROL_DESC, CONTROL_DBAIXA, CONTROLINTERN_ID,
         CONTROLINTERN_DESC, CONTROLINTERN_DCREACIO, REVISIO_ID, REVISIO_OBJECTIU, REVISIO_DINICI, REVISIO_DFI,
         REVISIO_DAVIS, DEPARTAMENTIMPLICAT_ID, DEPARTAMENTIMPLICAT_DESC, AUDITINTERNA_ID, AUDITINTERNA_DESC, 
         REVISIOEXTERNA_ID, REVISIOEXTERNA_DINICI, REVISIOEXTERNA_DFI, REVISIOEXTERNA_COST, AUDITEXTERNA_ID,
         AUDITEXTERNA_NOMEMPRESA, TOTAL_ACCIONS)
        VALUES
        (SQ_DWH_RISC.NEXTVAL, 'A', SYSDATE, 
         REG_RISC.RISC_ID, REG_RISC.DCREACIO, REG_RISC.DFI, REG_RISC.DESCRIPCIO, REG_RISC.IMPACTE, REG_RISC.ESTAT_ID, REG_RISC.ESTAT_DESC,
         REG_RISC.CATEGORIA_ID, REG_RISC.CATEGORIA_DESC, REG_RISC.ORIGEN_ID, REG_RISC.ORIGEN_DESC, REG_RISC.CRITICITAT_ID, REG_RISC.CRITICITAT_DESC,
         REG_RISC.RESPONSABLE_ID, REG_RISC.RESPONSABLE_NOM, REG_RISC.RESPONSABLE_COG1, REG_RISC.RESPONSABLE_COG2, REG_RISC.DEPARTAMENTREVISIO_ID,
         REG_RISC.DEPARTAMENTREVISIO_DESC, REG_RISC.DEPARTAMENTEXECUCIO_ID, REG_RISC.DEPARTAMENTEXECUCIO_DESC, REG_RISC.CONTROL_ID,
         REG_RISC.CONTROL_DESC, REG_RISC.CONTROL_DCREA, REG_RISC.TIPUSCONTROL_ID, REG_RISC.TIPUSCONTROL_DESC, REG_RISC.CONTROL_DBAIXA, REG_RISC.CONTROLINTERN_ID,
         REG_RISC.CONTROLINTERN_DESC, REG_RISC.CONTROLINTERN_DCREACIO, REG_RISC.REVISIO_ID, REG_RISC.REVISIO_OBJECTIU, REG_RISC.REVISIO_DINICI, REG_RISC.REVISIO_DFI,
         REG_RISC.REVISIO_DAVIS, REG_RISC.DEPARTAMENTIMPLICAT_ID, REG_RISC.DEPARTAMENTIMPLICAT_DESC, REG_RISC.AUDITINTERNA_ID, REG_RISC.AUDITINTERNA_DESC, 
         REG_RISC.REVISIOEXTERNA_ID, REG_RISC.REVISIOEXTERNA_DINICI, REG_RISC.REVISIOEXTERNA_DFI, REG_RISC.REVISIOEXTERNA_COST, REG_RISC.AUDITEXTERNA_ID,
         REG_RISC.AUDITEXTERNA_NOMEMPRESA, REG_RISC.TOTAL_ACCIONS);
        
    
    END LOOP;
    
    DBMS_OUTPUT.PUT_LINE('Registres Tractats='||VL_I);
  
  END P_RISC;


  /* ******************
  ****************** */
  PROCEDURE P_RISC_DADES IS

    CURSOR C_RESUM_RISCS_X_IMPACTE (PI_IMPACTE NUMBER) IS
        SELECT ESTAT_DESC, COUNT(1), (select count(1) from DWH_RISC WHERE TIPUS = 'A') AS TOTAL, 
               COUNT(1) / (select count(1) from DWH_RISC WHERE TIPUS = 'A') * 100 PRCT
        FROM DWH_RISC
        WHERE IMPACTE < PI_IMPACTE
        AND TIPUS = 'A'
        GROUP BY ESTAT_DESC;

  
    CURSOR C_DADES_DEPARTAMENTS_REV_EXT (PI_DEPT NUMBER) IS
        select gr_departament.id, gr_departament.descripcio, count(1) TOTAL--, gr_risc.id
        from urisc.gr_risc, urisc.gr_departament_categoria, urisc.gr_departament, urisc.gr_categoria
        where gr_risc.id_revisioexterna is not null
        and gr_risc.id_categoria = gr_categoria.id
        and gr_departament.id = gr_departament_categoria.id_departament
        and gr_categoria.id = gr_departament_categoria.id_categoria
        and gr_departament_categoria.dbaixa is null
        and gr_departament.id = PI_DEPT
        group by gr_departament.id, gr_departament.descripcio
        order by 3 desc;
        
    CURSOR C_ACCIONS_X_ANY IS
        select to_number(to_char(dcreacio,'YYYY')) anyo, id_estatmitigacio, gr_estatmitigacio.descripcio, count(1) total_en_curs
        from urisc.gr_accio, urisc.gr_estatmitigacio
        where id_estatmitigacio = gr_estatmitigacio.id
        --and gr_estatmitigacio.descripcio = 'En Curs'
        group by to_number(to_char(dcreacio,'YYYY')), id_estatmitigacio, gr_estatmitigacio.descripcio
        order by 1;
        
    CURSOR C_ANY_IMPACTE_ORIGEN IS
        select to_char(dcreacio,'YYYY') ANYO, impacte, 
               sum(decode(origen_id,1,1,0)) audit_interna, sum(decode(origen_id,2,1,0)) audit_externa, 
               sum(decode(origen_id,3,1,0)) autoevaluacio
        from dwh_risc
        where tipus = 'A'
        group by to_char(dcreacio,'YYYY'), impacte
        order by 1;
        
    CURSOR C_PERSONA_ACCIO IS
        select gr_persona.id persona_id, gr_persona.cognom1 || decode(gr_persona.cognom2,null,null,' ' || gr_persona.cognom2) || ', ' || gr_persona.nom persona_nom, 
               gr_estatmitigacio.id, gr_estatmitigacio.descripcio, count(1) total_accions
        from urisc.gr_accio, urisc.gr_persona, urisc.gr_estatmitigacio
        where gr_accio.id_personaresponsable = gr_persona.id
        and gr_estatmitigacio.id = gr_accio.id_estatmitigacio
        --and sysdate between gr_accio.dcreacio and decode(gr_accio.dbaixa,null,sysdate+1)
        group by gr_persona.id, gr_persona.cognom1 || decode(gr_persona.cognom2,null,null,' ' || gr_persona.cognom2) || ', ' || gr_persona.nom,
                 gr_estatmitigacio.id, gr_estatmitigacio.descripcio;

    CURSOR C_MOSTREIG IS
        SELECT TO_CHAR(DMOSTREIG,'YYYY') ANYO, COUNT(1) TOTAL
        FROM URISC.GR_MOSTREIG
        GROUP BY TO_CHAR(DMOSTREIG,'YYYY');


    CURSOR C_RISC_CAT_ORGI_X_ANY IS
        SELECT TO_CHAR(DCREACIO, 'YYYY') ANYO, CATEGORIA_ID, categoria_desc, ORIGEN_ID, origen_desc, COUNT(1) TOTAL
        FROM DWH_RISC
        WHERE TIPUS = 'A' 
        GROUP BY TO_CHAR(DCREACIO, 'YYYY'), CATEGORIA_ID, categoria_desc, ORIGEN_ID, origen_desc
        ORDER BY TO_CHAR(DCREACIO, 'YYYY'), CATEGORIA_ID, categoria_desc, ORIGEN_ID, origen_desc;

    CURSOR C_AUDITEXTERNA_X_ANY IS
        select to_char(dinici,'YYYY') anyo, COUNT(1) TOTAL, avg(cost) COST_MITJA
        from urisc.gr_revisioexterna
        GROUP BY to_char(dinici,'YYYY');

    CURSOR C_ACCIO_ESTAT_X_ANY IS
        select to_char(dcreacio,'YYYY') anyo, id_estatmitigacio, gr_estatmitigacio.descripcio, count(1) total
        from urisc.gr_accio, URISC.gr_estatmitigacio
        where id_estatmitigacio = gr_estatmitigacio.id
        group by to_char(dcreacio,'YYYY'), id_estatmitigacio, gr_estatmitigacio.descripcio
        order by total desc;


    
    VL_REV_EXT NUMBER;
  
  BEGIN
    
    DELETE FROM DWH_RISC_IMPACTE_4;
    FOR I_REG IN C_RESUM_RISCS_X_IMPACTE(4) LOOP
    
        INSERT INTO DWH_RISC_IMPACTE_4
        (ESTAT, PRCT) VALUES (I_REG.ESTAT_DESC, I_REG.PRCT);
        
    END LOOP;



    DELETE FROM DWH_RISC_IMPACTE_1_X_ANY;
    FOR I_REG IN (  select to_number(to_char(dcreacio,'YYYY')) ANYO, count(1) TOTAL
                    from DWH_RISC
                    where tipus = 'A'
                    and impacte = 1
                    group by to_char(dcreacio,'YYYY')
                    order by 1) LOOP
        
        INSERT INTO DWH_RISC_IMPACTE_1_X_ANY
        (ANYO, TOTAL) VALUES (I_REG.ANYO, I_REG.TOTAL);
        
    END LOOP;
    
    DELETE FROM DWH_DEPARTAMENT_DADES;
    FOR I_DPT IN (SELECT * FROM urisc.gr_departament) LOOP
    
        VL_REV_EXT := 0;
        FOR I_REG IN C_DADES_DEPARTAMENTS_REV_EXT(I_DPT.ID) LOOP
            
            VL_REV_EXT:= VL_REV_EXT + I_REG.TOTAL;
        
        END LOOP;
        
        INSERT INTO DWH_DEPARTAMENT_DADES
        (ID, DESCRIPCIO, NUM_REV_EXT)
        VALUES 
        (I_DPT.ID, I_DPT.DESCRIPCIO, VL_REV_EXT);
    
    END LOOP;
    
    DELETE FROM DWH_ACCIO_X_ANY;
    FOR I_REG IN C_ACCIONS_X_ANY LOOP
    
        INSERT INTO DWH_ACCIO_X_ANY
        (ANYO, TIPUS_ID, TIPUS_DESC, TOTAL)
        VALUES 
        (I_REG.ANYO, I_REG.ID_ESTATMITIGACIO, I_REG.DESCRIPCIO, I_REG.TOTAL_EN_CURS);

    END LOOP;
    
    DELETE FROM DWH_ANY_IMPACTE_ORIGEN;
    FOR I_REG IN C_ANY_IMPACTE_ORIGEN LOOP
    
        INSERT INTO DWH_ANY_IMPACTE_ORIGEN
        (ANYO, IMPACTE, TOTAL_AUDIT_INTERNA, TOTAL_AUDIT_EXTERNA, TOTAL_AUTOEVALUACIO)
        VALUES
        (I_REG.ANYO, I_REG.IMPACTE, I_REG.audit_interna, I_REG.audit_externa, I_REG.autoevaluacio);
    
    END LOOP;
    
    DELETE FROM DWH_PERSONA_ACCIO;
    FOR I_REG IN C_PERSONA_ACCIO LOOP
        
        INSERT INTO DWH_PERSONA_ACCIO
        (PERSONA_ID, PERSONA_NOM, ACCIO_ID, ACCIO_ESTAT, ACCIO_TOTALS)
        VALUES
        (I_REG.PERSONA_ID, I_REG.PERSONA_NOM, I_REG.ID, I_REG.DESCRIPCIO, I_REG.total_accions);
    
    END LOOP;
    
    DELETE FROM DWH_MOSTREIG;
    FOR I_REG IN C_MOSTREIG LOOP
        INSERT INTO DWH_MOSTREIG 
        (ANYO, TOTAL)
        VALUES
        (I_REG.ANYO, I_REG.TOTAL);
    END LOOP;
    
    DELETE FROM DWH_RISC_CAT_ORIG_X_ANY;
    FOR I_REG IN C_RISC_CAT_ORGI_X_ANY LOOP
        INSERT INTO DWH_RISC_CAT_ORIG_X_ANY
        (ANYO, CATEGORIA_ID, categoria_desc, ORIGEN_ID, origen_desc, TOTAL)
        VALUES 
        (I_REG.ANYO, I_REG.CATEGORIA_ID, I_REG.categoria_desc, I_REG.ORIGEN_ID, I_REG.origen_desc, I_REG.TOTAL);
    END LOOP;
    

    DELETE FROM DWH_AUDITEXTERNA_X_ANY;
    FOR I_REG IN C_AUDITEXTERNA_X_ANY LOOP
        INSERT INTO DWH_AUDITEXTERNA_X_ANY
        (ANYO, TOTAL, COST_MITJA)
        VALUES 
        (I_REG.ANYO, I_REG.TOTAL, I_REG.COST_MITJA);
    END LOOP;

    DELETE FROM DWH_ACCIO_ESTAT_X_ANY;
    FOR I_REG IN C_ACCIO_ESTAT_X_ANY LOOP
        INSERT INTO DWH_ACCIO_ESTAT_X_ANY
        (ANYO, ESTATMITIGACIO_ID, ESTATMITIGACIO_DESC, TOTAL)
        VALUES
        (I_REG.ANYO, I_REG.id_estatmitigacio, I_REG.descripcio, I_REG.total);
    END LOOP;
    
  
  END P_RISC_DADES;


  /* ******************

  ****************** */
  PROCEDURE P_CREAR_JSON IS
  
      js_object JSON_OBJECT_T;
      js_array JSON_ARRAY_T;
    
      js_object_final JSON_OBJECT_T;
      js_array_final JSON_ARRAY_T;
    
      cursor c_risc is
        SELECT json_object('risc_id'    VALUE dwh_risc.risc_id, 
                           'descripcio' value dwh_risc.descripcio) as lin
        FROM dwh_risc
        WHERE TIPUS='A';
    
      cursor c_risc_cat_orig_x_any is
        SELECT json_object('anyo'    VALUE anyo, 
                           'categoria_desc' value categoria_desc) as lin
        FROM dwh_risc_cat_orig_x_any;
  
      VL_TXT CLOB;
  
  BEGIN
  
      js_array := new JSON_ARRAY_T;
      for i in c_risc loop
        js_object := JSON_OBJECT_T.parse(i.lin);
        js_array.append(js_object);
      end loop;
      
      js_object_final := JSON_OBJECT_T.parse('{"riscs":'|| js_array.to_string ||'}');
      js_array_final := new JSON_ARRAY_T;
      js_array_final.append(js_object_final);
    
      -- ***************************
      js_array := new JSON_ARRAY_T;
      for i in c_risc_cat_orig_x_any loop
        js_object := JSON_OBJECT_T.parse(i.lin);
        js_array.append(js_object);
      end loop;
      
      js_object_final := JSON_OBJECT_T.parse('{"risc_cat_orig_x_any":'|| js_array.to_string ||'}');
      --js_array_final := new JSON_ARRAY_T;
      js_array_final.append(js_object_final);
      
    
      --dbms_output.put_line(js_array_final.to_string);
      VL_TXT := js_array_final.to_string;
      
      DELETE FROM DWH_JSON;
      INSERT INTO DWH_JSON
      (CONTINGUT_TXT, CONTINGUT_JSON)
      VALUES 
      (VL_TXT, VL_TXT);
      
      
      
  
  END P_CREAR_JSON;



END PKG_DWH_ETL;
